@@ -37,6 +37,21 @@ anjuta_shell_placement_get_type (void)
   return etype;
 }
 
+/* enumerations from "anjuta-convert.h" */
+GType
+gedit_convert_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { ANJUTA_CONVERT_ERROR_AUTO_DETECTION_FAILED, "ANJUTA_CONVERT_ERROR_AUTO_DETECTION_FAILED", "failed" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("GeditConvertError"), values);
+  }
+  return etype;
+}
+
 /* enumerations from "anjuta-plugin-description.h" */
 GType
 anjuta_plugin_description_parse_error_get_type (void)
@@ -164,9 +179,11 @@ anjuta_session_phase_get_type (void)
   static GType etype = 0;
   if (etype == 0) {
     static const GEnumValue values[] = {
+      { ANJUTA_SESSION_PHASE_START, "ANJUTA_SESSION_PHASE_START", "start" },
       { ANJUTA_SESSION_PHASE_FIRST, "ANJUTA_SESSION_PHASE_FIRST", "first" },
       { ANJUTA_SESSION_PHASE_NORMAL, "ANJUTA_SESSION_PHASE_NORMAL", "normal" },
       { ANJUTA_SESSION_PHASE_LAST, "ANJUTA_SESSION_PHASE_LAST", "last" },
+      { ANJUTA_SESSION_PHASE_END, "ANJUTA_SESSION_PHASE_END", "end" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static (g_intern_static_string ("AnjutaSessionPhase"), values);
