@@ -36,13 +36,14 @@
 #include <fcntl.h>
 #include <signal.h>
 
-#if !defined(__sun) && !defined(__NetBSD__)
-#  ifndef FREEBSD
-#    include <pty.h>
-#  else
-#    include <libutil.h>
-#  endif
+#if defined(__FreeBSD__)
+#  include <libutil.h>
+#elif defined(__OpenBSD__) || defined(__NetBSD__)
+#  include <util.h>
+#elif !defined(__sun)
+#  include <pty.h>
 #endif
+
 #include "anjuta-utils-priv.h"
 
 #include <assert.h>
@@ -65,11 +66,15 @@
 #define FILE_BUFFER_SIZE 1024
 #define FILE_INPUT_BUFFER_SIZE		1048576
 #ifndef __MAX_BAUD
-#ifdef __CYGWIN__
-#define __MAX_BAUD B256000
-#else
-#define __MAX_BAUD B460800
-#endif
+#  if defined(B460800)
+#    define __MAX_BAUX B460800
+#  elif defined(B307200)
+#    define __MAX_BAUX B307200
+#  elif defined(B256000)
+#    define __MAX_BAUX B256000
+#  else
+#    define __MAX_BAUX B230400
+#  endif
 #endif
 
 /*
@@ -110,9 +115,6 @@ struct _AnjutaLauncherPriv
 	/* Terminal echo */
 	gboolean terminal_echo_on;
 	
-	/* Check for passwords in input */
-	gboolean disable_password_check;
-	
 	/* The child */
 	pid_t child_pid;
 	guint source;
@@ -140,6 +142,13 @@ struct _AnjutaLauncherPriv
 	
 	/* Callback data */
 	gpointer callback_data;
+	
+	/* Encondig */
+	gboolean custom_encoding;
+	gchar* encoding;
+	
+	/* Env */
+	GHashTable* env;
 };
 
 enum
@@ -185,12 +194,12 @@ anjuta_launcher_initialize (AnjutaLauncher *obj)
 	obj->priv->pty_output_buffer = NULL;
 	
 	obj->priv->terminal_echo_on = TRUE;
-	obj->priv->disable_password_check = FALSE;
 	
 	/* The child */
 	obj->priv->child_pid = 0;
 	obj->priv->child_status = -1;
 	obj->priv->child_has_terminated = TRUE;
+	obj->priv->source = 0;
 	
 	/* Synchronization in progress */
 	obj->priv->in_cleanup = FALSE;
@@ -208,6 +217,14 @@ anjuta_launcher_initialize (AnjutaLauncher *obj)
 	/* Output callback */
 	obj->priv->output_callback = NULL;
 	obj->priv->callback_data = NULL;
+	
+	/* Encoding */
+	obj->priv->custom_encoding = FALSE;
+	obj->priv->encoding = NULL;
+	
+	/* Env */
+	obj->priv->env = g_hash_table_new_full (g_str_hash, g_str_equal,
+											g_free, g_free);
 }
 
 GType
@@ -242,16 +259,13 @@ anjuta_launcher_dispose (GObject *obj)
 	AnjutaLauncher *launcher = ANJUTA_LAUNCHER (obj);
 	if (anjuta_launcher_is_busy (launcher))
 	{
-		pid_t child_pid_save = launcher->priv->child_pid;
-		guint child_source = launcher->priv->source;
-		g_source_remove (child_source);
+		g_source_remove (launcher->priv->source);
+		launcher->priv->source = 0;
+		
 		anjuta_launcher_execution_done_cleanup (launcher, FALSE);
 		
-		/* We can not call anjuta_launcher_reset (launcher) to kill the
-		 * running child because launcher has been initialized in cleanup
-		 */
-		kill (child_pid_save, SIGTERM);
 		launcher->priv->busy = FALSE;
+		
 	}
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
 }
@@ -260,6 +274,11 @@ static void
 anjuta_launcher_finalize (GObject *obj)
 {
 	AnjutaLauncher *launcher = ANJUTA_LAUNCHER (obj);	
+	if (launcher->priv->custom_encoding && launcher->priv->encoding)
+		g_free (launcher->priv->encoding);
+	
+	g_hash_table_destroy (launcher->priv->env);
+	
 	g_free (launcher->priv);
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
 }
@@ -578,9 +597,6 @@ anjuta_launcher_check_password_real (AnjutaLauncher *launcher,
 	if (anjuta_launcher_is_busy (launcher) == FALSE) 
 		return;
 	
-	if (launcher->priv->disable_password_check)
-		return;
-	
 	if (last_line) {
 
 		/* DEBUG_PRINT ("(In password) Last line = %s", last_line); */
@@ -746,20 +762,22 @@ anjuta_launcher_scan_output (GIOChannel *channel, GIOCondition condition,
 		do
 		{
 			g_io_channel_read_chars (channel, buffer, FILE_BUFFER_SIZE-1, &n, &err);
-			if (n > 0 && !err) /* There is output */
+			if (n > 0) /* There is output */
 			{
-				gchar *utf8_chars;
+				gchar *utf8_chars = NULL;
 				buffer[n] = '\0';
-				utf8_chars = anjuta_util_convert_to_utf8 (buffer);
+				if (!launcher->priv->custom_encoding)
+					utf8_chars = anjuta_util_convert_to_utf8 (buffer);
+				else
+					utf8_chars = g_strdup(buffer);
 				anjuta_launcher_buffered_output (launcher,
 												 ANJUTA_LAUNCHER_OUTPUT_STDOUT,
 												 utf8_chars);
 				g_free (utf8_chars);
 			}
 			/* Ignore illegal characters */
-			else if (err && err->domain == G_CONVERT_ERROR)
+			if (err && err->domain == G_CONVERT_ERROR)
 			{
-				DEBUG_PRINT ("stdout: %s", err->message);
 				g_error_free (err);
 				err = NULL;
 			}
@@ -767,7 +785,6 @@ anjuta_launcher_scan_output (GIOChannel *channel, GIOCondition condition,
 			/* if not related to non blocking read or interrupted syscall */
 			else if (err && errno != EAGAIN && errno != EINTR)
 			{
-				DEBUG_PRINT ("stdout: %s", err->message);
 				launcher->priv->stdout_is_done = TRUE;
 				anjuta_launcher_synchronize (launcher);
 				ret = FALSE;
@@ -802,7 +819,7 @@ anjuta_launcher_scan_error (GIOChannel *channel, GIOCondition condition,
 		do
 		{
 			g_io_channel_read_chars (channel, buffer, FILE_BUFFER_SIZE-1, &n, &err);
-			if (n > 0 && !err) /* There is stderr output */
+			if (n > 0) /* There is stderr output */
 			{
 				gchar *utf8_chars;
 				buffer[n] = '\0';
@@ -813,9 +830,8 @@ anjuta_launcher_scan_error (GIOChannel *channel, GIOCondition condition,
 				g_free (utf8_chars);
 			}
 			/* Ignore illegal characters */
-			else if (err && err->domain == G_CONVERT_ERROR)
+			if (err && err->domain == G_CONVERT_ERROR)
 			{
-				DEBUG_PRINT ("stderr: %s", err->message);
 				g_error_free (err);
 				err = NULL;
 			}
@@ -823,7 +839,6 @@ anjuta_launcher_scan_error (GIOChannel *channel, GIOCondition condition,
 			/* if not related to non blocking read or interrupted syscall */
 			else if (err && errno != EAGAIN && errno != EINTR)
 			{
-				DEBUG_PRINT ("stderr: %s", err->message);
 				
 				launcher->priv->stderr_is_done = TRUE;
 				anjuta_launcher_synchronize (launcher);
@@ -859,7 +874,7 @@ anjuta_launcher_scan_pty (GIOChannel *channel, GIOCondition condition,
 		do
 		{
 			g_io_channel_read_chars (channel, buffer, FILE_BUFFER_SIZE-1, &n, &err);
-			if (n > 0 && !err) /* There is stderr output */
+			if (n > 0) /* There is stderr output */
 			{
 				gchar *utf8_chars;
 				gchar *old_str = launcher->priv->pty_output_buffer;
@@ -876,9 +891,8 @@ anjuta_launcher_scan_pty (GIOChannel *channel, GIOCondition condition,
 				g_free (utf8_chars);
 			}
 			/* Ignore illegal characters */
-			else if (err && err->domain == G_CONVERT_ERROR)
+			if (err && err->domain == G_CONVERT_ERROR)
 			{
-				DEBUG_PRINT ("pty: %s", err->message);
 				g_error_free (err);
 				err = NULL;
 			}
@@ -886,7 +900,6 @@ anjuta_launcher_scan_pty (GIOChannel *channel, GIOCondition condition,
 			/* if not related to non blocking read or interrupted syscall */
 			else if (err && errno != EAGAIN && errno != EINTR)
 			{
-				DEBUG_PRINT ("pty: %s", err->message);
 				ret = FALSE;
 			}
 		/* Read next chars if buffer was too small
@@ -1033,24 +1046,16 @@ anjuta_launcher_child_terminated (GPid pid, gint status, gpointer data)
 	anjuta_launcher_synchronize (launcher);
 }
 
-/**
- * anjuta_launcher_set_encoding:
- * @launcher: a #AnjutaLancher object.
- * @charset: Character set to use for Input/Output with the process.
- * 
- * Sets the character set to use for Input/Output with the process.
- *
- * Return value: TRUE if successful, otherwise FALSE.
- */
-gboolean
-anjuta_launcher_set_encoding (AnjutaLauncher *launcher, const gchar *charset)
+
+static gboolean
+anjuta_launcher_set_encoding_real (AnjutaLauncher *launcher, const gchar *charset)
 {
 	GIOStatus s;
 	gboolean r = TRUE;
 
 	g_return_val_if_fail (launcher != NULL, FALSE);
 	// charset can be NULL
-
+		
 	s = g_io_channel_set_encoding (launcher->priv->stderr_channel, charset, NULL);
 	if (s != G_IO_STATUS_NORMAL) r = FALSE;
 	s = g_io_channel_set_encoding (launcher->priv->stdout_channel, charset, NULL);
@@ -1065,6 +1070,57 @@ anjuta_launcher_set_encoding (AnjutaLauncher *launcher, const gchar *charset)
 	return r;
 }
 
+
+/**
+ * anjuta_launcher_set_encoding:
+ * @launcher: a #AnjutaLancher object.
+ * @charset: Character set to use for Input/Output with the process.
+ * 
+ * Sets the character set to use for Input/Output with the process.
+ *
+ */
+void
+anjuta_launcher_set_encoding (AnjutaLauncher *launcher, const gchar *charset)
+{
+	if (launcher->priv->custom_encoding)
+		g_free (launcher->priv->encoding);
+	
+	launcher->priv->custom_encoding = TRUE;
+	if (charset)
+	  launcher->priv->encoding = g_strdup(charset);
+	else
+	  launcher->priv->encoding = NULL;		
+}
+
+/**
+ * anjuta_launcher_set_env:
+ * @launcher: a #AnjutaLancher object.
+ * @name: Name of the environment variable to set
+ * @value: Value of the environment variable to set
+ * 
+ * Set an environment variable for the forked process
+ *
+ */ 
+void
+anjuta_launcher_set_env (AnjutaLauncher *launcher,
+						 const gchar *name,
+						 const gchar *value)
+{
+	g_return_if_fail (launcher && ANJUTA_IS_LAUNCHER(launcher));
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (value != NULL);
+	
+	g_hash_table_insert (launcher->priv->env,
+						 g_strdup(name),
+						 g_strdup(value));
+}
+
+static void
+anjuta_launcher_fork_setenv (const gchar* name, const gchar* value)
+{
+	setenv (name, value, TRUE);
+}
+
 static pid_t
 anjuta_launcher_fork (AnjutaLauncher *launcher, gchar *const args[])
 {
@@ -1073,7 +1129,6 @@ anjuta_launcher_fork (AnjutaLauncher *launcher, gchar *const args[])
 	int stdout_pipe[2], stderr_pipe[2];
 	pid_t child_pid;
 	struct termios termios_flags;
-	const gchar *charset;
 	
 	working_dir = g_get_current_dir ();
 	
@@ -1105,6 +1160,11 @@ anjuta_launcher_fork (AnjutaLauncher *launcher, gchar *const args[])
 		if ((md = fcntl (stderr_pipe[1], F_GETFL)) != -1)
 			fcntl (stderr_pipe[1], F_SETFL, O_SYNC | md);
 		
+		/* Set up environment */
+		g_hash_table_foreach (launcher->priv->env,
+							  (GHFunc) anjuta_launcher_fork_setenv,
+							  NULL);
+		
 		execvp (args[0], args);
 		g_warning (_("Cannot execute command: \"%s\""), args[0]);
 		perror(_("execvp failed"));
@@ -1143,18 +1203,18 @@ anjuta_launcher_fork (AnjutaLauncher *launcher, gchar *const args[])
 	launcher->priv->pty_channel = g_io_channel_unix_new (pty_master_fd);
 
 	g_io_channel_set_buffer_size (launcher->priv->pty_channel, FILE_INPUT_BUFFER_SIZE);
-	
-		
-	g_get_charset (&charset);
-	anjuta_launcher_set_encoding (launcher, charset);
 
+	if (!launcher->priv->custom_encoding)
+	  g_get_charset ((const gchar**)&launcher->priv->encoding);
+	anjuta_launcher_set_encoding_real (launcher, launcher->priv->encoding);
+	
 	tcgetattr(pty_master_fd, &termios_flags);
 	termios_flags.c_iflag &= ~(IGNPAR | INPCK | INLCR | IGNCR | ICRNL | IXON |
 					IXOFF | ISTRIP);
 	termios_flags.c_iflag |= IGNBRK | BRKINT | IMAXBEL | IXANY;
 	termios_flags.c_oflag &= ~OPOST;
 //	termios_flags.c_oflag |= 0;
-	termios_flags.c_cflag &= ~(CSTOPB | CREAD | PARENB | HUPCL);
+	termios_flags.c_cflag &= ~(CSTOPB | PARENB | HUPCL);
 	termios_flags.c_cflag |= CS8 | CLOCAL;
 
 	if (!launcher->priv->terminal_echo_on)
@@ -1349,22 +1409,6 @@ anjuta_launcher_set_terminate_on_exit (AnjutaLauncher *launcher,
 	return past_value;
 }
 
-/**
- * anjuta_launcher_disable_password_check:
- * @launcher: IAnjutaLauncher object
- * @disable: TRUE disables password check
- * 
- * Do not check for password prompts in input
- *
- * Return value: a new instance of #AnjutaLancher class.
- */
-
-void anjuta_launcher_disable_password_check (AnjutaLauncher* launcher,
-                                                 gboolean disable)
-{
-	launcher->priv->disable_password_check = disable;  
-}
-
 /**
  * anjuta_launcher_new:
  * 
