@@ -39,6 +39,7 @@
 #include <libanjuta/anjuta-plugin.h>
 #include <libanjuta/anjuta-c-plugin-factory.h>
 #include <libanjuta/interfaces/ianjuta-plugin-factory.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
 
 
 enum
@@ -791,6 +792,7 @@ plugin_toggled (GtkCellRendererToggle *cell, char *path_str, gpointer data)
 	AnjutaPluginHandle *plugin;
 	gboolean enabled;
 	GHashTable *activated_plugins;
+	AnjutaPlugin* plugin_object;
 	
 	path = gtk_tree_path_new_from_string (path_str);
 	
@@ -803,9 +805,34 @@ plugin_toggled (GtkCellRendererToggle *cell, char *path_str, gpointer data)
 			    COL_PLUGIN, &plugin,
 			    -1);
 	
+	if (enabled)
+	{
+		plugin_object = g_hash_table_lookup (priv->activated_plugins, plugin);
+		if (plugin_object &&
+			IANJUTA_IS_PREFERENCES(plugin_object))
+		{
+			ianjuta_preferences_unmerge (IANJUTA_PREFERENCES (plugin_object),
+									   anjuta_shell_get_preferences (ANJUTA_SHELL (priv->shell), NULL),
+									   NULL);
+		}
+	}
 	enabled = !enabled;
 
 	activated_plugins = plugin_set_update (plugin_manager, plugin, enabled);
+	plugin_object = g_hash_table_lookup (priv->activated_plugins, plugin);
+	
+	/* Make sure that it appears in the preferences. This method
+		can only be called when the preferences dialog is active so
+		it should be save
+	*/
+	if (plugin_object &&
+		IANJUTA_IS_PREFERENCES(plugin_object))
+	{
+		ianjuta_preferences_merge (IANJUTA_PREFERENCES (plugin_object),
+								   anjuta_shell_get_preferences (ANJUTA_SHELL (priv->shell), NULL),
+								   NULL);
+	}
+	
 	update_enabled (GTK_TREE_MODEL (store), activated_plugins);
 	gtk_tree_path_free (path);
 }
@@ -1796,6 +1823,7 @@ anjuta_plugin_manager_select (AnjutaPluginManager *plugin_manager,
 	gtk_window_set_default_size (GTK_WINDOW (dlg), 400, 300);
 
 	label = gtk_label_new (description);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label,
 						FALSE, FALSE, 5);
@@ -2354,7 +2382,7 @@ anjuta_plugin_manager_activate_plugins (AnjutaPluginManager *plugin_manager,
 			icon_path = g_strconcat (PACKAGE_PIXMAPS_DIR"/",
 									 icon_filename, NULL);
 			/* DEBUG_PRINT ("Icon: %s", icon_path); */
-			label = g_strconcat (_("Loaded: "), title, _("..."), NULL);
+			label = g_strconcat (_("Loaded: "), title, "...", NULL);
 			icon_pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
 			if (!icon_pixbuf)
 				g_warning ("Plugin does not define Icon: No such file %s",
