@@ -67,6 +67,7 @@
 
 #include <glade/glade-parser.h>
 #include <gconf/gconf-client.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include <libanjuta/anjuta-preferences.h>
 #include <libanjuta/anjuta-utils.h>
@@ -551,6 +552,7 @@ get_property_value_as_string (AnjutaProperty *prop)
 	gint  int_value;
 	gchar** values;
 	gchar *text_value = NULL;
+	gchar *uri;
 	
 	if (prop->custom)
 	{
@@ -619,8 +621,10 @@ get_property_value_as_string (AnjutaProperty *prop)
 		}
 		break;
 	case ANJUTA_PROPERTY_OBJECT_TYPE_FOLDER:
-		text_value = gtk_file_chooser_get_current_folder (
-				GTK_FILE_CHOOSER (prop->object));
+		uri = gtk_file_chooser_get_uri (
+ 				GTK_FILE_CHOOSER (prop->object));
+		text_value = gnome_vfs_get_local_path_from_uri (uri);
+		g_free (uri);
 		break;
 	case ANJUTA_PROPERTY_OBJECT_TYPE_FILE:
 		text_value = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (prop->object));
@@ -771,7 +775,27 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 		
 	case ANJUTA_PROPERTY_OBJECT_TYPE_FOLDER:
 		if (value)
-			gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (prop->object), value);
+		{
+			gchar *old_folder;
+
+			/* When the user change the folder, the
+			 * current-folder-changed signal is emitted the
+			 * gconf key is updated and this function is called.
+			 * But setting the current folder here emits
+			 * the current-folder-changed signal too.
+			 * Moreover this signal is emitted asynchronously so
+			 * it is not possible to block it here.
+			 * 
+			 * The solution here is to update the widget only
+			 * if it is really needed.
+			 */
+			old_folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (prop->object));
+			if ((old_folder == NULL) || strcmp (old_folder, value))
+			{
+				gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (prop->object), value);
+			}
+			g_free (old_folder);
+		}
 		break;
 	case ANJUTA_PROPERTY_OBJECT_TYPE_FILE:
 		if (value)
@@ -1296,7 +1320,7 @@ anjuta_preferences_register_property_custom (AnjutaPreferences *pr,
  * anjuta_preferences_register_property_from_string:
  * @pr: a #AnjutaPreferences object
  * @object: Widget to register
- * @property_desc: Property description (see anjuta_preferences_add_pag())
+ * @property_desc: Property description (see anjuta_preferences_add_page())
  *
  * This registers only one widget. The widget could be shown elsewhere.
  * the property_description should be of the form described before.
@@ -1627,6 +1651,12 @@ anjuta_preferences_get_dialog (AnjutaPreferences *pr)
 	}
 }
 
+gboolean
+anjuta_preferences_is_dialog_created (AnjutaPreferences *pr)
+{
+	return pr->priv->prefs_dialog;
+}
+
 static void anjuta_preferences_class_init    (AnjutaPreferencesClass *class);
 static void anjuta_preferences_instance_init (AnjutaPreferences      *pr);
 
