@@ -105,7 +105,12 @@ anjuta_status_dispose (GObject *widget)
 		g_hash_table_destroy (status->priv->widgets);
 		status->priv->widgets = NULL;
 	}
-	
+	if (status->priv->window)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (status->priv->window),
+						   (gpointer*)(gpointer)&status->priv->window);
+		status->priv->window = NULL;
+	}
 	GNOME_CALL_PARENT(G_OBJECT_CLASS, dispose, (widget));
 }
 
@@ -510,7 +515,7 @@ anjuta_status_set_title (AnjutaStatus *status, const gchar *title)
 	const gchar *app_name = g_get_application_name();
 	if (title)
 	{
-		gchar* str = g_strconcat (app_name, " - ", title, NULL);
+		gchar* str = g_strconcat (title, " - ", app_name, NULL);
 		gtk_window_set_title (status->priv->window, str);
 		g_free (str);
 	}
