@@ -47,8 +47,7 @@
 
 #include <glib/gi18n.h>
 #include <glib.h>
-
-#include <libgnome/gnome-util.h>
+#include <glib/gstdio.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
@@ -591,6 +590,35 @@ anjuta_util_glist_strings_dup (GList * list)
 	return new_list;
 }
 
+/* Join list of strings using the given delimiter */
+gchar*
+anjuta_util_glist_strings_join (GList * list, gchar *delimiter)
+{
+	GString *joined;
+	gboolean first = TRUE;
+	GList *node;
+	
+	joined = g_string_new (NULL);
+	node = list;
+	while (node)
+	{
+		if (node->data)
+		{
+			if (!first)
+				g_string_append (joined, delimiter);
+			else
+				first = FALSE;
+			g_string_append (joined, node->data);
+		}
+		node = g_list_next (node);
+	}
+	if (joined->len > 0)
+		return g_string_free (joined, FALSE);
+	else
+		g_string_free (joined, TRUE);
+	return NULL;
+}
+
 gchar*
 anjuta_util_get_real_path (const gchar *path)
 {
@@ -1319,3 +1347,209 @@ int scandir(const char *dir, struct dirent ***namelist,
 }
 
 #endif /* HAVE_LIBUTIL */
+
+void
+anjuta_util_help_display (GtkWindow   *parent,
+						  const gchar *doc_id,
+						  const gchar *file_name)
+{
+
+	GError *error = NULL;
+	GdkScreen *screen;
+	gchar *command;
+	const gchar *lang;
+	const gchar * const *langs;
+	gchar *uri = NULL;
+	gint i;
+	
+	g_return_if_fail (file_name != NULL);
+
+	langs = g_get_language_names ();
+	for (i = 0; langs[i]; i++)
+	{
+		lang = langs[i];
+		if (strchr (lang, '.'))
+			continue;
+
+		uri = g_build_filename (DATADIR, "/gnome/help/", doc_id,
+							    lang, file_name, NULL);
+
+		if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
+			break;
+		}
+		g_free (uri);
+		uri = NULL;
+	}
+
+	if (uri == NULL)
+	{
+		anjuta_util_dialog_error (parent, _("Unable to display help. Please make sure Anjuta "
+								  "documentation package is install. It can be downloaded "
+								  "from http://anjuta.org"));
+
+		return;
+	}
+	
+	command = g_strconcat ("gnome-help ghelp://", uri,  NULL);
+	g_free (uri);
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (parent));
+	gdk_spawn_command_line_on_screen (screen, command, &error);
+	if (error != NULL)
+	{
+		g_warning ("Error executing help application: %s",
+				   error->message);
+		g_error_free (error);
+		
+		return;
+	}
+	g_free (command);
+}
+
+/**
+ * anjuta_utils_get_user_config_dir:
+ *
+ * Returns a base directory in which to store user-specific application 
+ * configuration information such as user preferences and settings. 
+ *
+ * Return value: a new allocated string with the user config directory.
+ */
+gchar *
+anjuta_util_get_user_config_dir ()
+{
+	gchar *folder;
+	
+	folder =  g_build_filename (g_get_user_config_dir (), "anjuta", NULL);
+	
+	if (!g_file_test (folder, G_FILE_TEST_IS_DIR))
+	{
+		if (g_mkdir (folder, 755) == -1)
+		{
+			g_warning ("There was an error creating the Anjuta config directory");
+			g_free (folder);
+			
+			return NULL;
+		}
+	}
+
+	return folder;
+}
+
+/* The following functions are taken from gedit */
+
+/* Note that this function replace home dir with ~ */
+gchar *
+anjuta_util_uri_get_dirname (const gchar *uri)
+{
+	gchar *res;
+	gchar *str;
+
+	// CHECK: does it work with uri chaining? - Paolo
+	str = g_path_get_dirname (uri);
+	g_return_val_if_fail (str != NULL, ".");
+
+	if ((strlen (str) == 1) && (*str == '.'))
+	{
+		g_free (str);
+		
+		return NULL;
+	}
+
+	res = anjuta_util_replace_home_dir_with_tilde (str);
+
+	g_free (str);
+	
+	return res;
+}
+
+gchar*
+anjuta_util_replace_home_dir_with_tilde (const gchar *uri)
+{
+	gchar *tmp;
+	gchar *home;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	/* Note that g_get_home_dir returns a const string */
+	tmp = (gchar *)g_get_home_dir ();
+
+	if (tmp == NULL)
+		return g_strdup (uri);
+
+	home = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
+	if (home == NULL)
+		return g_strdup (uri);
+
+	if (strcmp (uri, home) == 0)
+	{
+		g_free (home);
+		
+		return g_strdup ("~");
+	}
+
+	tmp = home;
+	home = g_strdup_printf ("%s/", tmp);
+	g_free (tmp);
+
+	if (g_str_has_prefix (uri, home))
+	{
+		gchar *res;
+
+		res = g_strdup_printf ("~/%s", uri + strlen (home));
+
+		g_free (home);
+		
+		return res;		
+	}
+
+	g_free (home);
+
+	return g_strdup (uri);
+}
+
+gchar *
+anjuta_util_str_middle_truncate (const gchar *string,
+								  guint        truncate_length)
+{
+	GString     *truncated;
+	guint        length;
+	guint        n_chars;
+	guint        num_left_chars;
+	guint        right_offset;
+	guint        delimiter_length;
+	const gchar *delimiter = "\342\200\246";
+
+	g_return_val_if_fail (string != NULL, NULL);
+
+	length = strlen (string);
+
+	g_return_val_if_fail (g_utf8_validate (string, length, NULL), NULL);
+
+	/* It doesnt make sense to truncate strings to less than
+	 * the size of the delimiter plus 2 characters (one on each
+	 * side)
+	 */
+	delimiter_length = g_utf8_strlen (delimiter, -1);
+	if (truncate_length < (delimiter_length + 2)) {
+		return g_strdup (string);
+	}
+
+	n_chars = g_utf8_strlen (string, length);
+
+	/* Make sure the string is not already small enough. */
+	if (n_chars <= truncate_length) {
+		return g_strdup (string);
+	}
+
+	/* Find the 'middle' where the truncation will occur. */
+	num_left_chars = (truncate_length - delimiter_length) / 2;
+	right_offset = n_chars - truncate_length + num_left_chars + delimiter_length;
+
+	truncated = g_string_new_len (string,
+				      g_utf8_offset_to_pointer (string, num_left_chars) - string);
+	g_string_append (truncated, delimiter);
+	g_string_append (truncated, g_utf8_offset_to_pointer (string, right_offset));
+		
+	return g_string_free (truncated, FALSE);
+}
+
