@@ -26,7 +26,15 @@ extern void libanjuta_iface_cclosure_marshal_VOID__BOOLEAN_INT (GClosure     *cl
 /* VOID:INT (./libanjuta-iface-marshallers.list:5) */
 #define libanjuta_iface_cclosure_marshal_VOID__INT	g_cclosure_marshal_VOID__INT
 
-/* VOID:INT,INT,ULONG,STRING,UINT (./libanjuta-iface-marshallers.list:6) */
+/* VOID:INT,INT (./libanjuta-iface-marshallers.list:6) */
+extern void libanjuta_iface_cclosure_marshal_VOID__INT_INT (GClosure     *closure,
+                                                            GValue       *return_value,
+                                                            guint         n_param_values,
+                                                            const GValue *param_values,
+                                                            gpointer      invocation_hint,
+                                                            gpointer      marshal_data);
+
+/* VOID:INT,INT,ULONG,STRING,UINT (./libanjuta-iface-marshallers.list:7) */
 extern void libanjuta_iface_cclosure_marshal_VOID__INT_INT_ULONG_STRING_UINT (GClosure     *closure,
                                                                               GValue       *return_value,
                                                                               guint         n_param_values,
@@ -34,10 +42,10 @@ extern void libanjuta_iface_cclosure_marshal_VOID__INT_INT_ULONG_STRING_UINT (GC
                                                                               gpointer      invocation_hint,
                                                                               gpointer      marshal_data);
 
-/* VOID:OBJECT (./libanjuta-iface-marshallers.list:7) */
+/* VOID:OBJECT (./libanjuta-iface-marshallers.list:8) */
 #define libanjuta_iface_cclosure_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN,INT,INT,STRING (./libanjuta-iface-marshallers.list:8) */
+/* VOID:OBJECT,BOOLEAN,INT,INT,STRING (./libanjuta-iface-marshallers.list:9) */
 extern void libanjuta_iface_cclosure_marshal_VOID__OBJECT_BOOLEAN_INT_INT_STRING (GClosure     *closure,
                                                                                   GValue       *return_value,
                                                                                   guint         n_param_values,
@@ -45,7 +53,7 @@ extern void libanjuta_iface_cclosure_marshal_VOID__OBJECT_BOOLEAN_INT_INT_STRING
                                                                                   gpointer      invocation_hint,
                                                                                   gpointer      marshal_data);
 
-/* VOID:OBJECT,CHAR (./libanjuta-iface-marshallers.list:9) */
+/* VOID:OBJECT,CHAR (./libanjuta-iface-marshallers.list:10) */
 extern void libanjuta_iface_cclosure_marshal_VOID__OBJECT_CHAR (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -53,13 +61,13 @@ extern void libanjuta_iface_cclosure_marshal_VOID__OBJECT_CHAR (GClosure     *cl
                                                                 gpointer      invocation_hint,
                                                                 gpointer      marshal_data);
 
-/* VOID:POINTER (./libanjuta-iface-marshallers.list:10) */
+/* VOID:POINTER (./libanjuta-iface-marshallers.list:11) */
 #define libanjuta_iface_cclosure_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:STRING (./libanjuta-iface-marshallers.list:11) */
+/* VOID:STRING (./libanjuta-iface-marshallers.list:12) */
 #define libanjuta_iface_cclosure_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
 
-/* VOID:STRING,STRING (./libanjuta-iface-marshallers.list:12) */
+/* VOID:STRING,STRING (./libanjuta-iface-marshallers.list:13) */
 extern void libanjuta_iface_cclosure_marshal_VOID__STRING_STRING (GClosure     *closure,
                                                                   GValue       *return_value,
                                                                   guint         n_param_values,
@@ -67,7 +75,7 @@ extern void libanjuta_iface_cclosure_marshal_VOID__STRING_STRING (GClosure     *
                                                                   gpointer      invocation_hint,
                                                                   gpointer      marshal_data);
 
-/* VOID:UINT,INT (./libanjuta-iface-marshallers.list:13) */
+/* VOID:UINT,INT (./libanjuta-iface-marshallers.list:14) */
 extern void libanjuta_iface_cclosure_marshal_VOID__UINT_INT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -75,7 +83,7 @@ extern void libanjuta_iface_cclosure_marshal_VOID__UINT_INT (GClosure     *closu
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
 
-/* VOID:ULONG,STRING,UINT (./libanjuta-iface-marshallers.list:14) */
+/* VOID:ULONG,STRING,UINT (./libanjuta-iface-marshallers.list:15) */
 extern void libanjuta_iface_cclosure_marshal_VOID__ULONG_STRING_UINT (GClosure     *closure,
                                                                       GValue       *return_value,
                                                                       guint         n_param_values,
@@ -83,7 +91,7 @@ extern void libanjuta_iface_cclosure_marshal_VOID__ULONG_STRING_UINT (GClosure
                                                                       gpointer      invocation_hint,
                                                                       gpointer      marshal_data);
 
-/* VOID:VOID (./libanjuta-iface-marshallers.list:15) */
+/* VOID:VOID (./libanjuta-iface-marshallers.list:16) */
 #define libanjuta_iface_cclosure_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
 
 G_END_DECLS
