@@ -20,7 +20,6 @@
 
 #include <config.h>
 #include <ctype.h>
-#include <pcre.h>
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libanjuta/anjuta-shell.h>
@@ -30,6 +29,7 @@
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-loader.h>
 #include <libanjuta/interfaces/ianjuta-buildable.h>
+#include <libanjuta/interfaces/ianjuta-builder.h>
 #include <libanjuta/interfaces/ianjuta-message-manager.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
@@ -37,8 +37,11 @@
 #include <libanjuta/interfaces/ianjuta-preferences.h>
 
 #include "build-basic-autotools.h"
+#include "build-options.h"
 #include "executer.h"
 
+#include <sys/wait.h>
+
 #define ICON_FILE "anjuta-build-basic-autotools-plugin-48.png"
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-build-basic-autotools-plugin.ui"
 #define MAX_BUILD_PANES 3
@@ -55,6 +58,7 @@
 
 #define DEFAULT_COMMAND_COMPILE "make"
 #define DEFAULT_COMMAND_BUILD "make"
+#define DEFAULT_COMMAND_IS_BUILT "make -q"
 #define DEFAULT_COMMAND_BUILD_TARBALL "make dist"
 #define DEFAULT_COMMAND_INSTALL "make install"
 #define DEFAULT_COMMAND_CONFIGURE "./configure"
@@ -74,7 +78,7 @@ typedef struct
 	gchar *pattern;
 	int options;
 	gchar *replace;
-	pcre *regex;
+	GRegex *regex;
 } BuildPattern;
 
 typedef struct
@@ -88,9 +92,13 @@ typedef struct
 typedef struct
 {
 	AnjutaPlugin *plugin;
-	gchar *command;
-	IAnjutaMessageView *message_view;
+	
 	AnjutaLauncher *launcher;
+	gchar *command;
+	IAnjutaBuilderCallback callback;
+	gpointer user_data;
+
+	IAnjutaMessageView *message_view;
 	GHashTable *build_dir_stack;
 	
 	/* Indicator locations */
@@ -257,30 +265,95 @@ build_context_get_dir (BuildContext *context, const gchar *key)
 	return dir_stack->data;
 }
 
-static void
-build_context_destroy (BuildContext *context)
+static gboolean
+build_context_destroy_command (BuildContext *context)
 {
-	if (context->message_view)
-		gtk_widget_destroy (GTK_WIDGET (context->message_view));
+	if (context->callback)
+	{
+		GError *err;
+			
+		/* Emit command-finished signal abort */
+		err = g_error_new_literal (ianjuta_builder_error_quark (),
+								   IANJUTA_BUILDER_ABORTED,
+								   _("Command aborted"));
+		context->callback (G_OBJECT (context->plugin), err, context->user_data);
+		g_error_free (err);
+		context->callback = NULL;
+	}
+		
 	if (context->launcher)
 	{
-		/*
-		if (anjuta_launcher_is_busy (context->launcher))
-			anjuta_launcher_reset (context->launcher);
-		*/
 		g_object_unref (context->launcher);
+		context->launcher = NULL;
 	}
-	if (context->build_dir_stack)
-		g_hash_table_destroy (context->build_dir_stack);
-	if (context->indicators_updated_editors)
-		g_hash_table_destroy (context->indicators_updated_editors);
+	
 	g_free (context->command);
+	context->command = NULL;
+
+	/* Empty context, remove from pool */
+	if (context->message_view == NULL)
+	{
+		ANJUTA_PLUGIN_BASIC_AUTOTOOLS (context->plugin)->contexts_pool =
+			g_list_remove (ANJUTA_PLUGIN_BASIC_AUTOTOOLS (context->plugin)->contexts_pool,
+							context);
+		g_free (context);
+		
+		return TRUE;
+	}
+	else
+	{
+		return FALSE;
+	}
+}
+
+static gboolean
+build_context_destroy_view (BuildContext *context)
+{
+	if (context->message_view)
+	{
+		gtk_widget_destroy (GTK_WIDGET (context->message_view));
+		context->message_view = NULL;
+	}
+
+	if (context->build_dir_stack)
+	{
+		g_hash_table_destroy (context->build_dir_stack);
+		context->build_dir_stack = NULL;
+	}
+	if (context->indicators_updated_editors)
+	{
+		g_hash_table_destroy (context->indicators_updated_editors);
+		context->indicators_updated_editors = NULL;
+	}
 	
 	g_slist_foreach (context->locations, (GFunc) build_indicator_location_free,
 					 NULL);
 	g_slist_free (context->locations);
+	context->locations = NULL;
 	
-	g_free (context);
+	/* Empty context, remove from pool */
+	if (context->launcher == NULL)
+	{
+		ANJUTA_PLUGIN_BASIC_AUTOTOOLS (context->plugin)->contexts_pool =
+			g_list_remove (ANJUTA_PLUGIN_BASIC_AUTOTOOLS (context->plugin)->contexts_pool,
+							context);
+		g_free (context);
+		
+		return TRUE;
+	}
+	else
+	{
+		return FALSE;
+	}
+}
+	
+static void
+build_context_destroy (BuildContext *context)
+{
+	if (build_context_destroy_command (context))
+	{
+		build_context_destroy_view (context);
+	}
 }
 
 static void
@@ -350,8 +423,7 @@ static void
 build_regex_init ()
 {
 	GList *node;
-	const char *error;
-	int erroffset;
+	GError *error = NULL;
 
 	build_regex_load ();
 	if (!patterns_list)
@@ -367,15 +439,15 @@ build_regex_init ()
 		
 		pattern = node->data;
 		pattern->regex =
-			pcre_compile(
+			g_regex_new(
 			   pattern->pattern,
 			   pattern->options,
-			   &error,           /* for error message */
-			   &erroffset,       /* for error offset */
-			   NULL);            /* use default character tables */
-		if (pattern->regex == NULL) {
-			DEBUG_PRINT ("PCRE compilation failed: %s: regex \"%s\" at char %d",
-						pattern->pattern, error, erroffset);
+			   0,
+			   &error);           /* for error message */
+		if (error != NULL) {
+			DEBUG_PRINT ("GRegex compilation failed: pattern \"%s\": error %s",
+						pattern->pattern, error->message);
+			g_error_free (error);
 		}
 		node = g_list_next (node);
 	}
@@ -385,8 +457,8 @@ build_regex_init ()
 static gchar*
 build_get_summary (const gchar *details, BuildPattern* bp)
 {
-	int rc;
-	int ovector[30];
+	gboolean matched;
+	GMatchInfo *match_info;
 	const gchar *iter;
 	GString *ret;
 	gchar *final;
@@ -394,17 +466,13 @@ build_get_summary (const gchar *details, BuildPattern* bp)
 	if (!bp || !bp->regex)
 		return NULL;
 	
-	rc = pcre_exec(
-			bp->regex,       /* result of pcre_compile() */
-			NULL,            /* we didn’t study the pattern */
-			details,         /* the subject string */
-			strlen (details),/* the length of the subject string */
-			0,               /* start at offset 0 in the subject */
-			bp->options,     /* default options */
-			ovector,         /* vector for substring information */
-			30);             /* number of elements in the vector */
+	matched = g_regex_match(
+			  bp->regex,       /* result of g_regex_new() */
+			  details,         /* the subject string */
+			  0,
+			  &match_info);
 	
-	if (rc < 0)
+	if (!matched)
 		return NULL;
 	
 	ret = g_string_new ("");
@@ -414,11 +482,15 @@ build_get_summary (const gchar *details, BuildPattern* bp)
 		if (*iter == '\\' && isdigit(*(iter + 1)))
 		{
 			char temp[2] = {0, 0};
+			gint start_pos, end_pos;
 			
 			temp[0] = *(iter + 1);
 			int idx = atoi (temp);
-			ret = g_string_append_len (ret, details + ovector[2*idx],
-									   ovector[2*idx+1] - ovector[2*idx]);
+
+			g_match_info_fetch_pos (match_info, idx, &start_pos, &end_pos);
+
+			ret = g_string_append_len (ret, details + start_pos,
+									   end_pos - start_pos);
 			iter += 2;
 		}
 		else
@@ -433,6 +505,7 @@ build_get_summary (const gchar *details, BuildPattern* bp)
 			ret = g_string_append_len (ret, start, end - start);
 		}
 	}
+	g_match_info_free (match_info);
 	
 	final = g_string_free (ret, FALSE);
 	if (strlen (final) <= 0) {
@@ -754,6 +827,52 @@ on_build_terminated (AnjutaLauncher *launcher,
 	g_signal_handlers_disconnect_by_func (context->launcher,
 										  G_CALLBACK (on_build_terminated),
 										  context);
+	
+	if (context->callback != NULL)
+	{
+		GError *err = NULL;
+		
+		if (WIFEXITED (status))
+		{
+			if (WEXITSTATUS (status) != 0)
+			{
+				err = g_error_new (ianjuta_builder_error_quark (), 
+								   WEXITSTATUS (status),
+								   _("Command exited with status %d"), WEXITSTATUS (status));
+			}
+		}
+		else if (WIFSIGNALED (status))
+		{
+			switch (WTERMSIG (status))
+			{
+				case SIGTERM:
+					err = g_error_new (ianjuta_builder_error_quark (),
+									 IANJUTA_BUILDER_CANCELED,
+									 _("Command canceled by user"));
+					break;
+				case SIGKILL:
+					err = g_error_new (ianjuta_builder_error_quark (),
+									 IANJUTA_BUILDER_ABORTED,
+									 _("Command aborted by user"));
+					break;
+				default:
+					err = g_error_new (ianjuta_builder_error_quark (),
+									   IANJUTA_BUILDER_INTERRUPTED,
+									   _("Command terminated with signal %d"), WTERMSIG(status));
+					break;
+			}
+		}
+		else
+		{
+			err = g_error_new_literal (ianjuta_builder_error_quark (),
+							   IANJUTA_BUILDER_TERMINATED,
+							   _("Command terminated for an unknown reason"));
+		}
+		
+		context->callback (G_OBJECT (context->plugin), err, context->user_data);	
+		context->callback = NULL;
+	}
+	
 	/* Message view could have been destroyed before */
 	if (context->message_view)
 	{
@@ -780,17 +899,9 @@ on_build_terminated (AnjutaLauncher *launcher,
 			an_message_manager_next(app->messages);
 		*/
 	}
-	if (context->launcher)
-		g_object_unref (context->launcher);
-	context->launcher = NULL;
 	update_project_ui (ANJUTA_PLUGIN_BASIC_AUTOTOOLS (context->plugin));
-}
 
-static void
-build_release_context (BasicAutotoolsPlugin *plugin, BuildContext *context)
-{
-	plugin->contexts_pool = g_list_remove (plugin->contexts_pool, context);
-	build_context_destroy (context);
+	build_context_destroy_command (context);
 }
 
 static void
@@ -798,7 +909,8 @@ on_message_view_destroyed (BuildContext *context, GtkWidget *view)
 {
 	DEBUG_PRINT ("Destroying build context");
 	context->message_view = NULL;
-	build_release_context (ANJUTA_PLUGIN_BASIC_AUTOTOOLS (context->plugin), context);
+	
+	build_context_destroy_view (context);
 }
 
 static gboolean
@@ -808,8 +920,7 @@ g_hashtable_foreach_true (gpointer key, gpointer value, gpointer user_data)
 }
 
 static BuildContext*
-build_get_context (BasicAutotoolsPlugin *plugin, const gchar *dir,
-				   const gchar *command)
+build_get_context_with_message(BasicAutotoolsPlugin *plugin, const gchar *dir)
 {
 	IAnjutaMessageManager *mesg_manager;
 	gchar mname[128];
@@ -874,15 +985,7 @@ build_get_context (BasicAutotoolsPlugin *plugin, const gchar *dir,
 		g_object_weak_ref (G_OBJECT (context->message_view),
 						   (GWeakNotify)on_message_view_destroyed, context);
 	}
-	
-	context->command = g_strdup (command);
-	context->launcher = anjuta_launcher_new ();
-	g_signal_connect (G_OBJECT (context->launcher), "child-exited",
-					  G_CALLBACK (on_build_terminated), context);
-	build_context_push_dir (context, "default", dir);
-	chdir (dir);
-	
-	plugin->contexts_pool = g_list_append (plugin->contexts_pool, context);
+
 	ianjuta_message_manager_set_current_view (mesg_manager,
 											  context->message_view, NULL);
 	
@@ -898,6 +1001,39 @@ build_get_context (BasicAutotoolsPlugin *plugin, const gchar *dir,
 	
 	return context;
 }
+	
+static BuildContext*
+build_get_context (BasicAutotoolsPlugin *plugin, const gchar *dir,
+		   gboolean with_view, const gchar *command,
+		  IAnjutaBuilderCallback callback, gpointer user_data)
+{
+	BuildContext *context = NULL;
+
+	if (with_view)
+	{
+		context = build_get_context_with_message (plugin, dir);
+	}
+	else
+	{
+		/* Context without a message view */
+		context = g_new0 (BuildContext, 1);
+		DEBUG_PRINT ("new context %p", context);
+		context->plugin = ANJUTA_PLUGIN(plugin);
+	}
+		
+	context->callback = callback;
+	context->user_data = user_data;
+	context->command = g_strdup (command);
+	context->launcher = anjuta_launcher_new ();
+	g_signal_connect (G_OBJECT (context->launcher), "child-exited",
+					  G_CALLBACK (on_build_terminated), context);
+	build_context_push_dir (context, "default", dir);
+	chdir (dir);
+	
+	plugin->contexts_pool = g_list_append (plugin->contexts_pool, context);
+	
+	return context;
+}
 
 /* Save all current anjuta files */
 static void
@@ -915,22 +1051,32 @@ save_all_files (AnjutaPlugin *plugin)
 	}
 }
 
-static void
-build_execute_command (BasicAutotoolsPlugin* bplugin, const gchar *dir,
-					   const gchar *command,
-					   gboolean save_file)
+static void build_set_env (gpointer key, gpointer value, gpointer user_data)
+{
+	AnjutaLauncher* launcher = ANJUTA_LAUNCHER (user_data);
+	const gchar* name = key;
+	const gchar* env_value = value;
+	anjuta_launcher_set_env (launcher, name, env_value);
+}
+
+static BuildContext*
+build_execute_command_full (BasicAutotoolsPlugin* bplugin, const gchar *dir,
+			const gchar *command,
+			gboolean save_file, gboolean with_view, GHashTable* env,
+			IAnjutaBuilderCallback callback, gpointer user_data,
+			GError **err)
 {
 	AnjutaPlugin* plugin = ANJUTA_PLUGIN(bplugin);
 	AnjutaPreferences* prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	BuildContext *context;
 	gchar* real_command;
 	
-	g_return_if_fail (command != NULL);
+	g_return_val_if_fail (command != NULL, NULL);
 
 	if (save_file) 
 		save_all_files (ANJUTA_PLUGIN (plugin));
 	
-	context = build_get_context (bplugin, dir, command);
+	context = build_get_context (bplugin, dir, with_view, command, callback, user_data);
 	
 	if (anjuta_preferences_get_int (prefs , PREF_USE_SB))
 	{
@@ -945,16 +1091,63 @@ build_execute_command (BasicAutotoolsPlugin* bplugin, const gchar *dir,
 		real_command = g_strdup(command);
 	}
 	
-	ianjuta_message_view_buffer_append (context->message_view,
-										"Building in directory: ", NULL);
-	ianjuta_message_view_buffer_append (context->message_view, dir, NULL);
-	ianjuta_message_view_buffer_append (context->message_view, "\n", NULL);
-	ianjuta_message_view_buffer_append (context->message_view, command, NULL);
-	ianjuta_message_view_buffer_append (context->message_view, "\n", NULL);
+	if (env)
+	{
+		g_hash_table_foreach (env, build_set_env, context->launcher);
+	}
 	
-	anjuta_launcher_execute (context->launcher, real_command,
-							 on_build_mesg_arrived, context);
+	if (context->message_view)
+	{
+		ianjuta_message_view_buffer_append (context->message_view,
+										"Building in directory: ", NULL);
+		ianjuta_message_view_buffer_append (context->message_view, dir, NULL);
+		ianjuta_message_view_buffer_append (context->message_view, "\n", NULL);
+		ianjuta_message_view_buffer_append (context->message_view, command, NULL);
+		ianjuta_message_view_buffer_append (context->message_view, "\n", NULL);
+	
+		anjuta_launcher_execute (context->launcher, real_command,
+								 on_build_mesg_arrived, context);
+	}
+	else
+	{
+		anjuta_launcher_execute (context->launcher, real_command,
+								 NULL, NULL);
+	}		
 	g_free(real_command);
+	
+	return context;
+}
+
+
+static gboolean
+build_execute_command (BasicAutotoolsPlugin* bplugin, const gchar *dir,
+		   const gchar *command, gboolean save_file, GError **err)
+{
+	return build_execute_command_full (bplugin, dir, command, save_file, TRUE, NULL, NULL, NULL, NULL) != NULL;
+}
+
+static void
+build_cancel_command (BasicAutotoolsPlugin* bplugin, BuildContext *context,
+					  GError **err)
+{
+	GList *node;
+
+	if (context == NULL) return;
+		
+	for (node = g_list_first (bplugin->contexts_pool); node != NULL; node = g_list_next (node))
+	{
+		if (node->data == context)
+		{	
+			if (context->launcher != NULL)
+			{
+				anjuta_launcher_signal (context->launcher, SIGTERM);
+				return;
+			}
+		}	
+	}	
+
+	/* Invalid handle passed */
+	g_return_if_reached ();	
 }
 
 static gboolean
@@ -1000,7 +1193,7 @@ build_compile_file_real (BasicAutotoolsPlugin *plugin, const gchar *file)
 			*ext_ptr = '\0';
 			command = g_strconcat (CHOOSE_COMMAND (plugin, COMPILE), " ",
 								   file_basename, new_ext, NULL);
-			build_execute_command (plugin, file_dirname, command, TRUE);
+			build_execute_command (plugin, file_dirname, command, TRUE, NULL);
 			g_free (command);
 			ret = TRUE;
 		}
@@ -1009,7 +1202,7 @@ build_compile_file_real (BasicAutotoolsPlugin *plugin, const gchar *file)
 		gchar *command;
 		command = g_strconcat (CHOOSE_COMMAND(plugin, COMPILE), " ",
 							   file_basename, NULL);
-		build_execute_command (plugin, file_dirname, command, TRUE);
+		build_execute_command (plugin, file_dirname, command, TRUE, NULL);
 		g_free (command);
 		ret = TRUE;
 	}
@@ -1034,7 +1227,7 @@ static void
 build_build_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	build_execute_command (plugin, plugin->project_root_dir,
-						   CHOOSE_COMMAND (plugin, BUILD), TRUE);
+						   CHOOSE_COMMAND (plugin, BUILD), TRUE, NULL);
 }
 
 static void
@@ -1045,7 +1238,7 @@ build_install_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 									 CHOOSE_COMMAND (plugin, INSTALL));
 	g_free(root);
 	build_execute_command (plugin, plugin->project_root_dir,
-						   command, TRUE);
+						   command, TRUE, NULL);
 	g_free(command);
 }
 
@@ -1053,7 +1246,7 @@ static void
 build_clean_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	build_execute_command (plugin, plugin->project_root_dir,
-						   CHOOSE_COMMAND (plugin, CLEAN), FALSE);
+						   CHOOSE_COMMAND (plugin, CLEAN), FALSE, NULL);
 }
 
 static void
@@ -1062,11 +1255,14 @@ build_configure_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	gint response;
 	GtkWindow *parent;
 	gchar *input = NULL;
+	GHashTable* build_options = NULL;
 	
 	parent = GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell);
 	/* Configure = ./configure script */
-	response = anjuta_util_dialog_input (parent, _("Configure Parameters:"),
-										 plugin->configure_args, &input);
+	response = build_dialog_configure (parent, _("Configure"),
+									   &build_options, 
+									   plugin->configure_args, 
+									   &input);
 	if (response)
 	{
 		gchar *cmd;
@@ -1081,8 +1277,11 @@ build_configure_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		{
 			cmd = g_strdup (CHOOSE_COMMAND (plugin, CONFIGURE));
 		}
-		build_execute_command (plugin, plugin->project_root_dir, cmd, TRUE);
+		build_execute_command_full (plugin, plugin->project_root_dir, 
+					   cmd, TRUE, TRUE, build_options,
+					   NULL, NULL, NULL);
 		g_free (cmd);
+		g_hash_table_destroy (build_options);
 	}
 }
 
@@ -1092,10 +1291,13 @@ build_autogen_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	gint response;
 	GtkWindow *parent;
 	gchar *input = NULL;
+	GHashTable* build_options = NULL;
 	
 	parent = GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell);
-	response = anjuta_util_dialog_input (parent, _("Configure Parameters:"),
-										 plugin->configure_args, &input);
+	response = build_dialog_configure (parent, _("Autogenerate"),
+									   &build_options, 
+									   plugin->configure_args, 
+									   &input);
 	if (response)
 	{
 		gboolean has_autogen = directory_has_file (plugin->project_root_dir,
@@ -1119,8 +1321,11 @@ build_autogen_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 			else /* FIXME: Get override command for this too */
 				cmd = g_strdup ("autoreconf -i --force");
 		}
-		build_execute_command (plugin, plugin->project_root_dir, cmd, TRUE);
+		build_execute_command_full (plugin, plugin->project_root_dir, 
+					cmd, TRUE, TRUE, build_options,
+		 			NULL, NULL, NULL);
 		g_free (cmd);
+		g_hash_table_destroy (build_options);
 	}
 }
 
@@ -1128,22 +1333,17 @@ static void
 build_distribution_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	build_execute_command (plugin, plugin->project_root_dir,
-						   CHOOSE_COMMAND (plugin, BUILD_TARBALL),
-						   FALSE);
-}
-
-static void
-build_execute_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
-{
-	execute_program (plugin, NULL);
+			   CHOOSE_COMMAND (plugin, BUILD_TARBALL),
+			   FALSE, NULL);
 }
 
 static void
 build_build_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	gchar *dirname = g_dirname (plugin->current_editor_filename);
-	build_execute_command (plugin, dirname, CHOOSE_COMMAND (plugin, BUILD),
-						   TRUE);
+	build_execute_command (plugin, dirname,
+			   CHOOSE_COMMAND (plugin, BUILD),
+			   TRUE, NULL);
 	g_free (dirname);
 }
 
@@ -1155,8 +1355,7 @@ build_install_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	gchar* command = g_strdup_printf ("%s %s", root,
 									  CHOOSE_COMMAND (plugin, INSTALL));
 	g_free(root);
-	build_execute_command (plugin, dirname,
-						   command, TRUE);
+	build_execute_command (plugin, dirname, command, TRUE, NULL);
 	g_free(command);
 	g_free (dirname);
 }
@@ -1165,8 +1364,9 @@ static void
 build_clean_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	gchar *dirname = g_dirname (plugin->current_editor_filename);
-	build_execute_command (plugin, dirname, CHOOSE_COMMAND (plugin, CLEAN),
-						   FALSE);
+	build_execute_command (plugin, dirname,
+			   CHOOSE_COMMAND (plugin, CLEAN),
+			   FALSE, NULL);
 	g_free (dirname);
 }
 
@@ -1200,7 +1400,9 @@ fm_build (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		dir = g_strdup (plugin->fm_current_filename);
 	else
 		dir = g_path_get_dirname (plugin->fm_current_filename);
-	build_execute_command (plugin, dir, CHOOSE_COMMAND (plugin, BUILD), TRUE);
+	build_execute_command (plugin, dir,
+			   CHOOSE_COMMAND (plugin, BUILD),
+			   TRUE, NULL);
 }
 
 static void
@@ -1221,7 +1423,7 @@ fm_install (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	command = g_strdup_printf ("%s %s", root,
 							   CHOOSE_COMMAND (plugin, INSTALL));
 	g_free(root);
-	build_execute_command (plugin, dir, command, TRUE);
+	build_execute_command (plugin, dir, command, TRUE, NULL);
 }
 
 static void
@@ -1235,7 +1437,9 @@ fm_clean (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		dir = g_strdup (plugin->fm_current_filename);
 	else
 		dir = g_path_get_dirname (plugin->fm_current_filename);
-	build_execute_command (plugin, dir, CHOOSE_COMMAND (plugin, CLEAN), FALSE);
+	build_execute_command (plugin, dir,
+			   CHOOSE_COMMAND (plugin, CLEAN),
+			   FALSE, NULL);
 }
 
 /* Project manager context menu */
@@ -1259,7 +1463,9 @@ pm_build (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		dir = g_strdup (plugin->pm_current_filename);
 	else
 		dir = g_path_get_dirname (plugin->pm_current_filename);
-	build_execute_command (plugin, dir, CHOOSE_COMMAND (plugin, BUILD), TRUE);
+	build_execute_command (plugin, dir,
+			   CHOOSE_COMMAND (plugin, BUILD),
+			   TRUE, NULL);
 }
 
 static void
@@ -1280,7 +1486,7 @@ pm_install (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		dir = g_strdup (plugin->pm_current_filename);
 	else
 		dir = g_path_get_dirname (plugin->pm_current_filename);
-	build_execute_command (plugin, dir, command, TRUE);
+	build_execute_command (plugin, dir, command, TRUE, NULL);
 	g_free(command);
 }
 
@@ -1295,7 +1501,9 @@ pm_clean (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		dir = g_strdup (plugin->pm_current_filename);
 	else
 		dir = g_path_get_dirname (plugin->pm_current_filename);
-	build_execute_command (plugin, dir, CHOOSE_COMMAND (plugin, CLEAN), FALSE);
+	build_execute_command (plugin, dir,
+			   CHOOSE_COMMAND (plugin, CLEAN),
+			   FALSE, NULL);
 }
 
 static GtkActionEntry build_actions[] = 
@@ -1340,12 +1548,6 @@ static GtkActionEntry build_actions[] =
 		N_("Build project tarball distribution"),
 		G_CALLBACK (build_distribution_project)
 	},
-	{
-		"ActionBuildExecute", NULL,
-		N_("_Execute Program..."), "F3",
-		N_("Execute program"),
-		G_CALLBACK (build_execute_project)
-	},
 	{
 		"ActionBuildBuildModule", GTK_STOCK_EXECUTE,
 		N_("_Build Module"), "F11",
@@ -1934,9 +2136,9 @@ on_editor_changed (IAnjutaEditor *editor, IAnjutaIterable *position,
 		ianjuta_indicable_set (IANJUTA_INDICABLE (editor), begin_pos,
 							   end_pos, IANJUTA_INDICABLE_NONE, NULL);
 	}
-	DEBUG_PRINT ("Editor changed: line = %d, added = %d,"
-				 " length = %d, lines = %d, text = \'%s\'",
-				 line, added, length, lines, text);
+	DEBUG_PRINT ("Editor changed: line number = %d, added = %d,"
+				 " text length = %d, number of lines = %d",
+				 line, added, length, lines);
 	g_object_unref (begin_pos);
 	g_object_unref (end_pos);
 }
@@ -2100,7 +2302,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 static void
 dispose (GObject *obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (G_OBJECT(obj)));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
@@ -2129,7 +2331,7 @@ finalize (GObject *obj)
 	ba_plugin->program_args = NULL;
 	ba_plugin->configure_args = NULL;
 	
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (G_OBJECT(obj)));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -2216,8 +2418,9 @@ ibuildable_build (IAnjutaBuildable *manager, const gchar *directory,
 				  GError **err)
 {
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (manager);
-	build_execute_command (plugin, directory, CHOOSE_COMMAND (plugin, BUILD),
-						   TRUE);
+	build_execute_command (plugin, directory,
+			   CHOOSE_COMMAND (plugin, BUILD),
+			   TRUE, NULL);
 }
 
 static void
@@ -2225,8 +2428,9 @@ ibuildable_clean (IAnjutaBuildable *manager, const gchar *directory,
 				  GError **err)
 {
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (manager);
-	build_execute_command (plugin, directory, CHOOSE_COMMAND (plugin, CLEAN),
-						   FALSE);
+	build_execute_command (plugin, directory,
+			   CHOOSE_COMMAND (plugin, CLEAN),
+			   FALSE, NULL);
 }
 
 static void
@@ -2238,7 +2442,7 @@ ibuildable_install (IAnjutaBuildable *manager, const gchar *directory,
 	gchar* command = g_strdup_printf ("%s %s", root,
 									  CHOOSE_COMMAND (plugin, INSTALL));
 	g_free(root);
-	build_execute_command (plugin, directory, command, TRUE);
+	build_execute_command (plugin, directory, command, TRUE, NULL);
 	g_free(command);
 }
 
@@ -2248,8 +2452,8 @@ ibuildable_configure (IAnjutaBuildable *manager, const gchar *directory,
 {
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (manager);
 	build_execute_command (plugin, directory,
-						   CHOOSE_COMMAND (plugin, CONFIGURE),
-						   TRUE);
+			   CHOOSE_COMMAND (plugin, CONFIGURE),
+			   TRUE, NULL);
 }
 
 static void
@@ -2260,13 +2464,15 @@ ibuildable_generate (IAnjutaBuildable *manager, const gchar *directory,
 	if (directory_has_file (plugin->project_root_dir, "autogen.sh"))
 	{
 		build_execute_command (plugin, directory,
-							   CHOOSE_COMMAND (plugin, GENERATE), FALSE);
+				   CHOOSE_COMMAND (plugin, GENERATE),
+				   FALSE, NULL);
 	}
 	else
 	{
 		/* FIXME: get override command for this too */
 		build_execute_command (plugin, directory,
-							   "autoreconf -i --force", FALSE);
+				   "autoreconf -i --force", 
+				   FALSE, NULL);
 	}
 }
 
@@ -2317,6 +2523,88 @@ ifile_iface_init (IAnjutaFileIface *iface)
 	iface->get_uri = ifile_get_uri;
 }
 
+
+/* IAnjutaBuilder implementation
+ *---------------------------------------------------------------------------*/
+
+static IAnjutaBuilderHandle
+ibuilder_is_built (IAnjutaBuilder *builder, const gchar *uri,
+	       IAnjutaBuilderCallback callback, gpointer user_data,
+       	       GError **err)
+{
+	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
+	BuildContext *context;
+	gchar *filename;
+	gchar *target;
+	gchar *dirname;
+	gchar *cmd;
+	
+	filename = gnome_vfs_get_local_path_from_uri (uri);
+	if (filename == NULL) return NULL;
+	target = g_path_get_basename (filename);
+	dirname = g_path_get_dirname (filename);
+	g_free (filename);	
+	cmd = g_strconcat (CHOOSE_COMMAND (plugin, IS_BUILT), " ", target, NULL);
+	g_free (target);
+	
+	context = build_execute_command_full (plugin, dirname, cmd,
+			       			TRUE, FALSE, NULL,
+			       			callback, user_data, err);
+	g_free (cmd);
+	g_free (dirname);
+	
+	return (IAnjutaBuilderHandle)context;
+}
+
+static IAnjutaBuilderHandle
+ibuilder_build (IAnjutaBuilder *builder, const gchar *uri,
+	       	IAnjutaBuilderCallback callback, gpointer user_data,
+		GError **err)
+{
+	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
+	BuildContext *context;
+	gchar *filename;
+	gchar *target;
+	gchar *dirname;
+	gchar *cmd;
+
+	filename = gnome_vfs_get_local_path_from_uri (uri);
+	if (filename == NULL) return NULL;
+	target = g_path_get_basename (filename);
+	dirname = g_path_get_dirname (filename);
+	g_free (filename);	
+	cmd = g_strconcat (CHOOSE_COMMAND (plugin, BUILD), " ", target, NULL);
+	g_free (target);
+
+	context = build_execute_command_full (plugin, dirname, cmd,
+		       				TRUE, TRUE, NULL,
+						callback, user_data, err);
+
+	g_free (cmd);
+	g_free (dirname);
+	
+	return (IAnjutaBuilderHandle)context;
+}
+
+static void
+ibuilder_cancel (IAnjutaBuilder *builder, IAnjutaBuilderHandle handle, GError **err)
+{
+	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
+
+	build_cancel_command (plugin, (BuildContext *)handle, err);
+}
+
+static void
+ibuilder_iface_init (IAnjutaBuilderIface *iface)
+{
+	iface->is_built = ibuilder_is_built;
+	iface->build = ibuilder_build;
+	iface->cancel = ibuilder_cancel;
+}
+
+/* IAnjutaPreferences implementation
+ *---------------------------------------------------------------------------*/
+
 static GladeXML *gxml;
 
 static void
@@ -2371,6 +2659,7 @@ ipreferences_iface_init(IAnjutaPreferencesIface* iface)
 }
 
 ANJUTA_PLUGIN_BEGIN (BasicAutotoolsPlugin, basic_autotools_plugin);
+ANJUTA_PLUGIN_ADD_INTERFACE (ibuilder, IANJUTA_TYPE_BUILDER);
 ANJUTA_PLUGIN_ADD_INTERFACE (ibuildable, IANJUTA_TYPE_BUILDABLE);
 ANJUTA_PLUGIN_ADD_INTERFACE (ifile, IANJUTA_TYPE_FILE);
 ANJUTA_PLUGIN_ADD_INTERFACE (ipreferences, IANJUTA_TYPE_PREFERENCES);
