@@ -16,13 +16,15 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 #include <sexy-icon-entry.h>
 #include <string.h>
 #include <gtk/gtk.h>
 
+#include "anjuta-marshal.h"
+
 #define ICON_MARGIN 2
 #define MAX_ICONS 2
 
@@ -129,7 +131,7 @@ sexy_icon_entry_class_init(SexyIconEntryClass *klass)
 					 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 					 G_STRUCT_OFFSET(SexyIconEntryClass, icon_pressed),
 					 NULL, NULL,
-					 gtk_marshal_VOID__INT_INT,
+					 anjuta_marshal_VOID__INT_INT,
 					 G_TYPE_NONE, 2,
 					 G_TYPE_INT,
 					 G_TYPE_INT);
@@ -149,7 +151,7 @@ sexy_icon_entry_class_init(SexyIconEntryClass *klass)
 					 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 					 G_STRUCT_OFFSET(SexyIconEntryClass, icon_released),
 					 NULL, NULL,
-					 gtk_marshal_VOID__INT_INT,
+					 anjuta_marshal_VOID__INT_INT,
 					 G_TYPE_NONE, 2,
 					 G_TYPE_INT,
 					 G_TYPE_INT);
@@ -469,9 +471,10 @@ static GdkPixbuf *
 get_pixbuf_from_icon(SexyIconEntry *entry, SexyIconEntryPosition icon_pos)
 {
 	GdkPixbuf *pixbuf = NULL;
-	gchar *stock_id;
+	const gchar *stock_id;
 	SexyIconInfo *icon_info = &entry->priv->icons[icon_pos];
 	GtkIconSize size;
+	int w, h;
 
 	switch (gtk_image_get_storage_type(GTK_IMAGE(icon_info->icon)))
 	{
@@ -481,11 +484,17 @@ get_pixbuf_from_icon(SexyIconEntry *entry, SexyIconEntryPosition icon_pos)
 			break;
 
 		case GTK_IMAGE_STOCK:
-			gtk_image_get_stock(GTK_IMAGE(icon_info->icon), &stock_id, &size);
+			gtk_image_get_stock(GTK_IMAGE(icon_info->icon), (char**)&stock_id, &size);
 			pixbuf = gtk_widget_render_icon(GTK_WIDGET(entry),
 											stock_id, size, NULL);
 			break;
 
+		case GTK_IMAGE_ICON_NAME:
+			gtk_image_get_icon_name (GTK_IMAGE(icon_info->icon), &stock_id, &size);
+			gtk_icon_size_lookup (size, &w, &h);
+			pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), stock_id, size, 0, NULL);
+			break;
+          
 		default:
 			return NULL;
 	}
@@ -571,7 +580,6 @@ draw_icon(GtkWidget *widget, SexyIconEntryPosition icon_pos)
 
 		scale = height - (2 * ICON_MARGIN);
 
-		printf("scale = %d (height = %d)\n", scale, height);
 		temp_pixbuf = gdk_pixbuf_scale_simple(pixbuf, scale, scale,
 											  GDK_INTERP_BILINEAR);
 
@@ -657,7 +665,8 @@ update_icon(GObject *obj, GParamSpec *param, SexyIconEntry *entry)
 
 		if (strcmp(name, "pixbuf")   && strcmp(name, "stock")  &&
 			strcmp(name, "image")    && strcmp(name, "pixmap") &&
-			strcmp(name, "icon_set") && strcmp(name, "pixbuf_animation"))
+			strcmp(name, "icon-set") && strcmp(name, "pixbuf-animation") &&
+			strcmp(name, "icon-name"))
 		{
 			return;
 		}
