@@ -15,12 +15,15 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef _SIGNALS_H_
 #define _SIGNALS_H_
 
+#include "plugin.h"
+#include "queue.h"
+
 #include <libanjuta/anjuta-plugin.h>
 #include <libanjuta/interfaces/ianjuta-debugger.h>
 
@@ -44,7 +47,7 @@ struct _SignalsGui
 struct _Signals
 {
 	SignalsGui  widgets;
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	gboolean is_showing;
 	gint win_pos_x, win_pos_y, win_width, win_height;
 	gint	idx;
@@ -54,7 +57,7 @@ struct _Signals
 	gboolean pass;
 };
 
-Signals* signals_new (IAnjutaDebugger *debugger);
+Signals* signals_new (DebugManagerPlugin *plugin);
 
 void signals_clear (Signals *ew);
 
