@@ -37,6 +37,9 @@
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/resources.h>
 #include <libanjuta/interfaces/ianjuta-project-manager.h>
+#include <libanjuta/interfaces/ianjuta-document-manager.h>
+#include <libanjuta/interfaces/ianjuta-builder.h>
+#include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/anjuta-utils.h>
 
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -97,6 +100,12 @@ static char *column_names[COLUMNS_NB] = {
 	N_("Pid"), N_("User"), N_("Time"), N_("Command")
 };
 
+enum {
+	VARIABLE_COLUMN,
+	VALUE_COLUMN,
+	VAR_COLUMNS_NB
+};
+
 struct _AddSourceDialog
 {
 	GtkTreeView *tree;
@@ -116,24 +125,16 @@ struct _DmaStart
 
 	DmaDebuggerQueue *debugger;
 
-	gchar* target_uri;
-	gchar* program_args;
-	gboolean run_in_terminal;
 	gboolean stop_at_beginning;
-
- 	GList *source_dirs;
+	GList *source_dirs;
+	
+	gchar *build_target;
+	IAnjutaBuilderHandle build_handle;
 };
 
 /* Widgets found in glade file
  *---------------------------------------------------------------------------*/
 
-#define PARAMETER_DIALOG "parameter_dialog"
-#define TERMINAL_CHECK_BUTTON "parameter_run_in_term_check"
-#define STOP_AT_BEGINNING_CHECK_BUTTON "stop_at_beginning_check"
-#define PARAMETER_COMBO "parameter_combo"
-#define TARGET_COMBO "target_combo"
-#define TARGET_SELECT_SIGNAL "on_select_target_clicked"
-
 #define ADD_SOURCE_DIALOG "source_paths_dialog"
 #define SOURCE_ENTRY "src_entry"
 #define SOURCE_LIST "src_clist"
@@ -142,7 +143,17 @@ struct _DmaStart
 #define UP_BUTTON "up_button"
 #define DOWN_BUTTON "down_button"
 
-#define ANJUTA_RESPONSE_SELECT_TARGET 0
+/* Constants
+ *---------------------------------------------------------------------------*/
+
+#define RUN_PROGRAM_URI	"run_program_uri"
+#define RUN_PROGRAM_ARGS "run_program_args"
+#define RUN_PROGRAM_DIR	"run_program_directory"
+#define RUN_PROGRAM_ENV	"run_program_environment"
+#define RUN_PROGRAM_NEED_TERM	"run_program_need_terminal"
+
+#define RUN_PROGRAM_ACTION_GROUP "ActionGroupRun"
+#define RUN_PROGRAM_PARAMETER_ACTION "ActionProgramParameters"
 
 static void attach_process_clear (AttachProcess * ap, gint ClearRequest);
 
@@ -228,82 +239,26 @@ free_source_directories (GList *dirs)
  *---------------------------------------------------------------------------*/
 
 static void
-on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase, AnjutaSession *session, DmaStart *this)
+on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase, AnjutaSession *session, DmaStart *self)
 {
 	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
 		return;
 
-	if (this->program_args)
-	{
-		gchar *arg;
-		anjuta_session_set_string (session, "Execution", "Program arguments", this->program_args);
-		arg = anjuta_session_get_string (session, "Execution", "Program arguments");
-	}
-	if (this->target_uri)
-	{
-		gchar *uri;
-		anjuta_session_set_string (session, "Execution", "Program uri", this->target_uri);
-		uri = anjuta_session_get_string (session, "Execution", "Program uri");
-	}
-	anjuta_session_set_int (session, "Execution", "Run in terminal", this->run_in_terminal + 1);
-	anjuta_session_set_int (session, "Execution", "Stop at beginning", this->stop_at_beginning + 1);
- 	if (this->source_dirs)
- 	{
- 		anjuta_session_set_string_list (session, "Debugger", "Source directories", this->source_dirs);
- 	}
+	anjuta_session_set_int (session, "Debugger", "Stop at beginning", self->stop_at_beginning + 1);
 }
 
-static void on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase, AnjutaSession *session, DmaStart *this)
+static void on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase, AnjutaSession *session, DmaStart *self)
 {
-	gchar *program_args;
-	gchar *target_uri;
-    gint run_in_terminal;
 	gint stop_at_beginning;
 
 	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
 		return;
 
-	program_args = anjuta_session_get_string (session, "Execution", "Program arguments");
-	if (this->program_args)
-	{
-		g_free (this->program_args);
-		this->program_args = NULL;
-	}
-    if (program_args)
-    {
-		this->program_args = program_args;
-    }
-
-	target_uri = anjuta_session_get_string (session, "Execution", "Program uri");
-	if (this->target_uri)
-	{
-		g_free (this->target_uri);
-		this->target_uri = NULL;
-	}
-    if (target_uri)
-    {
-		this->target_uri = target_uri;
-    }
-	
-	/* The flag is store as 1 == FALSE, 2 == TRUE */
-	run_in_terminal = anjuta_session_get_int (session, "Execution", "Run in terminal");
-	if (run_in_terminal == 0)
-		this->run_in_terminal = TRUE;	/* Default value */
-	else
-		this->run_in_terminal = run_in_terminal - 1;
-	
-	stop_at_beginning = anjuta_session_get_int (session, "Execution", "Stop at beginning");
+	stop_at_beginning = anjuta_session_get_int (session, "Debugger", "Stop at beginning");
 	if (stop_at_beginning == 0)
-		this->stop_at_beginning = TRUE;	/* Default value */
+		self->stop_at_beginning = TRUE;	/* Default value */
 	else
-		this->stop_at_beginning = stop_at_beginning - 1;
-	/* Initialize source_dirs */
- 	if (this->source_dirs != NULL)
- 	{		
- 		g_list_foreach (this->source_dirs, (GFunc)g_free, NULL);
- 		g_list_free (this->source_dirs);
- 	}
- 	this->source_dirs = anjuta_session_get_string_list (session, "Debugger", "Source directories");
+		self->stop_at_beginning = stop_at_beginning - 1;
 }
 
 /* Attach to process private functions
@@ -625,7 +580,11 @@ attach_process_update (AttachProcess * ap)
 		return;
 
 	tmp = anjuta_util_get_a_tmp_file ();
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__)
+	cmd = g_strconcat ("ps axw -o pid,user,lstart,args > ", tmp, NULL);
+#else
 	cmd = g_strconcat ("ps axw -H -o pid,user,start_time,args > ", tmp, NULL);
+#endif
 	shell = gnome_util_user_shell ();
 	ch_pid = fork ();
 	if (ch_pid == 0)
@@ -842,242 +801,184 @@ attach_process_destroy (AttachProcess * ap)
  *---------------------------------------------------------------------------*/
 
 static void
-on_select_target (GtkButton *button, gpointer user_data)
+show_parameters_dialog (DmaStart *this)
 {
-	gtk_dialog_response (GTK_DIALOG (user_data), ANJUTA_RESPONSE_SELECT_TARGET);
-}
-
-static void
-dma_start_load_uri (DmaStart *this)
-{
-	GList *search_dirs;
-	GnomeVFSURI *vfs_uri;
-	gchar *mime_type;
-	gchar *filename;
-
-	if (!dma_quit_debugger (this)) return;
+	AnjutaUI *ui;
+	GtkAction *action;
 	
-	if ((this->target_uri != NULL) && (*(this->target_uri) != '\0'))
+	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (this->plugin)->shell, NULL);
+	action = anjuta_ui_get_action (ui, RUN_PROGRAM_ACTION_GROUP, RUN_PROGRAM_PARAMETER_ACTION);
+	if (action != NULL)
 	{
-		vfs_uri = gnome_vfs_uri_new (this->target_uri);
-		
-		g_return_if_fail (vfs_uri != NULL);
-		
-		if (!gnome_vfs_uri_is_local (vfs_uri)) return;
-			
-		search_dirs = get_source_directories (this->plugin);
-		
-		mime_type = gnome_vfs_get_mime_type (this->target_uri);
-	    filename = gnome_vfs_get_local_path_from_uri (this->target_uri);
-
-		dma_queue_load (this->debugger, filename, mime_type, this->source_dirs);
-		
-		g_free (filename);
-		g_free (mime_type);
-		gnome_vfs_uri_unref (vfs_uri);
-		free_source_directories (search_dirs);
-
+		gtk_action_activate (action);
 	}
 }
 
 static gboolean
-dma_set_parameters (DmaStart *this)
+start_target (DmaStart *this)
 {
-	GladeXML *gxml;
-	GtkWidget *dlg;
-	GtkWindow *parent;
-	GtkToggleButton *term;
-	GtkToggleButton *stop_at_beginning;
-	GtkComboBox *params;
-	GtkComboBox *target;
-	gint response;
-	GtkTreeModel* model;
-	GtkTreeIter iter;
-	GValue value = {0,};
-	const gchar *project_root_uri;
+	gchar *dir_uri;
+	gchar *dir;
+	gchar **env;
+	gchar *args;
+	gboolean run_in_terminal;
+
+	anjuta_shell_get (ANJUTA_PLUGIN (this->plugin)->shell,
+	 				  RUN_PROGRAM_DIR, G_TYPE_STRING, &dir_uri,
+					  RUN_PROGRAM_ARGS, G_TYPE_STRING, &args,
+					  RUN_PROGRAM_ENV, G_TYPE_STRV, &env,
+					  RUN_PROGRAM_NEED_TERM, G_TYPE_BOOLEAN, &run_in_terminal,
+					  NULL);	
 	
-	parent = GTK_WINDOW (this->plugin->shell);
-	gxml = glade_xml_new (GLADE_FILE, PARAMETER_DIALOG, NULL);
-	if (gxml == NULL)
+	if (dir_uri != NULL)
 	{
-		anjuta_util_dialog_error(parent, _("Missing file %s"), GLADE_FILE);
+		dir = gnome_vfs_get_local_path_from_uri (dir_uri);
+		g_free (dir_uri);
+	}
+	else
+	{
+		dir = NULL;
+	}
+	
+	dma_queue_set_working_directory (this->debugger, dir);
+	g_free (dir);
+	
+	dma_queue_set_environment (this->debugger, env);
+	g_strfreev (env);
+	
+	dma_queue_start (this->debugger, args, run_in_terminal, FALSE);
+	g_free (args);
+	
+	return TRUE;
+
+}
+
+static gboolean
+load_target (DmaStart *this, const gchar *target)
+{
+	gchar *mime_type;
+	gchar *filename;
+	
+	mime_type = gnome_vfs_get_mime_type ((const gchar *)target);
+	if (mime_type == NULL)
+	{
+		anjuta_util_dialog_error(GTK_WINDOW (this->plugin->shell),
+				_("Unable to open %s. Debugger cannot start."), target);
 		return FALSE;
 	}
-		
-	dlg = glade_xml_get_widget (gxml, PARAMETER_DIALOG);
-	term = GTK_TOGGLE_BUTTON (glade_xml_get_widget (gxml, TERMINAL_CHECK_BUTTON));
-	stop_at_beginning = GTK_TOGGLE_BUTTON (glade_xml_get_widget (gxml, STOP_AT_BEGINNING_CHECK_BUTTON));
-	params = GTK_COMBO_BOX (glade_xml_get_widget (gxml, PARAMETER_COMBO));
-	target = GTK_COMBO_BOX (glade_xml_get_widget (gxml, TARGET_COMBO));
-	glade_xml_signal_connect_data (gxml, TARGET_SELECT_SIGNAL, GTK_SIGNAL_FUNC (on_select_target), dlg);
-	g_object_unref (gxml);
+	filename = gnome_vfs_get_local_path_from_uri (target);
+	
+	dma_queue_load (this->debugger, filename, mime_type, this->source_dirs);
 
-	/* Fill parameter combo box */
-	model = GTK_TREE_MODEL (gtk_list_store_new (1, GTK_TYPE_STRING));
-	gtk_combo_box_set_model (params, model);
-	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY(params), 0);
-	if (this->program_args != NULL)
+	g_free (filename);
+	g_free (mime_type);
+
+	return start_target (this);
+}
+
+static void
+on_build_finished (GObject *builder, GError *err, gpointer user_data)
+{
+	DmaStart *this = (DmaStart *)user_data;
+	
+	if (err == NULL)
 	{
-		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, this->program_args, -1);
-		gtk_entry_set_text (GTK_ENTRY (GTK_BIN (params)->child), this->program_args);
+		/* Up to date, start debugger */
+		load_target (this, this->build_target);
 	}
-	g_object_unref (model);
+	
+	g_free (this->build_target);
+	this->build_target = NULL;
+}
 
-	/* Fill target combo box */
-	model = GTK_TREE_MODEL (gtk_list_store_new (1, GTK_TYPE_STRING));
-	gtk_combo_box_set_model (target, model);
-	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY(target), 0);
-
-    anjuta_shell_get_value (this->plugin->shell, "project_root_uri", &value, NULL);
-	project_root_uri = G_VALUE_HOLDS_STRING (&value) ? g_value_get_string (&value) : NULL;
-	if (project_root_uri != NULL)
+static void
+on_is_built_finished (GObject *builder, GError *err, gpointer user_data)
+{
+	DmaStart *this = (DmaStart *)user_data;
+	
+	if (err == NULL)
 	{
-		/* One project loaded, get all executable target */
-		IAnjutaProjectManager *pm;
-		GList *exec_targets = NULL;
-			
-		pm = anjuta_shell_get_interface (this->plugin->shell,
-										 IAnjutaProjectManager, NULL);
-		if (pm != NULL)
-		{
-			exec_targets = ianjuta_project_manager_get_targets (pm,
-							 IANJUTA_PROJECT_MANAGER_TARGET_EXECUTABLE,
-							 NULL);
-		}
-		if (exec_targets != NULL)
-		{
-			GList *node;
+		/* Up to date, start debugger */
+		load_target (this, this->build_target);
+	}
+	else if (err->code == IANJUTA_BUILDER_FAILED)
+	{
+		/* Target is not up to date */
+		this->build_handle = ianjuta_builder_build (IANJUTA_BUILDER (builder), this->build_target, on_build_finished, this, NULL);
+		return;
+	}
+	
+	g_free (this->build_target);
+	this->build_target = NULL;
+}
 
-			for (node = exec_targets; node; node = g_list_next (node))
+static gboolean
+check_target (DmaStart *this, const gchar *target)
+{
+	IAnjutaBuilder *builder;
+
+	builder = anjuta_shell_get_interface (this->plugin->shell, IAnjutaBuilder, NULL);
+	if (builder != NULL)
+	{
+		if (this->build_target)
+		{
+			/* a build operation is currently running */
+			if (strcmp (this->build_target, target) == 0)
 			{
-				if ((this->target_uri == NULL) || (strcmp(this->target_uri, node->data) != 0))
-				{
-					gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-					gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, node->data, -1);
-				}
-				g_free (node->data);
-			}
-			g_list_free (exec_targets);
-		}
-	}
-	g_object_unref (model);
-
-	if (this->target_uri != NULL)
-	{
-		gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
-		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, this->target_uri, -1);
-		gtk_entry_set_text (GTK_ENTRY (GTK_BIN (target)->child), this->target_uri);
-	}
-	else if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter))
-	{
-		gchar *txt;
-		
-		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0, &txt, -1);
-		gtk_entry_set_text (GTK_ENTRY (GTK_BIN (target)->child), txt);
-		g_free (txt);
-	}
-	
-	/* Set terminal option */	
-	if (this->run_in_terminal) gtk_toggle_button_set_active (term, TRUE);
-	if (this->stop_at_beginning) gtk_toggle_button_set_active (stop_at_beginning, TRUE);
-	
-	gtk_window_set_transient_for (GTK_WINDOW (dlg), parent);
-	
-	/* Run dialog */
-	for (;;)
-	{
-		response = gtk_dialog_run (GTK_DIALOG (dlg));
-		switch (response)
-		{
-		case GTK_RESPONSE_OK:
-		case GTK_RESPONSE_APPLY:
-		{
-			const gchar *arg;
-			const gchar *uri;
-			
-			if (this->program_args != NULL) g_free (this->program_args);
-			arg = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (params)->child));
-			this->program_args = g_strdup (arg);
-			
-			uri = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (target)->child));
-			if (this->target_uri != NULL)
-			{
-				if (strcmp(uri, this->target_uri) != 0)
-				{
-					g_free (this->target_uri);
-					this->target_uri = g_strdup (uri);
-				}
+				/* It is the same one, just ignore */
+				return TRUE;
 			}
 			else
 			{
-				this->target_uri = g_strdup (uri);
+				/* Cancel old operation */
+				ianjuta_builder_cancel (builder, this->build_handle, NULL);
 			}
-			
-			this->run_in_terminal = gtk_toggle_button_get_active (term);
-			this->stop_at_beginning = gtk_toggle_button_get_active (stop_at_beginning);
-			break;
 		}
-		case ANJUTA_RESPONSE_SELECT_TARGET:
-		{
-			GtkWidget *sel_dlg = gtk_file_chooser_dialog_new (
-				_("Load Target to debug"), GTK_WINDOW (dlg),
-				 GTK_FILE_CHOOSER_ACTION_OPEN,
-				 GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-				 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
-			gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(sel_dlg), FALSE);
-			gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (sel_dlg), TRUE);
+		
+		this->build_target = g_strdup (target);
 
-			GtkFileFilter *filter = gtk_file_filter_new ();
-			gtk_file_filter_set_name (filter, _("All files"));
-			gtk_file_filter_add_pattern (filter, "*");
-			gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (sel_dlg), filter);
-
-			//gtk_window_set_transient_for (GTK_WINDOW (dlg), parent);
-			
-			response = gtk_dialog_run (GTK_DIALOG (sel_dlg));
-
-			if (response == GTK_RESPONSE_OK)
-			{
-				gchar* uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (sel_dlg));
-				
-				if (this->target_uri != NULL)
-				{
-					if (strcmp(uri, this->target_uri) != 0)
-					{
-						g_free (this->target_uri);
-						this->target_uri = uri;
-					}
-					else
-					{
-						g_free (uri);
-					}
-				}
-				else
-				{
-					this->target_uri = uri;
-				}
-					
-				gtk_entry_set_text (GTK_ENTRY (GTK_BIN (target)->child), this->target_uri);
-
-			}
-			gtk_widget_destroy (GTK_WIDGET (sel_dlg));
-
-			continue;
-		}
-		default:
-			break;
-		}
-		break;
+		/* Check if target is up to date */
+		this->build_handle = ianjuta_builder_is_built (builder, target, on_is_built_finished, this, NULL);
+		return this->build_handle != 0;
 	}
-	gtk_widget_destroy (dlg);
+	else
+	{
+		/* Unable to build target, just launch debugger */
+		return load_target (this, target);
+	}	
+}
+
+static gboolean
+dma_start_load_and_start_uri (DmaStart *this, const gchar *target)
+{
+	GnomeVFSURI *vfs_uri;
+
+	if (!dma_quit_debugger (this)) return FALSE;
 	
-	return response == GTK_RESPONSE_OK;
+	vfs_uri = gnome_vfs_uri_new (target);
+	g_return_val_if_fail (vfs_uri != NULL, TRUE);
+	if (!gnome_vfs_uri_is_local (vfs_uri)) return FALSE;
+	gnome_vfs_uri_unref (vfs_uri);
+	
+	return check_target (this, target);
 }
 
 /* Add source dialog
  *---------------------------------------------------------------------------*/
 
+static void
+on_add_uri_in_model (gpointer data, gpointer user_data)
+{
+	GtkListStore* model = (GtkListStore *)user_data;
+	GtkTreeIter iter;
+	gchar *local;
+	
+	local = gnome_vfs_get_local_path_from_uri ((const char *)data);
+	gtk_list_store_append (model, &iter);
+	gtk_list_store_set (model, &iter, 0, local, -1);
+	g_free (local);
+}
+
 static gboolean
 on_add_source_in_list (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
 {
@@ -1093,19 +994,6 @@ on_add_source_in_list (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter
 	return FALSE;
 }
 
-static void
-on_add_source_in_model (gpointer data, gpointer user_data)
-{
-	GtkListStore* model = (GtkListStore *)user_data;
-	GtkTreeIter iter;
-	gchar *local;
-	
-	local = gnome_vfs_get_local_path_from_uri ((const char *)data);
-	gtk_list_store_append (model, &iter);
-	gtk_list_store_set (model, &iter, 0, local, -1);
-	g_free (local);
-}
-
 static void
 on_source_add_button (GtkButton *button, AddSourceDialog *dlg)
 {
@@ -1220,7 +1108,7 @@ add_source_show (DmaStart *this)
 	gtk_window_set_transient_for (GTK_WINDOW (widget), parent);
 
 	/* Initialize source path list */
-	g_list_foreach (this->source_dirs, on_add_source_in_model, dlg.model);
+	g_list_foreach (this->source_dirs, on_add_uri_in_model, dlg.model);
 	
 	/* Run dialog */
 	for (;;)
@@ -1239,7 +1127,7 @@ add_source_show (DmaStart *this)
 			break;
 		case GTK_RESPONSE_CANCEL:
 			gtk_list_store_clear (dlg.model);
-			g_list_foreach (this->source_dirs, on_add_source_in_model, dlg.model);
+			g_list_foreach (this->source_dirs, on_add_uri_in_model, dlg.model);
 			continue;
 		default:
 			break;
@@ -1304,22 +1192,23 @@ dma_attach_to_process (DmaStart* this)
 gboolean
 dma_run_target (DmaStart *this)
 {
-	if (dma_set_parameters (this) == TRUE)
-	{       
-		dma_start_load_uri (this);
-		dma_queue_start (this->debugger, this->program_args == NULL ? "" : this->program_args, this->run_in_terminal, this->stop_at_beginning);
+	gchar *target;
+
+	anjuta_shell_get (ANJUTA_PLUGIN (this->plugin)->shell,
+	 				  RUN_PROGRAM_URI, G_TYPE_STRING, &target, NULL);	
+	
+	if (target == NULL)
+	{
+		/* Launch parameter dialog to get a target name */
+		show_parameters_dialog (this);
+		anjuta_shell_get (ANJUTA_PLUGIN (this->plugin)->shell,
+	 				  RUN_PROGRAM_URI, G_TYPE_STRING, &target, NULL);
+		/* No target set by user */
+		if (target == NULL) return FALSE;
 	}
 	
-	return this->target_uri != NULL;
-}
-
-gboolean
-dma_rerun_target (DmaStart *this)
-{
-	if (this->target_uri == NULL) return FALSE;
-
-	dma_start_load_uri (this);
-	dma_queue_start (this->debugger, this->program_args == NULL ? "" : this->program_args, this->run_in_terminal, this->stop_at_beginning);
+	if (!dma_start_load_and_start_uri (this, target)) return FALSE;
+	g_free (target);
 	
 	return TRUE;
 }
@@ -1337,6 +1226,7 @@ dma_start_new (DebugManagerPlugin *plugin)
 	self->plugin = ANJUTA_PLUGIN (plugin);
 	self->debugger = dma_debug_manager_get_queue (plugin);
 	self->source_dirs = NULL;
+	self->build_target = NULL;
 	
 	g_signal_connect (self->plugin->shell, "save-session",
 					  G_CALLBACK (on_session_save), self);
@@ -1351,13 +1241,8 @@ dma_start_free (DmaStart *this)
 {
 	g_signal_handlers_disconnect_by_func (this->plugin->shell, G_CALLBACK (on_session_save), this);
     g_signal_handlers_disconnect_by_func (this->plugin->shell, G_CALLBACK (on_session_load), this);
-	if (this->source_dirs != NULL)
-	{
-		g_list_foreach (this->source_dirs, (GFunc)g_free, NULL);
-		g_list_free (this->source_dirs);
-	}
-	if (this->program_args != NULL) g_free (this->program_args);
-	if (this->target_uri != NULL) g_free (this->target_uri);
+	g_list_foreach (this->source_dirs, (GFunc)g_free, NULL);
+	g_list_free (this->source_dirs);
 	g_free (this);
 }
 
