@@ -23,7 +23,6 @@
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/anjuta-encodings.h>
-#include <libegg/menu/egg-entry-action.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
@@ -37,7 +36,7 @@
 #include "style-editor.h"
 #include "text_editor.h"
 
-#define PREFS_GLADE PACKAGE_DATA_DIR"/glade/editor.glade"
+#define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-editor-scintilla.glade"
 #define ICON_FILE "anjuta-editor-scintilla-plugin-48.png"
 
 gpointer parent_class;
@@ -67,7 +66,14 @@ dispose (GObject *obj)
 {
 	/* EditorPlugin *eplugin = ANJUTA_PLUGIN_EDITOR (obj); */
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
+}
+
+static void
+finalize (GObject *obj)
+{
+	/* Finalization codes here */
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -86,6 +92,7 @@ editor_plugin_class_init (GObjectClass *klass)
 	plugin_class->activate = activate_plugin;
 	plugin_class->deactivate = deactivate_plugin;
 	klass->dispose = dispose;
+	klass->finalize = finalize;
 }
 
 static IAnjutaEditor*
