@@ -33,6 +33,7 @@
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-encodings.h>
+#include <libanjuta/anjuta-convert.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
@@ -46,6 +47,7 @@
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
 #include <libanjuta/interfaces/ianjuta-editor-assist.h>
 #include <libanjuta/interfaces/ianjuta-editor-search.h>
+#include <libanjuta/interfaces/ianjuta-editor-hover.h>
 #include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-editor-factory.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
@@ -58,7 +60,6 @@
 #include "properties.h"
 #include "text_editor.h"
 #include "text_editor_cbs.h"
-#include "text_editor_menu.h"
 #include "text-editor-iterable.h"
 #include "print.h"
 
@@ -128,6 +129,7 @@ text_editor_instance_init (TextEditor *te)
 	te->first_time_expose = TRUE;
 	te->encoding = NULL;
 	te->gconf_notify_ids = NULL;
+	te->hover_tip_on = FALSE;
 }
 
 static void
@@ -537,7 +539,7 @@ text_editor_dispose (GObject *obj)
 		text_editor_prefs_finalize (te);
 		te->gconf_notify_ids = NULL;
 	}
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (G_OBJECT(te)));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 void
@@ -546,11 +548,10 @@ text_editor_finalize (GObject *obj)
 	TextEditor *te = TEXT_EDITOR (obj);
 	g_free (te->filename);
 	g_free (te->uri);
-	g_free (te->encoding);
 	g_free (te->force_hilite);
 	g_free (te->last_saved_content);
 	
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (G_OBJECT(te)));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 void
@@ -795,6 +796,17 @@ text_editor_get_current_lineno (TextEditor * te)
 	return linenum_scintilla_to_text_editor(count);
 }
 
+guint
+text_editor_get_position_lineno (TextEditor * te, gint position)
+{
+	guint count;
+	g_return_val_if_fail (te != NULL, 0);
+
+	count =	scintilla_send_message (SCINTILLA (te->scintilla),
+					SCI_LINEFROMPOSITION, position, 0);
+	return linenum_scintilla_to_text_editor(count);
+}
+
 guint
 text_editor_get_current_column (TextEditor * te)
 {
@@ -842,8 +854,7 @@ text_editor_goto_point (TextEditor * te, glong point)
 	g_return_val_if_fail (te != NULL, FALSE);
 	g_return_val_if_fail(IS_SCINTILLA (te->scintilla) == TRUE, FALSE);
 
-	scintilla_send_message (SCINTILLA (te->scintilla), SCI_GOTOPOS,
-				point, 0);
+	scintilla_send_message (SCINTILLA (te->scintilla), SCI_GOTOPOS, point, 0);
 	return TRUE;
 }
 
@@ -1200,112 +1211,36 @@ determine_editor_mode(gchar* buffer, glong size)
 	return mode;
 }
 
-static gchar *
-convert_to_utf8_from_charset (const gchar *content,
-							  gsize len,
-							  const gchar *charset)
-{
-	gchar *utf8_content = NULL;
-	GError *conv_error = NULL;
-	gchar* converted_contents = NULL;
-	gsize bytes_written;
-
-	g_return_val_if_fail (content != NULL, NULL);
-
-	/* DEBUG_PRINT ("Trying to convert from %s to UTF-8", charset); */
-
-	converted_contents = g_convert (content, len, "UTF-8",
-									charset, NULL, &bytes_written,
-									&conv_error); 
-						
-	if ((conv_error != NULL) || (converted_contents == NULL)  ||
-	    !g_utf8_validate (converted_contents, bytes_written, NULL))		
-	{
-		/* DEBUG_PRINT ("Couldn't convert from %s to UTF-8.", charset); */
-		
-		if (converted_contents != NULL)
-			g_free (converted_contents);
-
-		if (conv_error != NULL)
-		{
-			g_error_free (conv_error);
-			conv_error = NULL;
-		}
-
-		utf8_content = NULL;
-	} else {
-		/* DEBUG_PRINT ("Converted from %s to UTF-8.", charset); */
-		utf8_content = converted_contents;
-	}
-	return utf8_content;
-}
-
 static gchar *
 convert_to_utf8 (PropsID props, const gchar *content, gsize len,
-			     gchar **encoding_used)
+			     const AnjutaEncoding **encoding_used)
 {
-	GList *encodings = NULL;
-	GList *start;
-	const gchar *locale_charset;
-	GList *encoding_strings;
-	
-	g_return_val_if_fail (!g_utf8_validate (content, len, NULL), 
-			g_strndup (content, len < 0 ? strlen (content) : len));
+	GError* conv_error = NULL;
+	gchar* new_content;
+	gsize new_len;
 
-	encoding_strings = sci_prop_glist_from_data (props, SUPPORTED_ENCODINGS);
-	encodings = anjuta_encoding_get_encodings (encoding_strings);
-	anjuta_util_glist_strings_free (encoding_strings);
-	
-	if (g_get_charset (&locale_charset) == FALSE) 
+	new_content = anjuta_convert_to_utf8 (content, 
+										  len, 
+										  encoding_used,
+										  &new_len, 
+										  &conv_error);
+	if  (new_content == NULL)	
 	{
-		const AnjutaEncoding *locale_encoding;
-
-		/* not using a UTF-8 locale, so try converting
-		 * from that first */
-		if (locale_charset != NULL)
-		{
-			locale_encoding = anjuta_encoding_get_from_charset (locale_charset);
-			encodings = g_list_prepend (encodings,
-						(gpointer) locale_encoding);
-			/* DEBUG_PRINT ("Current charset = %s", locale_charset); */
-		}
+		/* Last change, let's try 8859-15 */
+		*encoding_used =  
+			anjuta_encoding_get_from_charset("ISO-8859-15");
+			
+		new_content = anjuta_convert_to_utf8 (content,
+											  len,
+											  encoding_used,
+											  &new_len,
+											  &conv_error);
 	}
-
-	start = encodings;
-
-	while (encodings != NULL) 
-	{
-		AnjutaEncoding *enc;
-		const gchar *charset;
-		gchar *utf8_content;
-
-		enc = (AnjutaEncoding *) encodings->data;
-
-		charset = anjuta_encoding_get_charset (enc);
-
-		/* DEBUG_PRINT ("Trying to convert %d bytes of data into UTF-8.", len); */
-		
-		fflush (stdout);
-		utf8_content = convert_to_utf8_from_charset (content, len, charset);
-
-		if (utf8_content != NULL) {
-			if (encoding_used != NULL)
-			{
-				if (*encoding_used != NULL)
-					g_free (*encoding_used);
-				
-				*encoding_used = g_strdup (charset);
-			}
-
-			return utf8_content;
-		}
-
-		encodings = encodings->next;
-	}
-
-	g_list_free (start);
 	
-	return NULL;
+	if (conv_error)
+		g_error_free (conv_error);
+	
+	return new_content;
 }
 
 static gboolean
@@ -1323,7 +1258,10 @@ load_from_file (TextEditor *te, gchar *uri, gchar **err)
 	scintilla_send_message (SCINTILLA (te->scintilla), SCI_CLEARALL,
 							0, 0);
 	vfs_uri = gnome_vfs_uri_new(uri);
-	result = gnome_vfs_get_file_info_uri(vfs_uri, &info, GNOME_VFS_FILE_INFO_DEFAULT);
+	result = gnome_vfs_get_file_info_uri (vfs_uri,
+										  &info,
+										  GNOME_VFS_FILE_INFO_DEFAULT |
+										  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 	if (result != GNOME_VFS_OK)
 	{
 		*err = g_strdup (_("Could not get file info"));
@@ -1378,8 +1316,6 @@ load_from_file (TextEditor *te, gchar *uri, gchar **err)
 	{
 		if (g_utf8_validate (buffer, nchars, NULL))
 		{
-			if (te->encoding)
-				g_free (te->encoding);
 			te->encoding = NULL;
 		}
 		else
@@ -1388,7 +1324,7 @@ load_from_file (TextEditor *te, gchar *uri, gchar **err)
 	
 			converted_text = convert_to_utf8 (te->props_base,
 											  buffer, nchars, &te->encoding);
-
+			
 			if (converted_text == NULL)
 			{
 				/* bail out */
@@ -1446,19 +1382,22 @@ save_to_file (TextEditor *te, gchar * uri)
 		
 		size = strlen (data);
 		
-		/* Save according to the correct encoding */
-		if (anjuta_preferences_get_int (te->preferences,
-										SAVE_ENCODING_CURRENT_LOCALE))
+
+		/* Save in original encoding */
+		if ((te->encoding != NULL))
 		{
-			/* Save in current locate */
 			GError *conv_error = NULL;
 			gchar* converted_file_contents = NULL;
-
-			/* DEBUG_PRINT ("Using current locale's encoding"); */
-
-			converted_file_contents = g_locale_from_utf8 (data, -1, NULL,
-														  NULL, &conv_error);
-	
+			gsize new_len;
+			
+			/* DEBUG_PRINT ("Using encoding %s", te->encoding); */
+			
+			/* Try to convert it from UTF-8 to original encoding */
+			converted_file_contents = anjuta_convert_from_utf8 (data, -1, 
+																te->encoding,
+																&new_len,
+																&conv_error); 
+			
 			if (conv_error != NULL)
 			{
 				/* Conversion error */
@@ -1473,40 +1412,9 @@ save_to_file (TextEditor *te, gchar * uri)
 		}
 		else
 		{
-			/* Save in original encoding */
-			if ((te->encoding != NULL) &&
-				anjuta_preferences_get_int (te->preferences,
-											SAVE_ENCODING_ORIGINAL))
-			{
-				GError *conv_error = NULL;
-				gchar* converted_file_contents = NULL;
-	
-				/* DEBUG_PRINT ("Using encoding %s", te->encoding); */
-
-				/* Try to convert it from UTF-8 to original encoding */
-				converted_file_contents = g_convert (data, -1, 
-													 te->encoding,
-													 "UTF-8", NULL, NULL,
-													 &conv_error); 
-	
-				if (conv_error != NULL)
-				{
-					/* Conversion error */
-					g_error_free (conv_error);
-				}
-				else
-				{
-					g_free (data);
-					data = converted_file_contents;
-					size = strlen (converted_file_contents);
-				}
-			}
-			else
-			{
-				/* Save in utf-8 */
-				/* DEBUG_PRINT ("Using utf-8 encoding"); */
-			}				
-		}
+			/* Save in utf-8 */
+			/* DEBUG_PRINT ("Using utf-8 encoding"); */
+		}				
 		
 		/* Strip trailing spaces */
 		strip = anjuta_preferences_get_int (te->preferences,
@@ -1621,7 +1529,7 @@ text_editor_save_file (TextEditor * te, gboolean update)
 		
 		text_editor_thaw (te);
 		anjuta_util_dialog_error (parent,
-								  _("Could not save intermediate file %s: %s."),
+								  _("Could not save intermediate file %s: %s"),
 								  save_uri,
 								  gnome_vfs_result_to_string (result));
 		
@@ -1637,10 +1545,26 @@ text_editor_save_file (TextEditor * te, gboolean update)
 	{
 		GnomeVFSURI *src_uri;
 		GnomeVFSURI *dest_uri;
+		GnomeVFSFileInfo info;
+		gboolean have_info;
 		
 		src_uri = gnome_vfs_uri_new (save_uri);
 		dest_uri = gnome_vfs_uri_new (te->uri);
-		
+	
+		/* have_info is FALSE on newly created file */	
+		have_info = gnome_vfs_get_file_info_uri (dest_uri,
+											  &info,
+											  GNOME_VFS_FILE_INFO_DEFAULT |
+											  GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == GNOME_VFS_OK;
+		if (have_info)
+		{
+			if (info.flags & GNOME_VFS_FILE_FLAGS_SYMLINK &&
+				info.valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME)
+			{
+				gnome_vfs_uri_unref (dest_uri);
+				dest_uri = gnome_vfs_uri_new (info.symlink_name);
+			}
+		}
 		/* Move 'file~' to 'file' */
 		result = gnome_vfs_xfer_uri (src_uri, dest_uri,
 									 GNOME_VFS_XFER_DELETE_ITEMS |
@@ -1653,14 +1577,26 @@ text_editor_save_file (TextEditor * te, gboolean update)
 		if (result != GNOME_VFS_OK)
 		{
 			anjuta_util_dialog_error (parent,
-						  _("Could not save file %s: %s."),
-						  te->uri,
-						  gnome_vfs_result_to_string (result));
+				_("Could not save file %s: %s."),
+				te->uri,
+				gnome_vfs_result_to_string (result));
 		}
 		else
 		{
+			if (have_info)
+			{
+				result = gnome_vfs_set_file_info_uri (dest_uri, &info,
+					GNOME_VFS_SET_FILE_INFO_PERMISSIONS);
+				if (result != GNOME_VFS_OK)
+				{
+					anjuta_util_dialog_warning (parent,
+						_("Could not set file permissions %s: %s."),
+						te->uri,
+						gnome_vfs_result_to_string (result));
+				}
+			}
 			scintilla_send_message (SCINTILLA (te->scintilla),
-						SCI_SETSAVEPOINT, 0, 0);
+									SCI_SETSAVEPOINT, 0, 0);
 			g_signal_emit_by_name (G_OBJECT (te), "saved", te->uri);
 			anjuta_status (te->status, _("File saved successfully"), 5);
 			ret = TRUE;
@@ -2101,6 +2037,31 @@ text_editor_can_redo (TextEditor *te)
 								   SCI_CANREDO, 0, 0);
 }
 
+void
+text_editor_show_hover_tip (TextEditor *te, gint position, const gchar *info)
+{
+	text_editor_hide_hover_tip (te);
+	if (!te->hover_tip_on)
+	{
+		scintilla_send_message (SCINTILLA (te->scintilla), SCI_CALLTIPSHOW,
+								position, (long)info);
+		scintilla_send_message (SCINTILLA (te->scintilla), SCI_CALLTIPSETHLT,
+								strlen (info), 0);
+		te->hover_tip_on = TRUE;
+	}
+}
+
+void
+text_editor_hide_hover_tip (TextEditor *te)
+{
+	if (te->hover_tip_on)
+	{
+		scintilla_send_message (SCINTILLA (te->scintilla),
+								SCI_CALLTIPCANCEL, 0, 0);
+		te->hover_tip_on = FALSE;
+	}
+}
+
 void
 text_editor_command (TextEditor *te, gint command, glong wparam, glong lparam)
 {
@@ -2173,46 +2134,69 @@ itext_editor_goto_line (IAnjutaEditor *editor, gint lineno, GError **e)
 }
 
 static void
-itext_editor_goto_position (IAnjutaEditor *editor, gint position, GError **e)
+itext_editor_goto_start (IAnjutaEditor *editor, GError **e)
 {
-	text_editor_goto_point (TEXT_EDITOR (editor), position);
+	text_editor_goto_point (TEXT_EDITOR (editor), 0);
+}
+
+static void
+itext_editor_goto_end (IAnjutaEditor *editor, GError **e)
+{
+	text_editor_goto_point (TEXT_EDITOR (editor), -1);
+}
+
+static void
+itext_editor_goto_position (IAnjutaEditor *editor, IAnjutaIterable *position,
+							GError **e)
+{
+	text_editor_goto_point (TEXT_EDITOR (editor),
+							text_editor_cell_get_position (TEXT_EDITOR_CELL
+														   (position)));
 }
 
 static gchar*
-itext_editor_get_text (IAnjutaEditor *editor, gint position, gint length,
-					   GError **e)
+itext_editor_get_text_all (IAnjutaEditor *editor, GError **e)
+{
+	TextEditor *te = TEXT_EDITOR (editor);
+	gint length = scintilla_send_message (SCINTILLA (te->scintilla),
+										  SCI_GETLENGTH, 0, 0);
+	if (length > 0)
+		return (gchar *) aneditor_command (te->editor_id, ANE_GETTEXTRANGE,
+										   0, length);
+	else
+		return NULL;
+}
+
+static gchar*
+itext_editor_get_text (IAnjutaEditor *editor, IAnjutaIterable* begin,
+					   IAnjutaIterable* end, GError **e)
 {
 	gchar *data;
-	gint end;
+	gint start_pos = text_editor_cell_get_position (TEXT_EDITOR_CELL (begin));
+	gint end_pos = text_editor_cell_get_position (TEXT_EDITOR_CELL (end));
 	TextEditor *te = TEXT_EDITOR (editor);
-
-	g_return_val_if_fail (position >= 0, NULL);
-	if (length == 0)
-		return NULL;
-
-	if (length < 0)
-		end = scintilla_send_message (SCINTILLA (te->scintilla),
-									  SCI_GETLENGTH, 0, 0);
-	else
-		end = position + length;
-
-	data =	(gchar *) aneditor_command (te->editor_id,
-										ANE_GETTEXTRANGE, position, end);
+	data =	(gchar *) aneditor_command (te->editor_id, ANE_GETTEXTRANGE,
+										  start_pos, end_pos);
 	return data;
 }
 
-static gint
-itext_editor_get_position (IAnjutaEditor *editor, GError **e)
+static IAnjutaIterable*
+itext_editor_get_position (IAnjutaEditor* editor, GError **e)
 {
-	return text_editor_get_current_position (TEXT_EDITOR(editor));
+	TextEditor *te = TEXT_EDITOR (editor);
+	gint position = text_editor_get_current_position (te);
+	TextEditorCell *position_iter = text_editor_cell_new (te, position);
+	return IANJUTA_ITERABLE (position_iter);
 }
 
-static IAnjutaIterable*
-itext_editor_get_position_iter (IAnjutaEditor* editor, GError **e)
+static gint
+itext_editor_get_offset (IAnjutaEditor *editor, GError **e)
 {
-	TextEditor* te = TEXT_EDITOR (editor);
-	return IANJUTA_ITERABLE (text_editor_cell_new (te,
-												   text_editor_get_current_position (te)));
+	gint pos;
+	IAnjutaIterable *iter = itext_editor_get_position (editor, NULL);
+	pos = ianjuta_iterable_get_position (iter, NULL);
+	g_object_unref (iter);
+	return pos;
 }
 
 static gint
@@ -2224,8 +2208,22 @@ itext_editor_get_lineno (IAnjutaEditor *editor, GError **e)
 static gint
 itext_editor_get_length (IAnjutaEditor *editor, GError **e)
 {
-	return aneditor_command (TEXT_EDITOR (editor)->editor_id,
-							 ANE_GETLENGTH, 0, 0);
+	TextEditor *te = TEXT_EDITOR (editor);
+	
+	/* FIXME: Find a more optimal solution */
+	gint length = scintilla_send_message (SCINTILLA (te->scintilla),
+										  SCI_GETLENGTH, 0, 0);
+	if (length > 0)
+	{
+		gint char_position;
+		gchar *data =
+			(gchar *) aneditor_command (te->editor_id, ANE_GETTEXTRANGE, 0,
+										length);
+		char_position = g_utf8_strlen (data, -1);
+		g_free (data);
+		return char_position;
+	}
+	return 0;
 }
 
 static gchar*
@@ -2242,8 +2240,8 @@ itext_editor_get_current_word (IAnjutaEditor *editor, GError **e)
 }
 
 static void
-itext_editor_insert (IAnjutaEditor *editor, gint pos, const gchar *txt,
-					 gint length, GError **e)
+itext_editor_insert (IAnjutaEditor *editor, IAnjutaIterable *position,
+					 const gchar *txt, gint length, GError **e)
 {
 	gchar *text_to_insert;
 	if (length >= 0)
@@ -2252,7 +2250,9 @@ itext_editor_insert (IAnjutaEditor *editor, gint pos, const gchar *txt,
 		text_to_insert = g_strdup (txt);
 	
 	aneditor_command (TEXT_EDITOR(editor)->editor_id, ANE_INSERTTEXT,
-					  pos, (long)text_to_insert);
+					  text_editor_cell_get_position
+					  (TEXT_EDITOR_CELL (position)),
+					  (long)text_to_insert);
 	g_free (text_to_insert);
 }
 
@@ -2273,87 +2273,128 @@ itext_editor_append (IAnjutaEditor *editor, const gchar *txt,
 }
 
 static void
-itext_editor_erase (IAnjutaEditor *editor, gint position, gint length,
-					GError **e)
+itext_editor_erase (IAnjutaEditor *editor,
+					IAnjutaIterable *position_start,
+					IAnjutaIterable *position_end, GError **e)
 {
-	gint end;
-
-	g_return_if_fail (position >= 0);
-	if (length == 0)
+	gint start, end;
+	
+	/* If both positions are NULL, erase all */
+	if (position_start == NULL && position_end == NULL)
+	{
+		scintilla_send_message (SCINTILLA (TEXT_EDITOR (editor)->scintilla), 
+								SCI_CLEARALL,
+								0, 0);
 		return;
-
-	if (length < 0)
+	}
+	
+	/* Determine correct start and end byte positions */
+	if (position_start)
+		start = text_editor_cell_get_position (TEXT_EDITOR_CELL (position_start));
+	else
+		start = 0;
+	
+	if (position_end)
+		end = text_editor_cell_get_position (TEXT_EDITOR_CELL (position_end));
+	else
 		end = scintilla_send_message (SCINTILLA (TEXT_EDITOR (editor)->scintilla),
 									  SCI_GETLENGTH, 0, 0);
-	else
-		end = position + length;
-
-	ianjuta_editor_selection_set (IANJUTA_EDITOR_SELECTION (editor) , position,
-								  end, FALSE, NULL);
-	text_editor_replace_selection (TEXT_EDITOR (editor), "");
+	if (start != end)
+	{
+		scintilla_send_message (SCINTILLA(TEXT_EDITOR (editor)->scintilla),
+								SCI_SETSEL, start, end);
+		text_editor_replace_selection (TEXT_EDITOR (editor), "");
+	}
 }
 
 static void
 itext_editor_erase_all (IAnjutaEditor *editor, GError **e)
 {
-	scintilla_send_message (SCINTILLA (TEXT_EDITOR (editor)->scintilla), SCI_CLEARALL,
+	scintilla_send_message (SCINTILLA (TEXT_EDITOR (editor)->scintilla), 
+							SCI_CLEARALL,
 							0, 0);
 }
 
 static int
-itext_editor_get_column(IAnjutaEditor *editor, GError **e)
+itext_editor_get_column (IAnjutaEditor *editor, GError **e)
 {
-	return text_editor_get_current_column(TEXT_EDITOR(editor));
+	return text_editor_get_current_column (TEXT_EDITOR(editor));
 }
 
 static gboolean
-itext_editor_get_overwrite(IAnjutaEditor *editor, GError **e)
+itext_editor_get_overwrite (IAnjutaEditor *editor, GError **e)
 {
-	return text_editor_get_overwrite(TEXT_EDITOR(editor));
+	return text_editor_get_overwrite (TEXT_EDITOR (editor));
 }
 
 static void
-itext_editor_set_popup_menu(IAnjutaEditor *editor, GtkWidget* menu, GError **e)
+itext_editor_set_popup_menu (IAnjutaEditor *editor, GtkWidget* menu, GError **e)
 {
-	text_editor_set_popup_menu(TEXT_EDITOR(editor), menu);
+	text_editor_set_popup_menu (TEXT_EDITOR (editor), menu);
 }
 
 static gint
-itext_editor_get_line_from_position (IAnjutaEditor *editor, gint pos, GError **e)
+itext_editor_get_line_from_position (IAnjutaEditor *editor,
+									 IAnjutaIterable *position, GError **e)
 {
-	return text_editor_get_line_from_position (TEXT_EDITOR (editor), pos);
-}
-
-static gint
-itext_editor_get_line_begin_position (IAnjutaEditor *editor, gint line,
-									  GError **e)
-{
-	gint ln;
-	
-	g_return_val_if_fail (line > 0, -1);
-	
-	ln = linenum_text_editor_to_scintilla (line);
-	return scintilla_send_message (SCINTILLA (TEXT_EDITOR (editor)->scintilla),
-								   SCI_POSITIONFROMLINE, ln, 0);
-}
-
-static gint
-itext_editor_get_line_end_position (IAnjutaEditor *editor, gint line,
-									  GError **e)
-{
-	gint ln;
-	
-	g_return_val_if_fail (line > 0, -1);
-	
-	ln = linenum_text_editor_to_scintilla (line);
-	return scintilla_send_message (SCINTILLA (TEXT_EDITOR (editor)->scintilla),
-								   SCI_GETLINEENDPOSITION, ln, 0);
+	return text_editor_get_line_from_position (TEXT_EDITOR (editor),
+				text_editor_cell_get_position (TEXT_EDITOR_CELL (position)));
 }
 
 static IAnjutaIterable*
-itext_editor_get_cell_iter (IAnjutaEditor *editor, gint position, GError **e)
+itext_editor_get_line_begin_position (IAnjutaEditor *editor, gint line,
+									  GError **e)
 {
-	TextEditorCell *editor_cell = text_editor_cell_new (TEXT_EDITOR (editor), position);
+	gint ln, byte_pos;
+	TextEditor *te;
+	
+	g_return_val_if_fail (line > 0, NULL);
+	te = TEXT_EDITOR (editor);
+	ln = linenum_text_editor_to_scintilla (line);
+	byte_pos = scintilla_send_message (SCINTILLA (te->scintilla),
+								   SCI_POSITIONFROMLINE, ln, 0);
+	return IANJUTA_ITERABLE (text_editor_cell_new (te, byte_pos));
+}
+
+static IAnjutaIterable*
+itext_editor_get_line_end_position (IAnjutaEditor *editor, gint line,
+									  GError **e)
+{
+	gint ln, byte_pos;
+	TextEditor *te;
+	
+	g_return_val_if_fail (line > 0, NULL);
+	te = TEXT_EDITOR (editor);
+	
+	ln = linenum_text_editor_to_scintilla (line);
+	byte_pos = scintilla_send_message (SCINTILLA (te->scintilla),
+								   SCI_GETLINEENDPOSITION, ln, 0);
+	return IANJUTA_ITERABLE (text_editor_cell_new (te, byte_pos));
+}
+
+static IAnjutaIterable*
+itext_editor_get_position_from_offset (IAnjutaEditor *editor, gint offset, GError **e)
+{
+	TextEditorCell *editor_cell = text_editor_cell_new (TEXT_EDITOR (editor), 0);
+	/* Set to the right utf8 character offset */
+	ianjuta_iterable_set_position (IANJUTA_ITERABLE (editor_cell), offset, NULL);
+	return IANJUTA_ITERABLE (editor_cell);
+}
+
+static IAnjutaIterable*
+itext_editor_get_start_position (IAnjutaEditor *editor, GError **e)
+{
+	TextEditorCell *editor_cell = text_editor_cell_new (TEXT_EDITOR (editor), 0);
+	return IANJUTA_ITERABLE (editor_cell);
+}
+
+static IAnjutaIterable*
+itext_editor_get_end_position (IAnjutaEditor *editor, GError **e)
+{
+	gint length = scintilla_send_message (SCINTILLA (TEXT_EDITOR (editor)->scintilla),
+										  SCI_GETLENGTH, 0, 0);
+	TextEditorCell *editor_cell = text_editor_cell_new (TEXT_EDITOR (editor),
+														length);
 	return IANJUTA_ITERABLE (editor_cell);
 }
 
@@ -2366,10 +2407,13 @@ itext_editor_iface_init (IAnjutaEditorIface *iface)
 	iface->set_use_spaces = itext_editor_set_use_spaces;
 	iface->set_auto_indent = itext_editor_set_auto_indent;	
 	iface->goto_line = itext_editor_goto_line;
+	iface->goto_start = itext_editor_goto_start;
+	iface->goto_end = itext_editor_goto_end;
 	iface->goto_position = itext_editor_goto_position;
+	iface->get_text_all = itext_editor_get_text_all;
 	iface->get_text = itext_editor_get_text;
+	iface->get_offset = itext_editor_get_offset;
 	iface->get_position = itext_editor_get_position;
-	iface->get_position_iter = itext_editor_get_position_iter;
 	iface->get_lineno = itext_editor_get_lineno;
 	iface->get_length = itext_editor_get_length;
 	iface->get_current_word = itext_editor_get_current_word;
@@ -2383,7 +2427,9 @@ itext_editor_iface_init (IAnjutaEditorIface *iface)
 	iface->get_line_from_position = itext_editor_get_line_from_position;
 	iface->get_line_begin_position = itext_editor_get_line_begin_position;
 	iface->get_line_end_position = itext_editor_get_line_end_position;
-	iface->get_cell_iter = itext_editor_get_cell_iter;
+	iface->get_position_from_offset = itext_editor_get_position_from_offset;
+	iface->get_start_position = itext_editor_get_start_position;
+	iface->get_end_position = itext_editor_get_end_position;
 }
 
 static const gchar *
@@ -2486,28 +2532,18 @@ iselection_get (IAnjutaEditorSelection *editor, GError **error)
 }
 
 static void
-iselection_set (IAnjutaEditorSelection *editor, gint start, gint end,
-		gboolean backwards, GError **e)
-{
-    if (!backwards)
-	scintilla_send_message(SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-						   SCI_SETSEL, start, end);
-    else
-	scintilla_send_message(SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-						   SCI_SETSEL, end, start);
-}
-
-static void
-iselection_set_iter (IAnjutaEditorSelection* edit, 
-				  IAnjutaEditorCell* istart,
-				  IAnjutaEditorCell* iend,
-				  GError** e)
+iselection_set (IAnjutaEditorSelection* edit, 
+				IAnjutaIterable* istart,
+				IAnjutaIterable* iend,
+				GError** e)
 {
 	TextEditorCell* start = TEXT_EDITOR_CELL (istart);
 	TextEditorCell* end = TEXT_EDITOR_CELL (iend);
+	int start_pos = text_editor_cell_get_position (start);
+	int end_pos = text_editor_cell_get_position (end);
 	
-	iselection_set (edit, text_editor_cell_get_position (start),
-					text_editor_cell_get_position (end), FALSE, e);
+	scintilla_send_message (SCINTILLA (TEXT_EDITOR (edit)->scintilla),
+						    SCI_SETSEL, start_pos, end_pos);
 }
 
 static gboolean
@@ -2516,21 +2552,14 @@ iselection_has_selection (IAnjutaEditorSelection *editor, GError **e)
 	return text_editor_has_selection (TEXT_EDITOR (editor));
 }
 
-static gint
-iselection_get_start (IAnjutaEditorSelection *editor, GError **e)
-{
-	gint start =  scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-										  SCI_GETSELECTIONSTART, 0, 0);
-	gint end = scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-									   SCI_GETSELECTIONEND, 0, 0);
-	return (start != end)? start: -1;
-}
-
 static IAnjutaIterable*
-iselection_get_start_iter (IAnjutaEditorSelection *edit, GError **e)
+iselection_get_start (IAnjutaEditorSelection *edit, GError **e)
 {
-	gint start = iselection_get_start (edit, e);
-	if (start != -1)
+	gint start =  scintilla_send_message (SCINTILLA(TEXT_EDITOR(edit)->scintilla),
+										  SCI_GETSELECTIONSTART, 0, 0);
+	gint end = scintilla_send_message (SCINTILLA(TEXT_EDITOR(edit)->scintilla),
+									   SCI_GETSELECTIONEND, 0, 0);
+	if (start != end)
 	{
 		return IANJUTA_ITERABLE (text_editor_cell_new (TEXT_EDITOR (edit), start));
 	}
@@ -2538,21 +2567,14 @@ iselection_get_start_iter (IAnjutaEditorSelection *edit, GError **e)
 		return NULL;
 }
 
-static gint
-iselection_get_end (IAnjutaEditorSelection *editor, GError **e)
-{
-	gint start =  scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-										  SCI_GETSELECTIONSTART, 0, 0);
-	gint end = scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-									   SCI_GETSELECTIONEND, 0, 0);
-	return (start != end)? end: -1;
-}
-
 static IAnjutaIterable*
-iselection_get_end_iter (IAnjutaEditorSelection *edit, GError **e)
+iselection_get_end (IAnjutaEditorSelection *edit, GError **e)
 {
-	gint end = iselection_get_end (edit, e);
-	if (end != -1)
+	gint start =  scintilla_send_message (SCINTILLA(TEXT_EDITOR(edit)->scintilla),
+										  SCI_GETSELECTIONSTART, 0, 0);
+	gint end = scintilla_send_message (SCINTILLA(TEXT_EDITOR(edit)->scintilla),
+									   SCI_GETSELECTIONEND, 0, 0);
+	if (start != end)
 	{
 		return IANJUTA_ITERABLE (text_editor_cell_new (TEXT_EDITOR (edit), end));
 	}
@@ -2576,25 +2598,25 @@ iselection_replace (IAnjutaEditorSelection *editor, const gchar *txt,
 }
 
 static void
-iselection_select_all(IAnjutaEditorSelection* te, GError** ee)
+iselection_select_all (IAnjutaEditorSelection* te, GError** ee)
 {
-	text_editor_command(TEXT_EDITOR(te), ANE_SELECTALL, 0, 0);
+	text_editor_command (TEXT_EDITOR (te), ANE_SELECTALL, 0, 0);
 }
 
 static void
-iselection_select_to_brace(IAnjutaEditorSelection* te, GError** ee)
+iselection_select_to_brace (IAnjutaEditorSelection* te, GError** ee)
 {
-	text_editor_command(TEXT_EDITOR(te), ANE_SELECTTOBRACE, 0, 0);
+	text_editor_command (TEXT_EDITOR (te), ANE_SELECTTOBRACE, 0, 0);
 }
 
 static void
-iselection_select_block(IAnjutaEditorSelection *te, GError **e)
+iselection_select_block (IAnjutaEditorSelection *te, GError **e)
 {
-	text_editor_command(TEXT_EDITOR(te), ANE_SELECTBLOCK, 0, 0);
+	text_editor_command (TEXT_EDITOR(te), ANE_SELECTBLOCK, 0, 0);
 }
 
 static void
-iselection_select_function(IAnjutaEditorSelection *editor, GError **e)
+iselection_select_function (IAnjutaEditorSelection *editor, GError **e)
 {
 	TextEditor* te = TEXT_EDITOR(editor);
 	gint pos;
@@ -2604,34 +2626,34 @@ iselection_select_function(IAnjutaEditorSelection *editor, GError **e)
 	gint line_count;
 	gint tmp;
 
-	line_count = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                                    SCI_GETLINECOUNT, 0, 0);
-	pos = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                             SCI_GETCURRENTPOS, 0, 0);
-	line = scintilla_send_message(SCINTILLA(te->scintilla),
-	                              SCI_LINEFROMPOSITION, pos, 0);
+	line_count = scintilla_send_message (SCINTILLA (te->scintilla), 
+										 SCI_GETLINECOUNT, 0, 0);
+	pos = scintilla_send_message (SCINTILLA (te->scintilla), 
+								  SCI_GETCURRENTPOS, 0, 0);
+	line = scintilla_send_message (SCINTILLA (te->scintilla),
+								   SCI_LINEFROMPOSITION, pos, 0);
 
 	tmp = line + 1;	
-	fold_level = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                                    SCI_GETFOLDLEVEL, line, 0) ;	
+	fold_level = scintilla_send_message (SCINTILLA (te->scintilla), 
+										 SCI_GETFOLDLEVEL, line, 0) ;	
 	if ((fold_level & 0xFF) != 0)
 	{
-		while((fold_level & 0x10FF) != 0x1000 && line >= 0)
-			fold_level = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                                    SCI_GETFOLDLEVEL, --line, 0) ;
-		start = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                                    SCI_POSITIONFROMLINE, line + 1, 0);
+		while ((fold_level & 0x10FF) != 0x1000 && line >= 0)
+			fold_level = scintilla_send_message (SCINTILLA (te->scintilla), 
+												 SCI_GETFOLDLEVEL, --line, 0) ;
+		start = scintilla_send_message (SCINTILLA (te->scintilla), 
+										SCI_POSITIONFROMLINE, line + 1, 0);
 		line = tmp;
-		fold_level = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                                        SCI_GETFOLDLEVEL, line, 0) ;
-		while((fold_level & 0x10FF) != 0x1000 && line < line_count)
-			fold_level = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                                            SCI_GETFOLDLEVEL, ++line, 0) ;
+		fold_level = scintilla_send_message (SCINTILLA (te->scintilla), 
+											 SCI_GETFOLDLEVEL, line, 0) ;
+		while ((fold_level & 0x10FF) != 0x1000 && line < line_count)
+			fold_level = scintilla_send_message (SCINTILLA (te->scintilla), 
+												 SCI_GETFOLDLEVEL, ++line, 0) ;
 
-		end = scintilla_send_message(SCINTILLA(te->scintilla), 
-	                                 SCI_POSITIONFROMLINE, line , 0);
-		scintilla_send_message(SCINTILLA(te->scintilla), 
-	                           SCI_SETSEL, start, end) ;
+		end = scintilla_send_message (SCINTILLA (te->scintilla), 
+									  SCI_POSITIONFROMLINE, line , 0);
+		scintilla_send_message (SCINTILLA (te->scintilla), 
+								SCI_SETSEL, start, end) ;
 	}
 }
 
@@ -2641,11 +2663,8 @@ iselection_iface_init (IAnjutaEditorSelectionIface *iface)
 	iface->has_selection = iselection_has_selection;
 	iface->get = iselection_get;
 	iface->set = iselection_set;
-	iface->set_iter = iselection_set_iter;
 	iface->get_start = iselection_get_start;
-	iface->get_start_iter = iselection_get_start_iter;
 	iface->get_end = iselection_get_end;
-	iface->get_end_iter = iselection_get_end_iter;
 	iface->replace = iselection_replace;
 	iface->select_all = iselection_select_all;
 	iface->select_to_brace = iselection_select_to_brace;
@@ -2698,11 +2717,30 @@ isaveable_save (IAnjutaFileSavable* editor, GError** e)
 static void
 isavable_save_as (IAnjutaFileSavable* editor, const gchar* filename, GError** e)
 {
+	const gchar *past_language;
+	const gchar *curr_language;
 	TextEditor *text_editor = TEXT_EDITOR(editor);
-	text_editor->uri = g_strdup(filename);
+	
+	past_language =
+		ianjuta_editor_language_get_language (IANJUTA_EDITOR_LANGUAGE (text_editor),
+											  NULL);
+	
+	text_editor->uri = g_strdup (filename);
 	/* Remove path */
-	text_editor->filename = g_strdup(strrchr(filename, '/') + 1);
-	text_editor_save_file(text_editor, FALSE);
+	text_editor->filename = g_path_get_basename (filename);
+	text_editor_save_file (text_editor, FALSE);
+	text_editor_set_hilite_type (text_editor, NULL);
+	text_editor_hilite (text_editor, FALSE);
+	
+	/* We have to take care of 'language-change' signal ourself because
+	 * text_editor_set_hilite_type() only emits it for forced hilite type
+	 */
+	curr_language =
+		ianjuta_editor_language_get_language (IANJUTA_EDITOR_LANGUAGE (text_editor),
+											  NULL);
+	if (past_language != curr_language)
+		g_signal_emit_by_name (text_editor, "language-changed", curr_language);
+	
 }
 
 static gboolean
@@ -2836,21 +2874,27 @@ itext_editor_factory_iface_init (IAnjutaEditorFactoryIface *iface)
 /* IAnjutaEditorConvert implementation */
 
 static void
-iconvert_to_upper(IAnjutaEditorConvert* te, gint start_position,
-				  gint end_position, GError** ee)
+iconvert_to_upper (IAnjutaEditorConvert* te, IAnjutaIterable *start_position,
+				   IAnjutaIterable *end_position, GError** ee)
 {
+	gint start, end;
+	start = text_editor_cell_get_position (TEXT_EDITOR_CELL (start_position));
+	end = text_editor_cell_get_position (TEXT_EDITOR_CELL (end_position));
 	scintilla_send_message (SCINTILLA (TEXT_EDITOR(te)->scintilla),
-						   SCI_SETSEL, start_position, end_position);
+							SCI_SETSEL, start, end);
 	text_editor_command (TEXT_EDITOR(te), ANE_UPRCASE, 0, 0);
 }
 
 static void
-iconvert_to_lower(IAnjutaEditorConvert* te, gint start_position,
-				  gint end_position, GError** ee)
+iconvert_to_lower(IAnjutaEditorConvert* te, IAnjutaIterable *start_position,
+				  IAnjutaIterable *end_position, GError** ee)
 {
+	gint start, end;
+	start = text_editor_cell_get_position (TEXT_EDITOR_CELL (start_position));
+	end = text_editor_cell_get_position (TEXT_EDITOR_CELL (end_position));
 	scintilla_send_message (SCINTILLA (TEXT_EDITOR(te)->scintilla),
-						   SCI_SETSEL, start_position, end_position);
-	text_editor_command (TEXT_EDITOR(te), ANE_LWRCASE, 0, 0);	
+							SCI_SETSEL, start, end);
+	text_editor_command (TEXT_EDITOR (te), ANE_LWRCASE, 0, 0);	
 }
 
 static void
@@ -2963,7 +3007,7 @@ ilinemode_iface_init (IAnjutaEditorLineModeIface *iface)
 
 static void
 iassist_suggest (IAnjutaEditorAssist *iassist, GList* choices,
-				 gint position, int char_alignment, GError **err)
+				 IAnjutaIterable *position, int char_alignment, GError **err)
 {
 	GString *words;
 	GList *choice;
@@ -3023,7 +3067,7 @@ iassist_get_suggestions (IAnjutaEditorAssist *iassist, const gchar *context, GEr
 
 static void 
 iassist_show_tips (IAnjutaEditorAssist *iassist, GList* tips,
-				   gint position, gint char_alignment, GError **err)
+				   IAnjutaIterable *position, gint char_alignment, GError **err)
 {
 	gint lineno, cur_pos, cur_col, real_pos, real_col;
 	GString *calltip;
@@ -3210,31 +3254,29 @@ ibookmark_iface_init(IAnjutaBookmarkIface* iface)
 }
 
 static void
-iindicable_set (IAnjutaIndicable *te, gint begin_location, gint end_location,
+iindicable_set (IAnjutaIndicable *te, IAnjutaIterable *begin_location,
+				IAnjutaIterable *end_location,
 				IAnjutaIndicableIndicator indicator, GError **err)
 {
+	gint begin = text_editor_cell_get_position (TEXT_EDITOR_CELL (begin_location));
+	gint end = text_editor_cell_get_position (TEXT_EDITOR_CELL (end_location));
 	switch (indicator)
 	{
 		case IANJUTA_INDICABLE_NONE:
-			text_editor_set_indicator (TEXT_EDITOR (te), begin_location,
-									   end_location, -1);
+			text_editor_set_indicator (TEXT_EDITOR (te), begin, end, -1);
 		break;
 		case IANJUTA_INDICABLE_IMPORTANT:
-			text_editor_set_indicator (TEXT_EDITOR (te), begin_location,
-									   end_location, 0);
+			text_editor_set_indicator (TEXT_EDITOR (te), begin, end, 0);
 		break;
 		case IANJUTA_INDICABLE_WARNING:
-			text_editor_set_indicator (TEXT_EDITOR (te), begin_location,
-									   end_location, 1);
+			text_editor_set_indicator (TEXT_EDITOR (te),  begin, end, 1);
 		break;
 		case IANJUTA_INDICABLE_CRITICAL:
-			text_editor_set_indicator (TEXT_EDITOR (te), begin_location,
-									   end_location, 2);
+			text_editor_set_indicator (TEXT_EDITOR (te),  begin, end, 2);
 		break;
 		default:
 			g_warning ("Unsupported indicator %d", indicator);
-			text_editor_set_indicator (TEXT_EDITOR (te), begin_location,
-									   end_location, -1);
+			text_editor_set_indicator (TEXT_EDITOR (te),  begin, end, -1);
 		break;
 	}
 }
@@ -3630,6 +3672,22 @@ isearch_iface_init(IAnjutaEditorSearchIface* iface)
 	iface->backward = isearch_backward;
 }
 
+static void
+ihover_display (IAnjutaEditorHover *ihover, IAnjutaIterable *pos,
+				const gchar *info, GError **e)
+{
+	TextEditor *te = TEXT_EDITOR (ihover);
+	gint position = text_editor_cell_get_position (TEXT_EDITOR_CELL (pos));
+	g_return_if_fail (position >= 0);
+	g_return_if_fail (info != NULL);
+	text_editor_show_hover_tip (te, position, info);
+}
+
+static void
+ihover_iface_init(IAnjutaEditorHoverIface* iface)
+{
+	iface->display = ihover_display;
+}
 
 ANJUTA_TYPE_BEGIN(TextEditor, text_editor, GTK_TYPE_VBOX);
 ANJUTA_TYPE_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
@@ -3651,6 +3709,7 @@ ANJUTA_TYPE_ADD_INTERFACE(icomment, IANJUTA_TYPE_EDITOR_COMMENT);
 ANJUTA_TYPE_ADD_INTERFACE(izoom, IANJUTA_TYPE_EDITOR_ZOOM);
 ANJUTA_TYPE_ADD_INTERFACE(igoto, IANJUTA_TYPE_EDITOR_GOTO);
 ANJUTA_TYPE_ADD_INTERFACE(isearch, IANJUTA_TYPE_EDITOR_SEARCH);
+ANJUTA_TYPE_ADD_INTERFACE(ihover, IANJUTA_TYPE_EDITOR_HOVER);
 
 /* FIXME: Is factory definition really required for editor class? */
 ANJUTA_TYPE_ADD_INTERFACE(itext_editor_factory, IANJUTA_TYPE_EDITOR_FACTORY);
