@@ -26,12 +26,12 @@
 #include <libanjuta/anjuta-preferences.h>
 #include <libgnomevfs/gnome-vfs-monitor.h>
 
-#include "text_editor_menu.h"
 #include "aneditor.h"
 
 #include "tm_tagmanager.h"
 
 #include <libanjuta/anjuta-status.h>
+#include <libanjuta/anjuta-encodings.h>
 
 #define TEXT_EDITOR_FIND_SCOPE_WHOLE 1
 #define TEXT_EDITOR_FIND_SCOPE_CURRENT 2
@@ -89,7 +89,7 @@ struct _TextEditor
 	gboolean first_time_expose;
 
 	/* File encoding */
-	gchar *encoding;
+	const AnjutaEncoding *encoding;
 	
 	/* Popup menu widget */
 	GtkWidget *popup_menu;
