@@ -20,6 +20,7 @@
 
 #include <config.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 
@@ -33,11 +34,10 @@
 #include <libanjuta/interfaces/ianjuta-file-loader.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-wizard.h>
-#include <libegg/menu/egg-submenu-action.h>
-#include <libegg/menu/egg-recent-action.h>
 
 #include "plugin.h"
 #include "dnd.h"
+#include "anjuta-recent-chooser-menu.h"
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-loader-plugin.ui"
 
@@ -69,23 +69,31 @@ static void
 set_recent_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 				 const gchar *mime)
 {
-	EggRecentItem *recent_item;
+	GtkRecentData *recent_data;
+	gchar *name;
+	
 	DEBUG_PRINT ("Adding recent item of mimi-type: %s", mime);
-	recent_item = egg_recent_item_new ();
-	egg_recent_item_set_uri (recent_item, uri);
-	egg_recent_item_set_mime_type (recent_item, mime);
-	if (strcmp (mime, "application/x-anjuta") == 0)
+	
+	name = g_path_get_basename (uri);
+	
+	recent_data = g_slice_new (GtkRecentData);
+
+	recent_data->display_name   = name;
+	recent_data->description    = NULL;
+	recent_data->mime_type      = (gchar *)mime;
+	recent_data->app_name       = (gchar *) g_get_application_name ();
+	recent_data->app_exec       = g_strjoin (" ", g_get_prgname (), "%u", NULL);
+	recent_data->groups         = NULL;
+	recent_data->is_private     = FALSE;
+
+	if (!gtk_recent_manager_add_full (plugin->recent_manager, uri, recent_data))
 	{
-		egg_recent_item_add_group (recent_item, "anjuta-projects");
-		egg_recent_model_add_full (plugin->recent_files_model_top,
-								   recent_item);
-	}
-	else
-	{
-		egg_recent_item_add_group (recent_item, "anjuta-files");
-		egg_recent_model_add_full (plugin->recent_files_model_bottom,
-								   recent_item);
+      		g_warning ("Unable to add '%s' to the list of recently used documents", uri);
 	}
+
+	g_free (name);
+	g_free (recent_data->app_exec);
+	g_slice_free (GtkRecentData, recent_data);
 }
 
 static void
@@ -122,6 +130,19 @@ get_available_plugins_for_mime (AnjutaFileLoaderPlugin* plugin,
 												"SupportedMimeTypes",
 												mime_type,
 												NULL);
+	if (!plugin_descs);
+	{
+		gchar* supertype = gnome_vfs_get_supertype_from_mime_type (mime_type);
+		plugin_descs = anjuta_plugin_manager_query (plugin_manager,
+												"Anjuta Plugin",
+												"Interfaces", "IAnjutaFile",
+												"File Loader",
+												"SupportedMimeTypes",
+												supertype,
+												NULL);
+		g_free (supertype);
+	}
+	
 	return plugin_descs;
 }
 
@@ -145,7 +166,7 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 								 "There is no plugin, default action, or application "
 								 "configured to handle this file type.\n"
 								 "\n"
-								 "Mime type: %s.\n"
+								 "Mime type: %s\n"
 								 "\n"
 								 "You may choose to try opening it with the following "
 								 "plugins or applications."),
@@ -340,6 +361,8 @@ open_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri)
 	chdir (dirname);
 	g_free (dirname);
 	
+	/* FIXME: We have to manage the error to know if we have to remove the recent file
+	 */
 	ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
 							  uri, FALSE, NULL);
 }
@@ -362,14 +385,14 @@ on_open_recent_file_idle (gpointer data)
 }
 
 static gboolean
-on_open_recent_file (EggRecentAction *action, AnjutaFileLoaderPlugin *plugin)
+on_open_recent_file (GtkRecentChooser *chooser, AnjutaFileLoaderPlugin *plugin)
 {
 	const gchar *uri;
 	GnomeVFSURI *vfs_uri;
 	gboolean ret = TRUE;
 	RecentIdelOpenData *rdata;
 	
-	uri = egg_recent_action_get_selected_uri (action);
+	uri = gtk_recent_chooser_get_current_uri (chooser);
 	vfs_uri = gnome_vfs_uri_new (uri);
 	rdata = g_new0 (RecentIdelOpenData, 1);
 	rdata->plugin = plugin;
@@ -523,6 +546,17 @@ create_file_open_dialog_gui(GtkWindow* parent, AnjutaFileLoaderPlugin* plugin)
 	return dialog;
 }
 
+static void
+on_new_activate (GtkAction *action, AnjutaFileLoaderPlugin *plugin)
+{
+	AnjutaShell* shell = ANJUTA_PLUGIN (plugin)->shell;
+	IAnjutaDocumentManager *docman = anjuta_shell_get_interface (shell, 
+																 IAnjutaDocumentManager,
+																 NULL);
+	if (docman)
+		ianjuta_document_manager_add_buffer (docman, NULL, NULL, NULL);
+}
+
 static void
 on_open_activate (GtkAction *action, AnjutaFileLoaderPlugin *plugin)
 {
@@ -651,22 +685,6 @@ on_create_submenu (gpointer user_data)
 	return submenu;
 }
 
-/*
-static void
-on_recent_files_tooltip (GtkTooltips *tooltips, GtkWidget *menu_item,
-						 EggRecentItem *item, gpointer user_data)
-{
-	char *uri, *tip;
-
-	uri = egg_recent_item_get_uri_for_display (item);
-	tip = g_strdup_printf ("Open '%s'", uri);
-
-	gtk_tooltips_set_tip (tooltips, menu_item, tip, NULL);
-
-	g_free (uri);
-}
-*/
-
 static void
 open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 				const gchar *uri)
@@ -762,6 +780,14 @@ pm_open_with (GtkMenuItem *menuitem, AnjutaFileLoaderPlugin *plugin)
 }
 
 static GtkActionEntry actions_file[] = {
+	{ 
+		"ActionFileNew", 
+		GTK_STOCK_NEW,
+		N_("_New"), 
+		"<control>n",
+		N_("New empty file"),
+		G_CALLBACK (on_new_activate)
+	},
 	{
 		"ActionFileOpen",
 		GTK_STOCK_OPEN,
@@ -1092,16 +1118,34 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 	}
 }
 
+static void
+setup_recent_chooser_menu (GtkRecentChooser* recent_menu, AnjutaFileLoaderPlugin* plugin)
+{
+	GtkRecentFilter *filter;
+
+	gtk_recent_chooser_set_local_only (GTK_RECENT_CHOOSER (recent_menu), TRUE);
+	gtk_recent_chooser_set_show_icons (GTK_RECENT_CHOOSER (recent_menu), TRUE);
+	gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (recent_menu), GTK_RECENT_SORT_MRU);
+	gtk_recent_chooser_set_limit (GTK_RECENT_CHOOSER (recent_menu), 20);
+
+	filter = gtk_recent_filter_new ();
+	gtk_recent_filter_add_application (filter, g_get_application_name ());
+	gtk_recent_chooser_set_filter (GTK_RECENT_CHOOSER (recent_menu), filter);
+
+	g_signal_connect (recent_menu, "item-activated",
+					  G_CALLBACK (on_open_recent_file), plugin);	
+}
+
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
-	EggSubmenuAction *saction;
-	GtkAction *action;
+	GtkAction *action, *saction;
 	AnjutaUI *ui;
 	AnjutaFileLoaderPlugin *loader_plugin;
-	// GtkWidget *recent_menu;
-	// EggRecentViewGtk *recent_view;
 	GtkActionGroup *group;
+	GtkWidget *widget;
+	GtkWidget* recent_menu;
+	GtkWidget* toolbar_menu;
 	
 	loader_plugin = ANJUTA_PLUGIN_FILE_LOADER (plugin);
 	
@@ -1109,6 +1153,9 @@ activate_plugin (AnjutaPlugin *plugin)
 	
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	
+	/* Recent manager */
+	loader_plugin->recent_manager = gtk_recent_manager_get_default();
+	
 	/* Add action group */
 	loader_plugin->action_group =
 		anjuta_ui_add_action_group_entries (ui, "ActionGroupLoader",
@@ -1122,13 +1169,10 @@ activate_plugin (AnjutaPlugin *plugin)
 											popup_actions_file,
 											G_N_ELEMENTS (popup_actions_file),
 											GETTEXT_PACKAGE, FALSE, plugin);
-	saction = g_object_new (EGG_TYPE_SUBMENU_ACTION,
-							"name", "ActionFileWizard",
-							"label", _("New"),
-							"tooltip", _("New file, project and project components."),
-							NULL);
-	egg_submenu_action_set_menu_factory (saction,
-										 on_create_submenu, plugin);
+	saction = gtk_recent_action_new ("ActionFileWizard", _("New"),
+							  _("New file, project and project components."), NULL);
+	g_object_set (saction, "stock-id", GTK_STOCK_NEW, NULL);
+	g_signal_connect (saction, "activate", G_CALLBACK (on_new_activate), loader_plugin);
 	gtk_action_group_add_action (loader_plugin->action_group,
 								 GTK_ACTION (saction));
 	
@@ -1138,17 +1182,12 @@ activate_plugin (AnjutaPlugin *plugin)
 				  "is-important", TRUE, NULL);
 
 	group = gtk_action_group_new ("ActionGroupLoaderRecent");
-	action = g_object_new (EGG_TYPE_RECENT_ACTION,
-						   "name", "ActionFileOpenRecent",
-						   "label", _("Open _Recent"),
-						   "tooltip", _("Open recent file"),
-							NULL);
-	egg_recent_action_add_model (EGG_RECENT_ACTION (action),
-								 loader_plugin->recent_files_model_top);
-	egg_recent_action_add_model (EGG_RECENT_ACTION (action),
-								 loader_plugin->recent_files_model_bottom);
-	g_signal_connect (action, "activate",
-					  G_CALLBACK (on_open_recent_file), plugin);
+	action = gtk_recent_action_new ("ActionFileOpenRecent", _("Open _Recent"),
+												_("Open recent file"), NULL);
+	g_object_set (action, "stock-id", GTK_STOCK_OPEN, NULL);
+	setup_recent_chooser_menu (GTK_RECENT_CHOOSER (action), loader_plugin);
+	g_signal_connect (action, "activate", G_CALLBACK (on_open_activate), loader_plugin);
+	
 	gtk_action_group_add_action (group, action);
 	anjuta_ui_add_action_group (ui, "ActionGroupLoaderRecent",
 								N_("Open recent files"), group, FALSE);
@@ -1157,6 +1196,29 @@ activate_plugin (AnjutaPlugin *plugin)
 	/* Add UI */
 	loader_plugin->uiid = anjuta_ui_merge (ui, UI_FILE);
 	
+	/* Adding submenus */
+	recent_menu = anjuta_recent_chooser_menu_new_for_manager (loader_plugin->recent_manager);
+	setup_recent_chooser_menu (GTK_RECENT_CHOOSER (recent_menu), loader_plugin);
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/MenuMain/MenuFile/PlaceholderFileMenus/OpenRecent");
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (widget), 
+							   recent_menu);
+	
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/MenuMain/MenuFile/PlaceholderFileMenus/Wizard");
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (widget), on_create_submenu(loader_plugin));
+	
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/ToolbarMain/PlaceholderFileToolbar/New");
+	gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (widget), on_create_submenu(loader_plugin));
+	
+	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER(ui),
+					"/ToolbarMain/PlaceholderFileToolbar/Open");
+	toolbar_menu = anjuta_recent_chooser_menu_new_for_manager (loader_plugin->recent_manager);
+	setup_recent_chooser_menu (GTK_RECENT_CHOOSER (toolbar_menu), loader_plugin);
+	gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (widget),
+								   toolbar_menu);
+	
 	/* Install drag n drop handler */
 	dnd_drop_init (GTK_WIDGET (plugin->shell), dnd_dropped, plugin,
 				   "text/plain", "text/html", "text/source", "application-x/anjuta",
@@ -1210,18 +1272,14 @@ deactivate_plugin (AnjutaPlugin *plugin)
 static void
 dispose (GObject *obj)
 {
-	AnjutaFileLoaderPlugin *plugin = ANJUTA_PLUGIN_FILE_LOADER (obj);
-	if (plugin->recent_files_model_top)
-	{
-		g_object_unref (plugin->recent_files_model_top);
-		plugin->recent_files_model_top = NULL;
-	}
-	if (plugin->recent_files_model_bottom)
-	{
-		g_object_unref (plugin->recent_files_model_bottom);
-		plugin->recent_files_model_bottom = NULL;
-	}
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
+}
+
+static void
+finalize (GObject *obj)
+{
+	/* Finalization codes here */
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -1231,17 +1289,6 @@ anjuta_file_loader_plugin_instance_init (GObject *obj)
 	
 	plugin->fm_current_uri = NULL;
 	plugin->pm_current_uri = NULL;
-	
-	plugin->recent_files_model_top =
-		egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
-	egg_recent_model_set_limit (plugin->recent_files_model_top, 5);
-	egg_recent_model_set_filter_groups (plugin->recent_files_model_top,
-										"anjuta-projects", NULL);
-	plugin->recent_files_model_bottom =
-		egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
-	egg_recent_model_set_limit (plugin->recent_files_model_bottom, 15);
-	egg_recent_model_set_filter_groups (plugin->recent_files_model_bottom,
-										"anjuta-files", NULL);
 }
 
 static void
@@ -1254,6 +1301,7 @@ anjuta_file_loader_plugin_class_init (GObjectClass *klass)
 	plugin_class->activate = activate_plugin;
 	plugin_class->deactivate = deactivate_plugin;
 	klass->dispose = dispose;
+	klass->finalize = finalize;
 }
 
 static GObject*
@@ -1309,11 +1357,16 @@ iloader_load (IAnjutaFileLoader *loader, const gchar *uri,
 	
 	if (g_list_length (plugin_descs) > 1)
 	{
+		gchar* basename = g_path_get_basename (uri);
+		gchar* message = g_strdup_printf ("Please select a plugin to open <b>%s</b>.", 
+										 basename);
 		plugin =
 			anjuta_plugin_manager_select_and_activate (plugin_manager,
 													   "Open With",
-								"Please select a plugin to open this file.",
+													   message,
 													   plugin_descs);
+		g_free (basename);
+		g_free (message);
 	}
 	else if (g_list_length (plugin_descs) == 1)
 	{
