@@ -122,7 +122,7 @@ gdb_plugin_start_terminal (GdbPlugin* plugin)
 
 	/* Launch terminal */
 	cmd = g_strconcat ("anjuta_launcher --__debug_terminal ", file, NULL);
-	plugin->term_pid = ianjuta_terminal_execute_command (term, NULL, cmd, NULL);
+	plugin->term_pid = ianjuta_terminal_execute_command (term, NULL, cmd, NULL, NULL);
 	g_free (cmd);
 
 	if (plugin->term_pid > 0)
@@ -253,7 +253,7 @@ gdb_plugin_dispose (GObject* obj)
 		debugger_free (this->debugger);
 		this->debugger = NULL;
 	}
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (G_OBJECT (obj)));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 /* finalize is the last destruction step. It must free all memory allocated
@@ -263,7 +263,7 @@ gdb_plugin_dispose (GObject* obj)
 static void
 gdb_plugin_finalize (GObject* obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (G_OBJECT (obj)));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 /* class_init intialize the class itself not the instance */
@@ -312,7 +312,8 @@ idebugger_load (IAnjutaDebugger *plugin, const gchar *file, const gchar* mime_ty
 	{
 		/* Hope that the target is supported */
 	}
-	else if (strcmp (mime_type, "application/x-executable") == 0)
+	else if ((strcmp (mime_type, "application/x-executable") == 0) ||
+		(strcmp (mime_type, "application/octet-stream") == 0))
 	{
 		/* Supported target */
 	}
@@ -349,6 +350,26 @@ idebugger_unload (IAnjutaDebugger *plugin, GError **err)
 	return TRUE;
 }
 
+static gboolean
+idebugger_set_working_directory (IAnjutaDebugger *plugin, const gchar *directory, GError **err)
+{
+	GdbPlugin *self = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_set_working_directory (self->debugger, directory);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_set_environment (IAnjutaDebugger *plugin, gchar **variables, GError **err)
+{
+	GdbPlugin *self = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_set_environment (self->debugger, variables);
+
+	return TRUE;
+}
+
 static gboolean
 idebugger_attach (IAnjutaDebugger *plugin, pid_t pid, const GList *search_dirs, GError **err)
 {
@@ -733,6 +754,8 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 	iface->get_state = idebugger_get_state;
 	iface->attach = idebugger_attach;
 	iface->load = idebugger_load;
+	iface->set_working_directory = idebugger_set_working_directory;
+	iface->set_environment = idebugger_set_environment;
 	iface->start = idebugger_start;
 	iface->unload = idebugger_unload;
 	iface->quit = idebugger_quit;
