@@ -47,6 +47,7 @@ typedef struct
 {
 	gchar *name;
 	gboolean is_func;
+	IAnjutaSymbolType type;
 } CppJavaAssistTag;
 
 struct _CppJavaAssistPriv {
@@ -72,7 +73,8 @@ completion_compare (gconstpointer a, gconstpointer b)
 {
 	CppJavaAssistTag * tag_a = (CppJavaAssistTag*) a;
 	CppJavaAssistTag * tag_b = (CppJavaAssistTag*) b;
-	return strcmp (tag_a->name, tag_b->name);
+	return (strcmp (tag_a->name, tag_b->name) &&
+			tag_a->type == tag_b->type);
 }
 
 static void
@@ -137,32 +139,29 @@ static GCompletion*
 create_completion (IAnjutaEditorAssist* iassist, IAnjutaIterable* iter)
 {
 	GCompletion *completion = g_completion_new (completion_function);
-	GHashTable *suggestions_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	GList* suggestions = NULL;
 	do
 	{
-		const gchar* name = ianjuta_symbol_name (IANJUTA_SYMBOL(iter), NULL);
+		const gchar* name = ianjuta_symbol_get_name (IANJUTA_SYMBOL(iter), NULL);
 		if (name != NULL)
 		{
-			if (!g_hash_table_lookup (suggestions_hash, name))
-			{
-				CppJavaAssistTag *tag = g_new0 (CppJavaAssistTag, 1);
-				tag->name = g_strdup (name);
-				tag->is_func = (ianjuta_symbol_args (IANJUTA_SYMBOL (iter),
-													 NULL)
-								!= NULL);
-				g_hash_table_insert (suggestions_hash, (gchar*)name,
-									 (gchar*)name);
+			CppJavaAssistTag *tag = g_new0 (CppJavaAssistTag, 1);
+			tag->name = g_strdup (name);
+			tag->type = ianjuta_symbol_get_sym_type (IANJUTA_SYMBOL (iter),
+													 NULL);
+			tag->is_func = (tag->type == IANJUTA_SYMBOL_TYPE_FUNCTION ||
+							tag->type == IANJUTA_SYMBOL_TYPE_METHOD ||
+							tag->type == IANJUTA_SYMBOL_TYPE_MACRO_WITH_ARG);
+			if (!g_list_find_custom (suggestions, tag, completion_compare))
 				suggestions = g_list_prepend (suggestions, tag);
-			}
+			else
+				g_free (tag);
 		}
 		else
 			break;
 	}
 	while (ianjuta_iterable_next (iter, NULL));
 	
-	g_hash_table_destroy (suggestions_hash);
-	
 	suggestions = g_list_sort (suggestions, completion_compare);
 	g_completion_add_items (completion, suggestions);
 	return completion;
@@ -307,6 +306,8 @@ cpp_java_assist_create_scope_completion_cache (CppJavaAssist *assist,
 											   const gchar *scope_operator,
 											   const gchar *scope_context)
 {
+	DEBUG_PRINT ("TODO: cpp_java_assist_create_scope_completion_cache ()");
+#if 0	
 	cpp_java_assist_destroy_completion_cache (assist);
 	if (g_str_equal (scope_operator, "::"))
 	{
@@ -320,24 +321,33 @@ cpp_java_assist_create_scope_completion_cache (CppJavaAssist *assist,
 			assist->priv->scope_context_cache = g_strdup (scope_context);
 			g_object_unref (iter);
 		}
-	}
+	} 
 	else if (g_str_equal (scope_operator, ".") ||
 			 g_str_equal (scope_operator, "->"))
 	{
 		/* TODO: Find the type of context by parsing the file somehow and
 		search for the member as it is done with the :: context */
 	}
+#endif	
 }
 
 static void
 cpp_java_assist_create_word_completion_cache (CppJavaAssist *assist,
 											  const gchar *pre_word)
 {
+	gint max_completions;
+	max_completions =
+		anjuta_preferences_get_int_with_default (assist->priv->preferences,
+												 PREF_AUTOCOMPLETE_CHOICES,
+												 MAX_COMPLETIONS);
+	
 	cpp_java_assist_destroy_completion_cache (assist);
 	IAnjutaIterable* iter = 
 		ianjuta_symbol_manager_search (assist->priv->isymbol_manager,
 										IANJUTA_SYMBOL_TYPE_MAX,
-										pre_word, TRUE, TRUE, NULL);
+									    TRUE,
+										IANJUTA_SYMBOL_FIELD_SIMPLE|IANJUTA_SYMBOL_FIELD_TYPE,
+										pre_word, TRUE, TRUE, max_completions, -1, NULL);
 	if (iter)
 	{
 		assist->priv->completion_cache =
@@ -472,23 +482,32 @@ cpp_java_assist_show_calltip (CppJavaAssist *assist, gchar *call_context,
 							  IAnjutaIterable *position_iter)
 {
 	GList *tips = NULL;
+	gint max_completions;
 	
+	max_completions =
+		anjuta_preferences_get_int_with_default (assist->priv->preferences,
+												 PREF_AUTOCOMPLETE_CHOICES,
+												 MAX_COMPLETIONS);
+
 	IAnjutaIterable* iter = 
 		ianjuta_symbol_manager_search (assist->priv->isymbol_manager,
 									   IANJUTA_SYMBOL_TYPE_PROTOTYPE|
 									   IANJUTA_SYMBOL_TYPE_FUNCTION|
 									   IANJUTA_SYMBOL_TYPE_METHOD|
 									   IANJUTA_SYMBOL_TYPE_MACRO_WITH_ARG,
-									   call_context, FALSE, TRUE, NULL);
+									   TRUE,
+									   IANJUTA_SYMBOL_FIELD_SIMPLE,
+									   call_context, FALSE, TRUE, max_completions,
+									   -1, NULL);
 	if (iter)
 	{
 		do
 		{
 			IAnjutaSymbol* symbol = IANJUTA_SYMBOL(iter);
-			const gchar* name = ianjuta_symbol_name(symbol, NULL);
+			const gchar* name = ianjuta_symbol_get_name(symbol, NULL);
 			if (name != NULL)
 			{
-				const gchar* args = ianjuta_symbol_args(symbol, NULL);
+				const gchar* args = ianjuta_symbol_get_args(symbol, NULL);
 				gchar* print_args;
 				gchar* separator;
 				gchar* white_name = g_strnfill (strlen(name) + 1, ' ');
