@@ -49,7 +49,6 @@ on_ok_clicked (MacroPlugin * plugin)
 	GtkTreeIter iter;
 	gchar* text;
 	gint offset = 0;
-	gint pos;
 	
 	g_return_if_fail (plugin != NULL);
 	g_return_if_fail (model != NULL);
@@ -61,13 +60,17 @@ on_ok_clicked (MacroPlugin * plugin)
 	{
 		if (plugin->current_editor != NULL)
 		{
+			gint i;
+			IAnjutaIterable *pos;
 			pos = ianjuta_editor_get_position (IANJUTA_EDITOR(plugin->current_editor),
-			                                   NULL);
+											   NULL);
 			ianjuta_editor_insert (IANJUTA_EDITOR(plugin->current_editor),
 			                       pos, text, -1, NULL);
+			for (i = 0; i < offset; i++)
+				ianjuta_iterable_next (pos, NULL);
 			ianjuta_editor_goto_position (IANJUTA_EDITOR(plugin->current_editor), 
-			                              pos + offset, 
-			                              NULL);
+			                              pos, NULL);
+			g_object_unref (pos);
 		}
 		g_free(text);
 		gtk_widget_hide (plugin->macro_dialog);
@@ -151,6 +154,7 @@ on_macro_selection_changed (GtkTreeSelection * selection,
 
 				gtk_widget_set_sensitive (edit, !predefined);
 				gtk_widget_set_sensitive (remove, !predefined);
+				g_free (text);
 				return;
 			}
 		}
@@ -228,7 +232,13 @@ macro_dialog_dispose (GObject * object)
 		g_object_unref (dialog->macro_db);
 		dialog->macro_db = NULL;
 	}
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (G_OBJECT (object)));
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
+static void
+macro_dialog_finalize (GObject *object)
+{
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -237,6 +247,7 @@ macro_dialog_class_init (MacroDialogClass * klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	parent_class = g_type_class_peek_parent (klass);
 	object_class->dispose = macro_dialog_dispose;
+	object_class->finalize = macro_dialog_finalize;
 }
 
 static void
