@@ -1524,14 +1524,14 @@ static void
 project_manager_plugin_finalize (GObject *obj)
 {
 	/* FIXME: */
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
 project_manager_plugin_dispose (GObject *obj)
 {
 	/* FIXME: */
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
@@ -2217,6 +2217,28 @@ iproject_manager_is_open (IAnjutaProjectManager *project_manager, GError **err)
 	return GBF_IS_PROJECT (plugin->project);
 }
 
+static GList*
+iproject_manager_get_packages (IAnjutaProjectManager *project_manager, GError **err)
+{
+	ProjectManagerPlugin *plugin;
+	GList *modules = NULL;
+	GList *packages = NULL;
+	GList* node;
+	
+	plugin = ANJUTA_PLUGIN_PROJECT_MANAGER (G_OBJECT (project_manager));
+	
+	modules = gbf_project_get_config_modules (plugin->project, NULL);
+	for (node = modules; node != NULL; node = g_list_next (node))
+	{
+		GList* mod_pkgs = gbf_project_get_config_packages (plugin->project,
+														   node->data, NULL);
+		packages = g_list_concat (packages, mod_pkgs);
+	}
+	g_list_foreach (modules, (GFunc)g_free, NULL);
+	g_list_free (modules);
+	return packages;
+}
+
 static void
 iproject_manager_iface_init(IAnjutaProjectManagerIface *iface)
 {
@@ -2233,6 +2255,7 @@ iproject_manager_iface_init(IAnjutaProjectManagerIface *iface)
 	iface->add_target = iproject_manager_add_target;
 	iface->add_group = iproject_manager_add_group;
 	iface->is_open = iproject_manager_is_open;
+	iface->get_packages = iproject_manager_get_packages;
 }
 
 static void
@@ -2254,7 +2277,9 @@ ifile_open (IAnjutaFile *ifile, const gchar* uri, GError **e)
 	/* If there is already a project loaded, load in separate anjuta window */
 	if (plugin->project_root_uri)
 	{
-		gchar *cmd = g_strconcat ("anjuta --no-splash --no-client ", uri, NULL);
+		gchar *quoted_uri = g_shell_quote (uri);
+		gchar *cmd = g_strconcat ("anjuta --no-splash --no-client ", quoted_uri, NULL);
+		g_free (quoted_uri);
 		anjuta_util_execute_shell (NULL, cmd);
 		g_free (cmd);
 		return;
