@@ -23,6 +23,7 @@
 
 #include <gnome.h>
 #include <glade/glade.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-plugin.h>
@@ -35,8 +36,6 @@
 #include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-indicable.h>
 
-#include <libegg/menu/egg-entry-action.h>
-
 #include "search-replace_backend.h"
 #include "search-replace.h"
 #include "search_preferences.h"
@@ -229,6 +228,7 @@ static GladeWidget glade_widgets[] = {
 static void
 write_message_pane(IAnjutaMessageView* view, FileBuffer *fb, SearchEntry *se, MatchInfo *mi);
 static gboolean on_message_clicked (GObject* object, gchar* message, gpointer data);
+static void on_message_view_destroyed (gpointer unused, GObject* where_the_object_was);
 static void on_message_buffer_flush (IAnjutaMessageView *view, const gchar *one_line, gpointer data);
 static void save_not_opened_files(FileBuffer *fb);
 static gboolean replace_in_not_opened_files(FileBuffer *fb, MatchInfo *mi, gchar *repl_str);
@@ -327,7 +327,6 @@ search_and_replace (void)
 		view = ianjuta_message_manager_get_view_by_name(msgman, name, NULL);
 		if (view == NULL)	
 		{
-			// FIXME: Put a nice icon here:
 			view = ianjuta_message_manager_add_view(msgman, name,
 													GTK_STOCK_FIND_AND_REPLACE, NULL);	
 			g_return_if_fail(view != NULL);
@@ -335,6 +334,7 @@ search_and_replace (void)
 			                  G_CALLBACK (on_message_buffer_flush), NULL);
 			g_signal_connect (G_OBJECT(view), "message_clicked",
 			                  G_CALLBACK (on_message_clicked), NULL);
+			g_object_weak_ref (G_OBJECT(view), on_message_view_destroyed, NULL);
 		}
 		else
 			ianjuta_message_view_clear(view, NULL);
@@ -450,8 +450,8 @@ search_and_replace (void)
 							if (fb->te)
 								ianjuta_editor_goto_line (fb->te, mi->line, NULL);
 							else
-								fb->te = ianjuta_document_manager_goto_file_line_mark
-									(sr->docman, fb->path, mi->line, FALSE, NULL);
+								fb->te = ianjuta_document_manager_goto_uri_line_mark
+									(sr->docman, fb->uri, mi->line, FALSE, NULL);
 							found_line = mi->line;
 						}
 						{
@@ -476,8 +476,8 @@ search_and_replace (void)
 							if (fb->te)
 								ianjuta_editor_goto_line (fb->te, mi->line, NULL);
 							else
-								fb->te = ianjuta_document_manager_goto_file_line_mark
-									(sr->docman, fb->path, mi->line, FALSE, NULL);
+								fb->te = ianjuta_document_manager_goto_uri_line_mark
+									(sr->docman, fb->uri, mi->line, FALSE, NULL);
 							found_line = mi->line;
 							}
 
@@ -674,6 +674,12 @@ write_message_pane(IAnjutaMessageView* view, FileBuffer *fb, SearchEntry *se,
 	ianjuta_message_view_buffer_append (view, buf, NULL);
 }
 
+static void
+on_message_view_destroyed (gpointer unused, GObject* where_the_object_was)
+{
+	end_activity = TRUE;
+}
+
 static void
 on_message_buffer_flush (IAnjutaMessageView *view, const gchar *one_line,
 						 gpointer data)
@@ -687,6 +693,7 @@ on_message_clicked (GObject* object, gchar* message, gpointer data)
 {
 	gchar *ptr, *ptr2;
 	gchar *path, *nline;
+	gchar *uri;
 	gint line;
 		
 	if (!(ptr = g_strstr_len(message, strlen(message), ":")) )
@@ -698,8 +705,10 @@ on_message_clicked (GObject* object, gchar* message, gpointer data)
 		return FALSE;
 	nline = g_strndup(ptr, ptr2 - ptr);
 	line = atoi(nline);
-							
-	ianjuta_document_manager_goto_file_line_mark (sr->docman, path, line, TRUE, NULL);  
+	
+	uri = gnome_vfs_get_uri_from_local_path (path);  
+	ianjuta_document_manager_goto_uri_line_mark (sr->docman, uri, line, TRUE, NULL);
+	g_free(uri);
 	g_free(path);
 	g_free(nline);
 	return FALSE;
@@ -1091,7 +1100,8 @@ nb_results_alert(gint nb)
 {
 	GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW (sg->dialog),
 				GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
-		        	_("%d matches have been replaced."), nb);
+		        	ngettext("%d match has been replaced.",
+							 "%d matches have been replaced.", nb), nb);
 	g_signal_connect(G_OBJECT(dialog), "key-press-event",
 			G_CALLBACK(on_search_dialog_key_press_event), NULL);
 	gtk_dialog_run(GTK_DIALOG(dialog));
@@ -1349,21 +1359,6 @@ list_max_items(GList *list, guint nb_max)
 
 #define MAX_ITEMS_SEARCH_COMBO 16
 
-void
-search_toolbar_set_text(gchar *search_text)
-{
-	AnjutaUI *ui; 
-	GtkAction *action; 
-	
-	AnjutaShell* shell;
-	g_object_get(G_OBJECT(sr->docman), "shell", &shell, NULL);
-	
-	ui = anjuta_shell_get_ui (shell, NULL);
-	action = anjuta_ui_get_action (ui, "ActionGroupNavigation",
-								   "ActionEditSearchEntry");
-	egg_entry_action_set_text (EGG_ENTRY_ACTION(action), search_text);	
-}
-
 static void
 search_update_combos(void)
 {
@@ -1611,7 +1606,8 @@ void
 on_search_regex_toggled (GtkToggleButton *togglebutton, gpointer user_data)
 {
 	static GladeWidgetId dependent_widgets[] = {
-		GREEDY, IGNORE_CASE, WHOLE_WORD, WHOLE_LINE, WORD_START
+		WHOLE_WORD, WHOLE_LINE, WORD_START,
+		SEARCH_BACKWARD, SEARCH_FULL_BUFFER
 	};
 	int i;
 	GtkWidget *dircombo = sr_get_gladewidget(SEARCH_DIRECTION_COMBO)->widget;
@@ -1804,7 +1800,11 @@ on_search_button_stop_clicked(GtkButton *button, gpointer user_data)
 void
 on_search_button_next_clicked(GtkButton *button, gpointer user_data)
 {	
-	clear_pcre(); 	
+	if (sr->search.expr.regex_info)
+	{
+		g_regex_unref (sr->search.expr.regex_info);
+		sr->search.expr.regex_info = NULL;
+	}
 	search_replace_populate();
 
 	search_and_replace();
@@ -1827,7 +1827,7 @@ void search_replace_find_usage(const gchar *symbol)
 	sr->search.expr.word_start = FALSE;
 	sr->search.expr.no_limit = TRUE;
 	sr->search.expr.actions_max = G_MAXINT;
-	sr->search.expr.re = NULL;
+	sr->search.expr.regex_info = NULL;
 		
 	g_object_get(G_OBJECT(sr->docman), "shell", &shell, NULL);
 	
