@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA 02110-1301, USA.
  */
  
 /*
@@ -25,15 +25,15 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: anjuta-document-loader.c 2450 2006-03-20 22:24:24Z jhs $
+ * $Id: anjuta-document-loader.c 3965 2008-05-27 19:59:18Z sgranjoux $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <sys/mman.h>
 #include <sys/types.h>
+#include <sys/mman.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
@@ -43,10 +43,12 @@
 #include <glib/gstdio.h>
 #include <libgnomevfs/gnome-vfs.h>
 
-#include "anjuta-encodings.h"
+#include <libanjuta/anjuta-encodings.h>
+#include <libanjuta/anjuta-convert.h>
+
 #include "anjuta-document-loader.h"
 #include "anjuta-marshal.h"
-#include "anjuta-convert.h"
+
 
 #define READ_CHUNK_SIZE 8192
 
