@@ -25,15 +25,15 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: anjuta-document-saver.c 3079 2007-08-30 22:38:54Z jhs $
+ * $Id: anjuta-document-saver.c 3965 2008-05-27 19:59:18Z sgranjoux $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <sys/mman.h>
 #include <sys/types.h>
+#include <sys/mman.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
@@ -43,10 +43,11 @@
 #include <glib/gfileutils.h>
 #include <libgnomevfs/gnome-vfs.h>
 
-#include "anjuta-encodings.h"
+#include <libanjuta/anjuta-encodings.h>
+#include <libanjuta/anjuta-convert.h>
+
 #include "anjuta-document-saver.h"
 #include "anjuta-marshal.h"
-#include "anjuta-convert.h"
 
 #define ANJUTA_DOCUMENT_SAVER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), ANJUTA_TYPE_DOCUMENT_SAVER, AnjutaDocumentSaverPrivate))
 
