@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: anjuta-document.c 3514 2008-01-24 11:21:05Z jhs $
+ * $Id: anjuta-document.c 3977 2008-06-02 19:36:23Z sgranjoux $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,7 +41,8 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libanjuta/anjuta-debug.h>
 
-#include "anjuta-encodings.h"
+#include <libanjuta/anjuta-encodings.h>
+
 #include "anjuta-document.h"
 #include "anjuta-document-loader.h"
 #include "anjuta-document-saver.h"
@@ -49,7 +50,6 @@
 #include "anjuta-utils.h"
 
 #include <gtksourceview/gtksourceiter.h>
-#include <pcre.h>
 
 #define ANJUTA_MAX_PATH_LEN  2048
 
