@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: anjuta-utils.c 3079 2007-08-30 22:38:54Z jhs $
+ * $Id: anjuta-utils.c 3848 2008-04-12 14:13:44Z jhs $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,7 +54,8 @@
 #include "anjuta-utils.h"
 
 #include "anjuta-document.h"
-#include "anjuta-convert.h"
+
+#include <libanjuta/anjuta-convert.h>
 
 #define STDIN_DELAY_MICROSECONDS 100000
 
