@@ -44,7 +44,7 @@ anjuta_symbol_finalize (GObject * obj)
 	if (s->priv->uri)
 		g_free (s->priv->uri);
 	g_free (s->priv);
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -83,6 +83,15 @@ anjuta_symbol_new (const TMTag *tm_tag)
 	return s;
 }
 
+
+const gchar* 
+anjuta_symbol_get_name (AnjutaSymbol* symbol)
+{
+	g_return_val_if_fail (symbol != NULL, NULL);
+	
+	return symbol->priv->tm_tag->name;
+}
+
 void
 anjuta_symbol_set_tag (AnjutaSymbol *symbol, const TMTag *tm_tag)
 {
@@ -103,9 +112,19 @@ anjuta_symbol_set_tag (AnjutaSymbol *symbol, const TMTag *tm_tag)
 }
 
 /* IAnjutaSymbol implementation */
+static const gchar*
+isymbol_get_name (IAnjutaSymbol *isymbol, GError **err)
+{
+	AnjutaSymbol *s;
+
+	g_return_val_if_fail (ANJUTA_IS_SYMBOL (isymbol), NULL);
+	s = ANJUTA_SYMBOL (isymbol);
+	g_return_val_if_fail (s->priv->tm_tag != NULL, NULL);
+	return s->priv->tm_tag->name;
+}
 
 static IAnjutaSymbolType
-isymbol_type (IAnjutaSymbol *isymbol, GError **err)
+isymbol_get_sym_type (IAnjutaSymbol *isymbol, GError **err)
 {
 	AnjutaSymbol *s;
 	
@@ -116,36 +135,7 @@ isymbol_type (IAnjutaSymbol *isymbol, GError **err)
 }
 
 static const gchar*
-isymbol_type_name (IAnjutaSymbol *isymbol, GError **err)
-{
-	AnjutaSymbol *s;
-
-	g_return_val_if_fail (ANJUTA_IS_SYMBOL (isymbol), NULL);
-	s = ANJUTA_SYMBOL (isymbol);
-	g_return_val_if_fail (s->priv->tm_tag != NULL, NULL);
-	return s->priv->tm_tag->atts.entry.type_ref[1];
-}
-
-static const gchar*
-isymbol_type_str (IAnjutaSymbol *isymbol, GError **err)
-{
-	DEBUG_PRINT ("TODO: isymbol_type_str ()");
-	return NULL;
-}
-
-static const gchar*
-isymbol_name (IAnjutaSymbol *isymbol, GError **err)
-{
-	AnjutaSymbol *s;
-
-	g_return_val_if_fail (ANJUTA_IS_SYMBOL (isymbol), NULL);
-	s = ANJUTA_SYMBOL (isymbol);
-	g_return_val_if_fail (s->priv->tm_tag != NULL, NULL);
-	return s->priv->tm_tag->name;
-}
-
-static const gchar*
-isymbol_args (IAnjutaSymbol *isymbol, GError **err)
+isymbol_get_args (IAnjutaSymbol *isymbol, GError **err)
 {
 	AnjutaSymbol *s;
 
@@ -156,55 +146,62 @@ isymbol_args (IAnjutaSymbol *isymbol, GError **err)
 }
 
 static const gchar*
-isymbol_scope (IAnjutaSymbol *isymbol, GError **err)
+isymbol_get_extra_info_string (IAnjutaSymbol *isymbol, IAnjutaSymbolField sym_info,
+							   GError **err)
 {
 	AnjutaSymbol *s;
 
 	g_return_val_if_fail (ANJUTA_IS_SYMBOL (isymbol), NULL);
 	s = ANJUTA_SYMBOL (isymbol);
 	g_return_val_if_fail (s->priv->tm_tag != NULL, NULL);
-	return s->priv->tm_tag->atts.entry.scope;
-}
-
-static const gchar*
-isymbol_inheritance (IAnjutaSymbol *isymbol, GError **err)
-{
-	AnjutaSymbol *s;
-
-	g_return_val_if_fail (ANJUTA_IS_SYMBOL (isymbol), NULL);
-	s = ANJUTA_SYMBOL (isymbol);
-	g_return_val_if_fail (s->priv->tm_tag != NULL, NULL);
-//	return s->priv->tm_tag->atts.entry.inheritance;
-	DEBUG_PRINT ("isymbol_inheritance () FIXME");
-	return NULL;
-}
-
-static const gchar*
-isymbol_access (IAnjutaSymbol *isymbol, GError **err)
-{
-	AnjutaSymbol *s;
-
-	g_return_val_if_fail (ANJUTA_IS_SYMBOL (isymbol), '\0');
-	s = ANJUTA_SYMBOL (isymbol);
-	g_return_val_if_fail (s->priv->tm_tag != NULL, '\0');
-	DEBUG_PRINT ("isymbol_access () FIXME");
-	return "fixme";
-//	return s->priv->tm_tag->atts.entry.access;
-}
-
-static const gchar*
-isymbol_impl (IAnjutaSymbol *isymbol, GError **err)
-{
-	AnjutaSymbol *s;
-
-	g_return_val_if_fail (ANJUTA_IS_SYMBOL (isymbol), '\0');
-	s = ANJUTA_SYMBOL (isymbol);
-	g_return_val_if_fail (s->priv->tm_tag != NULL, '\0');
-	DEBUG_PRINT ("isymbol_impl () FIXME");
-	return "fixme";
 	
-//	return s->priv->tm_tag->atts.entry.impl;	
-}
+	switch (sym_info) {
+		case IANJUTA_SYMBOL_FIELD_FILE_PATH:
+			if (s->priv->tm_tag->atts.entry.file == NULL)
+				return NULL;
+			return s->priv->tm_tag->atts.entry.file->work_object.file_name;
+			
+		case IANJUTA_SYMBOL_FIELD_IMPLEMENTATION:
+			g_message ("TODO IANJUTA_SYMBOL_FIELD_IMPLEMENTATION");
+			return NULL;
+			/*return s->priv->tm_tag->atts.entry.impl;*/
+		
+		case IANJUTA_SYMBOL_FIELD_ACCESS:
+			g_message ("TODO IANJUTA_SYMBOL_FIELD_ACCESS");
+			return NULL;
+			/*return s->priv->tm_tag->atts.entry.access;*/
+		
+		case IANJUTA_SYMBOL_FIELD_KIND:
+			g_message ("No symbol kind to string translation available");
+			return NULL;
+			
+		case IANJUTA_SYMBOL_FIELD_TYPE:
+			g_message ("No symbol type to string translation available");
+			return NULL;		
+		
+		case IANJUTA_SYMBOL_FIELD_TYPE_NAME:
+			return s->priv->tm_tag->atts.entry.type_ref[1];
+		
+		case IANJUTA_SYMBOL_FIELD_LANGUAGE:
+			g_message ("No file language available");
+			return NULL;
+		
+		case IANJUTA_SYMBOL_FIELD_FILE_IGNORE:
+			return NULL;
+		
+		case IANJUTA_SYMBOL_FIELD_FILE_INCLUDE:
+			return NULL;
+		
+		case IANJUTA_SYMBOL_FIELD_PROJECT_NAME:
+			return NULL;
+		
+		case IANJUTA_SYMBOL_FIELD_WORKSPACE_NAME:
+			return NULL;
+		
+		default:
+			return NULL;
+	}
+} 
 
 static const gchar*
 isymbol_uri (IAnjutaSymbol *isymbol, GError **err)
@@ -226,7 +223,7 @@ isymbol_uri (IAnjutaSymbol *isymbol, GError **err)
 }
 
 static gulong
-isymbol_line (IAnjutaSymbol *isymbol, GError **err)
+isymbol_get_line (IAnjutaSymbol *isymbol, GError **err)
 {
 	AnjutaSymbol *s;
 
@@ -248,7 +245,7 @@ isymbol_is_local (IAnjutaSymbol *isymbol, GError **err)
 }
 
 static const GdkPixbuf*
-isymbol_icon (IAnjutaSymbol *isymbol, GError **err)
+isymbol_get_icon (IAnjutaSymbol *isymbol, GError **err)
 {
 	AnjutaSymbol *s;
 	SVNodeType node_type;
@@ -263,23 +260,19 @@ isymbol_icon (IAnjutaSymbol *isymbol, GError **err)
 static void
 isymbol_iface_init (IAnjutaSymbolIface *iface)
 {
-	iface->type = isymbol_type;
-	iface->type_str = isymbol_type_str;	
-	iface->type_name = isymbol_type_name;
-	iface->name = isymbol_name;
-	iface->args = isymbol_args;
-	iface->scope = isymbol_scope;
-	iface->inheritance = isymbol_inheritance;	
-	iface->access = isymbol_access;
-	iface->impl = isymbol_impl;
-	iface->uri = isymbol_uri;
-	iface->line = isymbol_line;
+	iface->get_uri = isymbol_uri;
+	iface->get_name = isymbol_get_name;	
+	iface->get_line = isymbol_get_line;
 	iface->is_local = isymbol_is_local;
-	iface->icon = isymbol_icon;
+	iface->get_sym_type = isymbol_get_sym_type;
+	iface->get_args = isymbol_get_args;
+	iface->get_extra_info_string = isymbol_get_extra_info_string;
+	iface->get_icon = isymbol_get_icon;
 }
 
+
 ANJUTA_TYPE_BEGIN (AnjutaSymbol, anjuta_symbol, G_TYPE_OBJECT);
 ANJUTA_TYPE_ADD_INTERFACE (isymbol, IANJUTA_TYPE_SYMBOL);
 ANJUTA_TYPE_END;
 
-/* IAnjutaIterable implementation */
+
