@@ -32,7 +32,6 @@
 #include <libanjuta/interfaces/ianjuta-markable.h>
 #include <libanjuta/interfaces/ianjuta-symbol-manager.h>
 #include <libanjuta/interfaces/ianjuta-preferences.h>
-#include <libegg/menu/egg-combo-action.h>
 
 #include <tm_tagmanager.h>
 #include "an_symbol_view.h"
@@ -46,6 +45,7 @@
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-symbol-browser-plugin.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-symbol-browser-plugin.glade"
 #define ICON_FILE "anjuta-symbol-browser-plugin-48.png"
+#define SYMBOL_BROWSER_TAGS "symbol.browser.tags"
 
 #define TIMEOUT_INTERVAL_SYMBOLS_UPDATE		10000
 
@@ -90,7 +90,7 @@ goto_file_line (AnjutaPlugin *plugin, const gchar *filename, gint lineno)
 										 NULL);
 		
 	uri = gnome_vfs_get_uri_from_local_path (filename);
-	ianjuta_document_manager_goto_file_line (docman, uri, lineno, NULL);
+	ianjuta_document_manager_goto_uri_line (docman, uri, lineno, NULL);
 	g_free (uri);
 }
 
@@ -351,6 +351,8 @@ on_project_element_removed (IAnjutaProjectManager *pm, const gchar *uri,
 	}
 }
 
+#define LOCAL_TAGS_DIR ".anjuta/tags"
+
 // add a new project
 static void
 project_root_added (AnjutaPlugin *plugin, const gchar *name,
@@ -360,6 +362,7 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 	IAnjutaProjectManager *pm;
 	SymbolBrowserPlugin *sv_plugin;
 	const gchar *root_uri;
+	gchar* tags;
 
 	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	  
@@ -394,7 +397,31 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 					  G_CALLBACK (on_project_element_added), sv_plugin);
 	g_signal_connect (G_OBJECT (pm), "element_removed",
 					  G_CALLBACK (on_project_element_removed), sv_plugin);
-}
+	/* Load from project */
+	if (anjuta_preferences_get_int (sv_plugin->prefs,
+											"symbol_browser.tags_auto_load"))
+	{
+		gchar* dirname = g_build_filename (g_get_home_dir (), LOCAL_TAGS_DIR, NULL);
+		GList* packages = ianjuta_project_manager_get_packages (pm, NULL);
+		GList* node;
+		GString* str = g_string_new ("");
+		for (node = packages; node != NULL; node = g_list_next (node))
+		{
+			gchar* pathname = g_build_filename (dirname, node->data, NULL);
+			if (str->len)
+				g_string_append_c (str, ':');
+			g_string_append (str, pathname);
+			g_string_append (str, ".anjutatags.gz");
+			g_free (pathname);
+		}
+		g_list_foreach (packages, (GFunc) g_free, NULL);
+		g_list_free (packages);
+		anjuta_preferences_set (sv_plugin->prefs, SYMBOL_BROWSER_TAGS, str->str);
+		g_string_free (str, TRUE);
+		g_free (dirname);
+		symbol_browser_load_global_tags (plugin);
+	}
+}	   
 
 static void
 project_root_removed (AnjutaPlugin *plugin, const gchar *name,
@@ -543,17 +570,6 @@ goto_tree_iter (SymbolBrowserPlugin *sv_plugin, GtkTreeIter *iter)
 	}
 }
 
-static void
-on_symbol_selected (GtkAction *action, SymbolBrowserPlugin *sv_plugin)
-{
-	GtkTreeIter iter;
-	
-	if (egg_combo_action_get_active_iter (EGG_COMBO_ACTION (action), &iter))
-	{
-		goto_tree_iter (sv_plugin, &iter);
-	}
-}
-
 static void
 on_local_treeview_row_activated (GtkTreeView *view, GtkTreePath *arg1,
 								 GtkTreeViewColumn *arg2,
@@ -644,8 +660,6 @@ on_editor_saved (IAnjutaEditor *editor, const gchar *saved_uri,
 		
 		/* Update File symbol view */
 		ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (sv_plugin)->shell, NULL);
-		action = anjuta_ui_get_action (ui, "ActionGroupSymbolNavigation",
-									   "ActionGotoSymbol");
 		
 		
 		file_symbol_model =
@@ -653,18 +667,11 @@ on_editor_saved (IAnjutaEditor *editor, const gchar *saved_uri,
 		g_object_set_data (G_OBJECT (editor), "tm_file",
 						   g_object_get_data (G_OBJECT (file_symbol_model),
 											  "tm_file"));
-		/* Set toolbar version */
-		egg_combo_action_set_model (EGG_COMBO_ACTION (action), file_symbol_model);
 		/* Set local view version */
 		gtk_tree_view_set_model (GTK_TREE_VIEW (sv_plugin->sl_tree),
 								 file_symbol_model);
 		sv_plugin->locals_line_number = 0;
 		on_editor_update_ui (editor, sv_plugin);
-		
-		if (gtk_tree_model_iter_n_children (file_symbol_model, NULL) > 0)
-			g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
-		else
-			g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 
 #if 0
 		/* FIXME: Re-hilite all editors on tags update */
@@ -733,8 +740,6 @@ update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin)
 		g_free (local_filename);
 		
 		anjuta_symbol_view_workspace_add_file (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree), uri);
-		action = anjuta_ui_get_action (ui, "ActionGroupSymbolNavigation",
-									   "ActionGotoSymbol");
 		g_free (uri);
 		
 		file_symbol_model =
@@ -744,19 +749,12 @@ update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin)
 			g_object_set_data (G_OBJECT (editor), "tm_file",
 						   g_object_get_data (G_OBJECT (file_symbol_model),
 											  "tm_file"));
-			/* Set toolbar version */
-			egg_combo_action_set_model (EGG_COMBO_ACTION (action), file_symbol_model);
 			
 			/* Set local view version */
 			gtk_tree_view_set_model (GTK_TREE_VIEW (sv_plugin->sl_tree),
 									 file_symbol_model);
 			sv_plugin->locals_line_number = 0;
 			on_editor_update_ui (IANJUTA_EDITOR (editor), sv_plugin);
-			
-			if (gtk_tree_model_iter_n_children (file_symbol_model, NULL) > 0)
-				g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
-			else
-				g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 		}
 	}
 }
@@ -770,11 +768,7 @@ iter_matches (SymbolBrowserPlugin *sv_plugin, GtkTreeIter* iter,
 	if (line == lineno)
 	{
 		GtkTreePath* path = gtk_tree_model_get_path (model, iter);
-		GtkAction* action = anjuta_ui_get_action (sv_plugin->ui, 
-												  "ActionGroupSymbolNavigation",
-												  "ActionGotoSymbol");
 		
-		egg_combo_action_set_active_iter (EGG_COMBO_ACTION (action), iter);
 		gtk_tree_view_set_cursor (GTK_TREE_VIEW (sv_plugin->sl_tree), path, NULL,
 								 FALSE);
 		gtk_tree_path_free (path);
@@ -875,9 +869,6 @@ value_removed_current_editor (AnjutaPlugin *plugin,
 	
 	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	action = anjuta_ui_get_action (ui, "ActionGroupSymbolNavigation",
-								   "ActionGotoSymbol");
-	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 	sv_plugin->current_editor = NULL;
 }
 
@@ -885,7 +876,6 @@ static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
 	GtkActionGroup *group;
-	GtkAction *action;
 	SymbolBrowserPlugin *sv_plugin;
 	
 	DEBUG_PRINT ("SymbolBrowserPlugin: Activating Symbol Manager plugin...");
@@ -906,7 +896,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv_plugin->sl),
 									GTK_POLICY_AUTOMATIC,
 									GTK_POLICY_AUTOMATIC);
-	
+	/* Local symbols of the file */
 	sv_plugin->sl_tab_label = gtk_label_new (_("Local" ));
 	sv_plugin->sl_tree = anjuta_symbol_locals_new ();
 	g_object_add_weak_pointer (G_OBJECT (sv_plugin->sl_tree),
@@ -926,7 +916,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv_plugin->sv),
 									GTK_POLICY_AUTOMATIC,
 									GTK_POLICY_AUTOMATIC);
-	
+	/* Global project-wide symbols */
 	sv_plugin->sv_tab_label = gtk_label_new (_("Global" ));
 	sv_plugin->sv_tree = anjuta_symbol_view_new ();
 	symbol_browser_load_global_tags (plugin);
@@ -983,19 +973,7 @@ activate_plugin (AnjutaPlugin *plugin)
 											G_N_ELEMENTS (popup_actions),
 											GETTEXT_PACKAGE, FALSE, plugin);
 	group = gtk_action_group_new ("ActionGroupSymbolNavigation");
-
-	/* create a new combobox in style of libegg... */
-	action = g_object_new (EGG_TYPE_COMBO_ACTION,
-						   "name", "ActionGotoSymbol",
-						   "label", _("Goto symbol"),
-						   "tooltip", _("Select the symbol to go"),
-						   "stock_id", GTK_STOCK_JUMP_TO,
-							NULL);
-
-	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
-	g_signal_connect (action, "activate",
-					  G_CALLBACK (on_symbol_selected), sv_plugin);
-	gtk_action_group_add_action (group, action);
+	
 	anjuta_ui_add_action_group (sv_plugin->ui, "ActionGroupSymbolNavigation",
 								N_("Symbol navigations"), group, FALSE);
 	sv_plugin->action_group_nav = group;
@@ -1090,14 +1068,14 @@ dispose (GObject *obj)
 										   (gpointer)&sv_plugin->ss);
 
 	
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
 finalize (GObject *obj)
 {
 	/* SymbolBrowserPlugin *plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (obj); */
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -1133,9 +1111,13 @@ symbol_browser_plugin_class_init (SymbolBrowserPluginClass *klass)
 static IAnjutaIterable*
 isymbol_manager_search (IAnjutaSymbolManager *sm,
 						IAnjutaSymbolType match_types,
+						gboolean include_types,
+						IAnjutaSymbolField info_fields,
 						const gchar *match_name,
 						gboolean partial_name_match,
 						gboolean global_search,
+						gint results_limit, /* unused */
+						gint results_offset, /* unused */
 						GError **err)
 {
 	const GPtrArray *tags_array;
@@ -1154,19 +1136,23 @@ isymbol_manager_search (IAnjutaSymbolManager *sm,
 		iter = anjuta_symbol_iter_new (tags_array);
 		return IANJUTA_ITERABLE (iter);
 	}
-	return NULL;
+	return NULL;	
 }
 
 static IAnjutaIterable*
 isymbol_manager_get_members (IAnjutaSymbolManager *sm,
-							 const gchar *symbol_name,
+							 IAnjutaSymbol *symbol, 
+							 IAnjutaSymbolField info_fields,
 							 gboolean global_search,
 							 GError **err)
 {
 	const GPtrArray *tags_array;
 	AnjutaSymbolIter *iter = NULL;
+	AnjutaSymbol *s;
 	
-	tags_array = tm_workspace_find_scope_members (NULL, symbol_name,
+	s = ANJUTA_SYMBOL (symbol);
+	
+	tags_array = tm_workspace_find_scope_members (NULL, anjuta_symbol_get_name (s),
 												  global_search, TRUE);
 												  
 	
@@ -1179,14 +1165,18 @@ isymbol_manager_get_members (IAnjutaSymbolManager *sm,
 }
 
 static IAnjutaIterable*
-isymbol_manager_get_parents (IAnjutaSymbolManager *sm,
-							 const gchar *symbol_name,
+isymbol_manager_get_class_parents (IAnjutaSymbolManager *sm,
+							 IAnjutaSymbol *symbol,
+							 IAnjutaSymbolField info_fields,
 							 GError **err)
 {
 	const GPtrArray *tags_array;
 	AnjutaSymbolIter *iter = NULL;
+	AnjutaSymbol *s;
 	
-	tags_array = tm_workspace_get_parents (symbol_name);
+	s = ANJUTA_SYMBOL (symbol);
+	
+	tags_array = tm_workspace_get_parents (anjuta_symbol_get_name (s));
 	if (tags_array && tags_array->len)
 	{
 		iter = anjuta_symbol_iter_new (tags_array);
@@ -1200,7 +1190,7 @@ isymbol_manager_iface_init (IAnjutaSymbolManagerIface *iface)
 {
 	iface->search = isymbol_manager_search;
 	iface->get_members = isymbol_manager_get_members;
-	iface->get_parents = isymbol_manager_get_parents;
+	iface->get_class_parents = isymbol_manager_get_class_parents;
 }
 
 static void
