@@ -26,7 +26,6 @@
 #include <gtk/gtk.h>
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
-#include <libegg/menu/egg-combo-action.h>
 
 #include "symbol-db-view-locals.h"
 #include "symbol-db-engine.h"
@@ -44,18 +43,6 @@ enum {
 
 static GtkTreeViewClass *parent_class = NULL;
 
-struct _SymbolDBViewLocalsPriv {
-	gchar *current_db_file;
-	gchar *current_local_file_path;
-	gint insert_handler;
-	gint update_handler;
-	gint remove_handler;	
-	gint scan_end_handler;
-	
-	GTree *nodes_displayed;
-	GTree *waiting_for;	
-	
-};
 
 typedef struct _WaitingForSymbol {
 	gint child_symbol_id;
@@ -70,6 +57,31 @@ typedef struct _TraverseData {
 
 } TraverseData;
 
+typedef struct _FileSymbolsStatus {
+	GtkTreeStore *store;
+	GTree *nodes_displayed;
+	GTree *waiting_for;	
+	GQueue *symbols_inserted_ids;
+	
+} FileSymbolsStatus;
+
+struct _SymbolDBViewLocalsPriv {
+	gchar *current_db_file;
+	gchar *current_local_file_path;
+	gint insert_handler;
+	gint remove_handler;	
+	gint scan_end_handler;
+	gint insertion_idle_handler;
+	
+	GTree *nodes_displayed;
+	GTree *waiting_for;	
+	GQueue *symbols_inserted_ids;
+
+	gboolean recv_signals;
+	GHashTable *files_view_status;
+};
+
+
 static void
 trigger_on_symbol_inserted (SymbolDBViewLocals *dbvl, gint symbol_id);
 
@@ -88,6 +100,148 @@ waiting_for_symbol_destroy (WaitingForSymbol *wfs)
 	g_free (wfs);
 }
 
+static gboolean
+traverse_free_waiting_for (gpointer key, gpointer value, gpointer data)
+{
+	if (value == NULL || key == NULL)
+		return FALSE;
+		
+	g_slist_foreach ((GSList*)value, (GFunc)waiting_for_symbol_destroy, NULL);
+	g_slist_free ((GSList*)value);	
+	return FALSE;
+}
+
+static void
+file_view_status_destroy (FileSymbolsStatus *fss)
+{
+	g_object_unref (fss->store);
+	
+	if (fss->nodes_displayed)
+		g_tree_destroy (fss->nodes_displayed);
+	
+	/* free the waiting_for structs before destroying the tree itself */
+	if (fss->waiting_for)
+	{
+		g_tree_foreach (fss->waiting_for, traverse_free_waiting_for, NULL);
+		g_tree_destroy (fss->waiting_for);
+	}	
+	
+	g_free (fss);
+}
+
+static GtkTreeStore *
+sdb_view_locals_create_new_store ()
+{
+	GtkTreeStore *store;
+	store = gtk_tree_store_new (COLUMN_MAX, GDK_TYPE_PIXBUF,
+				    G_TYPE_STRING, G_TYPE_INT);	
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+										  COLUMN_NAME,
+										  GTK_SORT_ASCENDING);
+	return store;
+}
+
+static gboolean
+traverse_files_view_status (gpointer key, gpointer value, gpointer user_data)
+{
+	return TRUE;
+}
+
+/* the function will check for a NULL row_ref of its associated path. In that
+ * case will return FALSE
+ */
+static inline gboolean
+sdb_view_locals_get_iter_from_row_ref (SymbolDBViewLocals *dbvl, 
+									   GtkTreeRowReference *row_ref, 
+									   GtkTreeIter *OUT_iter)
+{
+	GtkTreePath *path;
+	if (row_ref == NULL) 
+	{
+		/* no node displayed found */
+		return FALSE;
+	}
+			
+	path = gtk_tree_row_reference_get_path (row_ref);
+	if (path == NULL) 
+	{
+		DEBUG_PRINT ("sdb_view_locals_get_iter_from_row_ref (): path is null, something "
+					 "went wrong ?!");
+		return FALSE;
+	}
+		
+	if (gtk_tree_model_get_iter (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)),
+                             OUT_iter, path) == FALSE) 
+	{
+		gtk_tree_path_free (path);
+		return FALSE;
+	}
+	gtk_tree_path_free (path);
+	
+	return TRUE;
+}
+
+void 
+symbol_db_view_locals_clear_cache (SymbolDBViewLocals *dbvl)
+{
+	SymbolDBViewLocalsPriv *priv;	
+	GtkTreeStore *store;
+	gpointer hash_node = NULL;
+	
+	g_return_if_fail (dbvl != NULL);
+	
+	priv = dbvl->priv;
+		
+	DEBUG_PRINT ("symbol_db_view_locals_clear_cache ()");
+	/* check whether we already have the view status saved on hash table or not.
+	 * If we saved that then don't remove it, or there may be a memory leak
+	 */	
+	if (priv->current_db_file != NULL)
+	{
+		DEBUG_PRINT ("priv->current_db_file %s ", priv->current_db_file);
+		hash_node = g_hash_table_lookup (priv->files_view_status, 
+										 priv->current_db_file);
+	}
+	
+	if (hash_node == NULL)
+	{
+		if (priv->nodes_displayed)
+		{
+			g_tree_destroy (priv->nodes_displayed);
+			priv->nodes_displayed = NULL;
+		}
+	
+		/* free the waiting_for structs before destroying the tree itself */
+		if (priv->waiting_for)
+		{
+			g_tree_foreach (priv->waiting_for, traverse_free_waiting_for, NULL);
+			g_tree_destroy (priv->waiting_for);
+			priv->waiting_for = NULL;
+		}
+
+		store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));
+		if (store != NULL)
+			g_object_unref (store);
+	}
+	
+	gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), NULL);
+	
+	g_free (priv->current_db_file);
+	priv->current_db_file = NULL;
+	
+	g_free (priv->current_local_file_path);
+	priv->current_local_file_path = NULL;
+	
+	if (priv->files_view_status)
+	{
+		g_hash_table_foreach_remove (priv->files_view_status, 
+									 traverse_files_view_status, NULL);
+	}
+	
+	priv->waiting_for = NULL;
+	priv->nodes_displayed = NULL;
+}
+
 
 static void
 sdb_view_locals_init (SymbolDBViewLocals *dbvl)
@@ -106,14 +260,20 @@ sdb_view_locals_init (SymbolDBViewLocals *dbvl)
 		
 	priv->current_db_file = NULL;
 	priv->current_local_file_path = NULL;
-	priv->insert_handler = -1;
-	priv->update_handler = -1;
-	priv->remove_handler = -1;
 	priv->nodes_displayed = NULL;
 	priv->waiting_for = NULL;
-	
-	store = gtk_tree_store_new (COLUMN_MAX, GDK_TYPE_PIXBUF,
-				    G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);
+	priv->symbols_inserted_ids = NULL;
+	priv->insert_handler = 0;
+	priv->scan_end_handler = 0;
+	priv->remove_handler = 0;
+	priv->insertion_idle_handler = 0;
+	priv->files_view_status = g_hash_table_new_full (g_str_hash, 
+								g_str_equal, g_free, (GDestroyNotify)file_view_status_destroy);
+
+	priv->recv_signals = FALSE;
+
+	/* initially set it to NULL */
+	store = NULL;
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), GTK_TREE_MODEL (store));	
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dbvl), FALSE);
@@ -148,37 +308,21 @@ sdb_view_locals_init (SymbolDBViewLocals *dbvl)
 	 * gtk_tree_view_set_show_expanders (GTK_TREE_VIEW (dbvl), FALSE); */
 }
 
-static gboolean
-traverse_free_waiting_for (gpointer key, gpointer value, gpointer data)
-{
-	waiting_for_symbol_destroy ((WaitingForSymbol *)value);
-	return FALSE;
-}
-
 static void
 sdb_view_locals_finalize (GObject *object)
 {
-	SymbolDBViewLocals *locals = SYMBOL_DB_VIEW_LOCALS (object);
-	SymbolDBViewLocalsPriv *priv = locals->priv;
+	SymbolDBViewLocals *locals;
+	SymbolDBViewLocalsPriv *priv;
+	
+	g_return_if_fail (object != NULL);
+	
+	locals = SYMBOL_DB_VIEW_LOCALS (object);
+	priv = locals->priv;
 
 	DEBUG_PRINT ("finalizing symbol_db_view_locals ()");
-	
-	g_free (priv->current_db_file);
-	priv->current_db_file = NULL;
-	
-	g_free (priv->current_local_file_path);
-	priv->current_local_file_path = NULL;	
-	
-	if (priv->nodes_displayed)
-		g_tree_destroy (priv->nodes_displayed);
-	
-	/* free the waiting_for structs before destroying the tree itself */
-	if (priv->waiting_for)
-	{
-		g_tree_foreach (priv->waiting_for, traverse_free_waiting_for, NULL);
-		g_tree_destroy (priv->waiting_for);
-	}
-	
+
+	symbol_db_view_locals_clear_cache (locals);
+	g_hash_table_destroy (priv->files_view_status);
 	g_free (priv);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -243,8 +387,7 @@ do_add_root_symbol_to_view (SymbolDBViewLocals *dbvl, const GdkPixbuf *pixbuf,
 	
 	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));
  	
-	gtk_tree_store_append (store, &child_iter, NULL);
-			
+	gtk_tree_store_append (store, &child_iter, NULL);			
 	gtk_tree_store_set (store, &child_iter,
 		COLUMN_PIXBUF, pixbuf,
 		COLUMN_NAME, symbol_name,
@@ -276,27 +419,17 @@ do_add_child_symbol_to_view (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 	priv = dbvl->priv;	
 	
 	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));
-	
-	/* look into nodes_displayed g_tree for the gtktreepath of the parent iter,
-	 * get the gtktreeiter, and add a child 
-	 */
+
+	/* look up the row ref in the hashtable, then get its associated gtktreeiter */
 	row_ref = g_tree_lookup (priv->nodes_displayed, (gpointer)parent_symbol_id);
-	path = gtk_tree_row_reference_get_path (row_ref);
 	
-	if (path == NULL) {
-		DEBUG_PRINT ("do_add_symbol_to_view (): something went wrong.");
-		return NULL;		
-	}
-	
-	if (gtk_tree_model_get_iter (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)),
-                                 &iter, path) == FALSE) {									 
-		DEBUG_PRINT ("do_add_symbol_to_view (): iter was not set ?![%s %d] parent %d",
-					 symbol_name, symbol_id, parent_symbol_id);
+	if (sdb_view_locals_get_iter_from_row_ref (dbvl, row_ref, &iter) == FALSE)
+	{
+		g_warning ("do_add_symbol_to_view (): something went wrong.");
 		return NULL;
 	}
-
-	gtk_tree_path_free (path);
 	
+	/* append a new child &child_iter, with a parent of &iter */
 	gtk_tree_store_append (store, &child_iter, &iter);
 			
 	gtk_tree_store_set (store, &child_iter,
@@ -305,19 +438,12 @@ do_add_child_symbol_to_view (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 		COLUMN_SYMBOL_ID, symbol_id,
 		-1);	
 	
-	gchar *tmp_str = gtk_tree_path_to_string (
-					gtk_tree_model_get_path (
-						gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)),
-                                          &child_iter));
-
-	g_free (tmp_str);
-	
+	/* grab the row ref and return it */
 	path = gtk_tree_model_get_path (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)),
                                           &child_iter);
 	row_ref = gtk_tree_row_reference_new (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)), 
 										  path);
 	gtk_tree_path_free (path);
-	
 	return row_ref;
 }
 
@@ -409,6 +535,8 @@ traverse_on_scan_end (gpointer key, gpointer value, gpointer data)
 
 		/* now the waiters should be added as children */
 		trigger_on_symbol_inserted  (dbvl, parent_id);
+		
+		g_object_unref (iterator);
 	}
 	
 	/* continue the traversing */
@@ -416,124 +544,6 @@ traverse_on_scan_end (gpointer key, gpointer value, gpointer data)
 }
 
 
-static void
-on_scan_end (SymbolDBEngine *dbe, gpointer data)
-{
-	SymbolDBViewLocals *dbvl;
-	SymbolDBViewLocalsPriv *priv;
-	gint waiting_for_size;
-	TraverseData tdata;
-
-	dbvl = SYMBOL_DB_VIEW_LOCALS (data);
-	g_return_if_fail (dbvl != NULL);	
-	priv = dbvl->priv;
-
-	/* ok, symbol parsing has ended, are we sure that all the waiting_for
-	 * objects have been checked?
-	 * If it's not the case then try to add it to the on the root of the gtktreeview
-	 * and to trigger the insertion.
-	 */
-	if ((waiting_for_size = g_tree_nnodes (priv->waiting_for)) <= 0)
-		return;
-
-	/* we have something left. Search the parent_symbol_id [identified by the
-	 * waiting_for id]
-	 */
-	tdata.dbvl = dbvl;
-	tdata.dbe = dbe;
-
-	g_tree_foreach (priv->waiting_for, traverse_on_scan_end, &tdata);
-}
-
-static void
-do_recurse_subtree_and_remove (SymbolDBViewLocals *dbvl, 
-							   GtkTreeIter *parent_subtree_iter)
-{
-	gint curr_symbol_id;
-	const GdkPixbuf *curr_pixbuf;
-	GtkTreeStore *store;
-	gchar *curr_symbol_name;
-
-	SymbolDBViewLocalsPriv *priv;
-	
-	g_return_if_fail (dbvl != NULL);
-	
-	priv = dbvl->priv;
-	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));	
-	
-	gtk_tree_model_get (GTK_TREE_MODEL (store), parent_subtree_iter,
-				COLUMN_SYMBOL_ID, &curr_symbol_id,
-			    COLUMN_PIXBUF, &curr_pixbuf, 
-				COLUMN_NAME, &curr_symbol_name,	/* no strdup required */
-				-1);
-	
-	/*DEBUG_PRINT ("do_recurse_subtree_and_remove (): curr_symbol_id %d", 
-				 curr_symbol_id);*/
-				 
-	while (gtk_tree_model_iter_has_child  (GTK_TREE_MODEL (store), parent_subtree_iter)) 
-	{
-		GtkTreeIter child;
-		gtk_tree_model_iter_children (GTK_TREE_MODEL (store), &child, parent_subtree_iter);
-		
-		/* recurse */
-		do_recurse_subtree_and_remove (dbvl, &child);
-	}
-
-	gtk_tree_store_remove (store, parent_subtree_iter);
-	g_tree_remove (priv->nodes_displayed, (gpointer) curr_symbol_id);
-
-	/* don't forget to free this gchar */				   
-	g_free (curr_symbol_name);
-}
-
-
-static void 
-on_symbol_removed (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
-{
-	GtkTreeStore *store;
-	SymbolDBViewLocals *dbvl;
-	SymbolDBViewLocalsPriv *priv;
-    GtkTreeIter  iter;	
-	GtkTreeRowReference *row_ref;
-	GtkTreePath *path;
-	
-	dbvl = SYMBOL_DB_VIEW_LOCALS (data);
-
-	g_return_if_fail (dbvl != NULL);
-	priv = dbvl->priv;
-
-	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));		
-
-	/*DEBUG_PRINT ("on_symbol_removed (): -local- %d", symbol_id);		*/
-
-	row_ref = g_tree_lookup (priv->nodes_displayed, (gpointer)symbol_id);
-	if (row_ref == NULL) 
-	{
-		/*DEBUG_PRINT ("on_symbol_removed (): ERROR: cannot remove %d", symbol_id);*/
-		return;
-	}
- 
-	path = gtk_tree_row_reference_get_path (row_ref);
-	if (path == NULL) 
-	{
-		/*DEBUG_PRINT ("on_symbol_removed (): ERROR2: cannot remove %d", symbol_id);*/
-		return;
-	}
-	
-	if (gtk_tree_model_get_iter (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)),
-                                 &iter, path) == FALSE) 
-	{
-		DEBUG_PRINT ("on_symbol_removed (): iter was not set ?![%d]",
-					 symbol_id);
-		gtk_tree_path_free (path);
-		return;
-	}
-	gtk_tree_path_free (path);
-
-	do_recurse_subtree_and_remove (dbvl, &iter);
-}
-
-
 static void
 add_waiting_for_symbol_to_view (SymbolDBViewLocals *dbvl, WaitingForSymbol *wfs,
 								gint parent_symbol_id)
@@ -572,7 +582,7 @@ trigger_on_symbol_inserted (SymbolDBViewLocals *dbvl, gint symbol_id)
 	
 	priv = dbvl->priv;	
 
-	DEBUG_PRINT ("trigger_on_symbol_inserted (): triggering %d", symbol_id);
+/*	DEBUG_PRINT ("trigger_on_symbol_inserted (): triggering %d", symbol_id);*/
 	
 	/* try to find a waiting for symbol */
 	slist = g_tree_lookup (priv->waiting_for, (gpointer)symbol_id);
@@ -580,7 +590,7 @@ trigger_on_symbol_inserted (SymbolDBViewLocals *dbvl, gint symbol_id)
 	if (slist == NULL) 
 	{
 		/* nothing waiting for us */
-		/*DEBUG_PRINT ("trigger_on_symbol_inserted (): no children waiting for us...");*/
+/*		DEBUG_PRINT ("trigger_on_symbol_inserted (): no children waiting for us...");*/
 		return;
 	}
 	else {
@@ -588,8 +598,8 @@ trigger_on_symbol_inserted (SymbolDBViewLocals *dbvl, gint symbol_id)
 		gint length = g_slist_length (slist);
 
 /*		DEBUG_PRINT ("trigger_on_symbol_inserted (): consuming slist for parent %d",
-					 symbol_id);
-		DEBUG_PRINT ("trigger_on_symbol_inserted (): length is %d", length);*/
+					 symbol_id);*/
+
 		for (i=0; i < length-1; i++)
 		{
 			wfs = g_slist_nth_data (slist, 0);
@@ -630,7 +640,7 @@ add_new_waiting_for (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 	wfs->child_symbol_name = g_strdup (symbol_name);
 	wfs->pixbuf = pixbuf;
 				
-	/*DEBUG_PRINT ("add_new_waiting_for (): looking up waiting_for %d", 
+/*	DEBUG_PRINT ("add_new_waiting_for (): looking up waiting_for %d", 
 				 parent_symbol_id);*/
 	node = g_tree_lookup (priv->waiting_for, (gpointer)parent_symbol_id);
 	if (node == NULL) 
@@ -734,7 +744,7 @@ prepare_for_adding (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 	if (parent_symbol_id <= 0)
 	{			
 		GtkTreeRowReference *curr_tree_row_ref;
-		/*DEBUG_PRINT ("prepare_for_adding(): parent_symbol_id <= 0 root with id [%d]",
+/*		DEBUG_PRINT ("prepare_for_adding(): parent_symbol_id <= 0 root with id [%d]",
 					 symbol_id);*/
 		
 		/* get the current iter row reference in the just added root gtktreeview 
@@ -764,7 +774,7 @@ prepare_for_adding (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 		{
 			/* hey we found it */
 			GtkTreeRowReference *child_row_ref;
-			/*DEBUG_PRINT ("prepare_for_adding(): found node already displayed %d",
+/*			DEBUG_PRINT ("prepare_for_adding(): found node already displayed %d",
 						 parent_symbol_id);*/
 			
 			child_row_ref = do_add_child_symbol_to_view (dbvl, parent_symbol_id,
@@ -777,6 +787,10 @@ prepare_for_adding (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 		}
 		else 
 		{
+/*			DEBUG_PRINT ("prepare_for_adding(): gonna pass parent: %d name: %s "
+						 "id: %d to add_new_waiting_for", parent_symbol_id,
+						 symbol_name, symbol_id);*/
+			
 			/* add it to the waiting_for trigger list */
 			add_new_waiting_for (dbvl, parent_symbol_id, symbol_name, symbol_id, 
 								 pixbuf);
@@ -784,23 +798,84 @@ prepare_for_adding (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 	}
 }
 
-static void 
-on_symbol_inserted (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
+static void
+consume_symbols_inserted_queue_idle_destroy (gpointer data)
 {
+	TraverseData *tdata;
+	SymbolDBViewLocals *dbvl;
+	SymbolDBEngine *dbe;
+	SymbolDBViewLocalsPriv *priv;
+	gint waiting_for_size;
+	
+	tdata = (TraverseData *)data;
+
+	dbvl = tdata->dbvl;
+	dbe = tdata->dbe;	
+	g_return_if_fail (dbvl != NULL);
+	priv = dbvl->priv;
+	
+	tdata = (TraverseData *)data;
+
+	DEBUG_PRINT ("consume_symbols_inserted_queue_idle_destroy");
+	priv->insertion_idle_handler = 0;
+
+	if (g_queue_get_length (priv->symbols_inserted_ids) <= 0)
+	{
+		/* ok, symbol parsing has ended, are we sure that all the waiting_for
+	 	 * objects have been checked?
+	 	 * If it's not the case then try to add it to the on the root of the gtktreeview
+	 	 * and to trigger the insertion.
+	 	 */
+		if (priv->waiting_for == NULL || 
+			(waiting_for_size = g_tree_nnodes (priv->waiting_for)) <= 0)
+			return;
+
+		/* we have something left. Search the parent_symbol_id [identified by the
+	 	 * waiting_for id]
+	 	 */
+		DEBUG_PRINT ("destroying tdata");
+		g_tree_foreach (priv->waiting_for, traverse_on_scan_end, tdata);
+	}
+
+	g_free (tdata);
+}
+
+static gboolean
+consume_symbols_inserted_queue_idle (gpointer data)
+{
+	TraverseData *tdata;
+	SymbolDBViewLocals *dbvl;
+	SymbolDBEngine *dbe;
+	SymbolDBViewLocalsPriv *priv;
 	SymbolDBEngineIterator *iterator;
 	GtkTreeStore *store;
-	SymbolDBViewLocals *dbvl;
-	SymbolDBViewLocalsPriv *priv;
+	gint consumed_symbol_id;
+	gint queue_length;
 	
 	/* it's not obligatory referred to a class inheritance */
 	gint parent_symbol_id;
 	
-	dbvl = SYMBOL_DB_VIEW_LOCALS (data);
+	tdata = (TraverseData *)data;
 
-	g_return_if_fail (dbvl != NULL);	
-	priv = dbvl->priv;	
+	dbvl = tdata->dbvl;
+	dbe = tdata->dbe;	
+	g_return_val_if_fail (dbvl != NULL, FALSE);
+	priv = dbvl->priv;
+	
+	
+	queue_length = g_queue_get_length (priv->symbols_inserted_ids);
+	
+/*	DEBUG_PRINT ("consume_symbols_inserted_queue_idle [remaining %d]", queue_length);*/
+	
+	/* consume a symbol */
+	if (queue_length > 0)
+	{
+		consumed_symbol_id = (gint) g_queue_pop_head (priv->symbols_inserted_ids);
+	}
+	else {
+		return FALSE;
+	}
 	
-	/*DEBUG_PRINT ("on_symbol_inserted (): -local- %d", symbol_id);*/
 	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));
 	
 	/* again we use a little trick to insert symbols here. First of all forget chars
@@ -817,13 +892,17 @@ on_symbol_inserted (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 	 *
 	 */
 	parent_symbol_id = symbol_db_engine_get_parent_scope_id_by_symbol_id (dbe, 
-																	symbol_id,
-																	NULL);
-	
+															consumed_symbol_id,
+															priv->current_db_file);
+	/* try in a global fashion */
+	if (parent_symbol_id <= 0)
+		parent_symbol_id = symbol_db_engine_get_parent_scope_id_by_symbol_id (dbe, 
+															consumed_symbol_id,
+															NULL);
+		
 	/* get the original symbol infos */
-	iterator = symbol_db_engine_get_symbol_info_by_id (dbe, symbol_id, 
+	iterator = symbol_db_engine_get_symbol_info_by_id (dbe, consumed_symbol_id, 
 													   SYMINFO_SIMPLE |
-													   SYMINFO_FILE_PATH |
 													   SYMINFO_ACCESS |
 													   SYMINFO_KIND);	
 	
@@ -848,7 +927,8 @@ on_symbol_inserted (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 		 */
 		SymbolDBEngineIterator *iterator_for_children;
 		iterator_for_children = 
-			symbol_db_engine_get_scope_members_by_symbol_id (dbe, symbol_id, 
+			symbol_db_engine_get_scope_members_by_symbol_id (dbe, consumed_symbol_id, -1,
+															 -1,
 															 SYMINFO_SIMPLE);
 		if (iterator_for_children == NULL) 
 		{
@@ -865,7 +945,6 @@ on_symbol_inserted (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 				gint curr_child_id;
 				GtkTreeIter child_iter;
 				GtkTreeRowReference *row_ref;
-				GtkTreePath *path;
 				SymbolDBEngineIteratorNode *iter_node;
 
 				iter_node = SYMBOL_DB_ENGINE_ITERATOR_NODE (iterator_for_children);
@@ -873,47 +952,151 @@ on_symbol_inserted (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 				curr_child_id = 
 					symbol_db_engine_iterator_node_get_symbol_id (iter_node);
 
-				/*DEBUG_PRINT ("on_symbol_inserted (): %d has child %d",
-							 symbol_id, curr_child_id);*/
 				row_ref = g_tree_lookup (priv->nodes_displayed,
 										 (gpointer)curr_child_id);
-
-				if (row_ref == NULL) 
-				{
-					/* no node displayed found */
-					continue;
-				}
 				
-				path = gtk_tree_row_reference_get_path (row_ref);
-				if (path == NULL) 
-				{
-					DEBUG_PRINT ("on_symbol_inserted (): path is null, something "
-								 "went wrong ?!");
+				if (row_ref == NULL)
 					continue;
-				}
-		
-				if (gtk_tree_model_get_iter (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)),
-                                 &child_iter, path) == FALSE) 
+				
+				if (sdb_view_locals_get_iter_from_row_ref (dbvl, 
+								row_ref, &child_iter) == FALSE)
 				{
-					gtk_tree_path_free (path);
-					continue;		
-				}
-				gtk_tree_path_free (path);
+					g_warning ("on_symbol_inserted (): something went wrong");
+					continue;
+				}		
 				
 				/* put on waiting_for the subtree */
-				do_recurse_subtree_and_invalidate (dbvl, &child_iter, symbol_id);
+				do_recurse_subtree_and_invalidate (dbvl, &child_iter, consumed_symbol_id);
 			} while (symbol_db_engine_iterator_move_next (iterator_for_children) 
 					 == TRUE);
 			
 			g_object_unref (iterator_for_children);
 		}		
 		
-		prepare_for_adding (dbvl, parent_symbol_id, symbol_name, symbol_id, pixbuf);
+		prepare_for_adding (dbvl, parent_symbol_id, symbol_name, consumed_symbol_id, 
+							pixbuf);
 		
 		g_object_unref (iterator);
 	}	
 	
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (dbvl));
+	
+	return TRUE;
+}
+
+static void
+on_scan_end (SymbolDBEngine *dbe, gpointer data)
+{
+	SymbolDBViewLocals *dbvl;
+	SymbolDBViewLocalsPriv *priv;
+	TraverseData *tdata;
+
+	dbvl = SYMBOL_DB_VIEW_LOCALS (data);
+	g_return_if_fail (dbvl != NULL);
+	priv = dbvl->priv;
+
+	tdata = g_new (TraverseData, 1);
+	tdata->dbvl = dbvl;
+	tdata->dbe = dbe;
+	
+	DEBUG_PRINT ("locals: on_scan_end");
+	if (priv->symbols_inserted_ids != NULL)
+	{
+		if (g_queue_get_length (priv->symbols_inserted_ids) > 0) 
+		{
+			/* reverswe the queue */
+			g_queue_reverse (priv->symbols_inserted_ids);
+			
+			priv->insertion_idle_handler = g_idle_add_full (G_PRIORITY_LOW, 
+						 (GSourceFunc) consume_symbols_inserted_queue_idle,
+						 (gpointer) tdata,
+						 (GDestroyNotify) consume_symbols_inserted_queue_idle_destroy);
+		}
+	}
+}
+
+static void
+do_recurse_subtree_and_remove (SymbolDBViewLocals *dbvl, 
+							   GtkTreeIter *parent_subtree_iter)
+{
+	gint curr_symbol_id;
+	const GdkPixbuf *curr_pixbuf;
+	GtkTreeStore *store;
+	gchar *curr_symbol_name;
+
+	SymbolDBViewLocalsPriv *priv;
+	
+	g_return_if_fail (dbvl != NULL);
+	
+	priv = dbvl->priv;
+	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));	
+	
+	gtk_tree_model_get (GTK_TREE_MODEL (store), parent_subtree_iter,
+				COLUMN_SYMBOL_ID, &curr_symbol_id,
+			    COLUMN_PIXBUF, &curr_pixbuf, 
+				COLUMN_NAME, &curr_symbol_name,	/* no strdup required */
+				-1);
+	
+	/*DEBUG_PRINT ("do_recurse_subtree_and_remove (): curr_symbol_id %d", 
+				 curr_symbol_id);*/
+				 
+	while (gtk_tree_model_iter_has_child  (GTK_TREE_MODEL (store), parent_subtree_iter)) 
+	{
+		GtkTreeIter child;
+		gtk_tree_model_iter_children (GTK_TREE_MODEL (store), &child, parent_subtree_iter);
+		
+		/* recurse */
+		do_recurse_subtree_and_remove (dbvl, &child);
+	}
+
+	gtk_tree_store_remove (store, parent_subtree_iter);
+	g_tree_remove (priv->nodes_displayed, (gpointer) curr_symbol_id);
+
+	/* don't forget to free this gchar */				   
+	g_free (curr_symbol_name);
+}
+
+
+static void 
+on_symbol_removed (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
+{
+	GtkTreeStore *store;
+	SymbolDBViewLocals *dbvl;
+	SymbolDBViewLocalsPriv *priv;
+    GtkTreeIter  iter;	
+	GtkTreeRowReference *row_ref;
+	
+	dbvl = SYMBOL_DB_VIEW_LOCALS (data);
+
+	g_return_if_fail (dbvl != NULL);
+	priv = dbvl->priv;
+
+	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));		
+
+	DEBUG_PRINT ("on_symbol_removed (): -local- %d", symbol_id);
+
+	row_ref = g_tree_lookup (priv->nodes_displayed, (gpointer)symbol_id);
+	if (sdb_view_locals_get_iter_from_row_ref (dbvl, row_ref, &iter) == FALSE)
+	{
+		return;
+	}
+
+	do_recurse_subtree_and_remove (dbvl, &iter);
+}
+
+static void 
+on_symbol_inserted (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
+{
+	SymbolDBViewLocals *dbvl;
+	SymbolDBViewLocalsPriv *priv;
+	
+	dbvl = SYMBOL_DB_VIEW_LOCALS (data);
+
+	g_return_if_fail (dbvl != NULL);	
+	priv = dbvl->priv;	
+	
+	/* save the symbol_id to be added in the queue and just return */
+	g_queue_push_head (priv->symbols_inserted_ids, (gpointer)symbol_id);
 }
 
 gint
@@ -922,7 +1105,7 @@ symbol_db_view_locals_get_line (SymbolDBViewLocals *dbvl,
 								GtkTreeIter * iter) 
 {
 	GtkTreeStore *store;
-		
+
 	g_return_val_if_fail (dbvl != NULL, -1);
 	g_return_val_if_fail (dbe != NULL, -1);	
 	g_return_val_if_fail (iter != NULL, -1);
@@ -954,6 +1137,64 @@ symbol_db_view_locals_get_line (SymbolDBViewLocals *dbvl,
 	return -1;
 }								
 
+void
+symbol_db_view_locals_recv_signals_from_engine (SymbolDBViewLocals *dbvl, 
+							SymbolDBEngine *dbe, gboolean enable_status)
+{
+	SymbolDBViewLocalsPriv *priv;
+
+	g_return_if_fail (dbvl != NULL);
+	priv = dbvl->priv;		
+	
+	if (enable_status == TRUE) 
+	{
+		gtk_widget_set_sensitive (GTK_WIDGET (dbvl), TRUE);
+		
+		priv->recv_signals = TRUE;
+		/* connect some signals */
+		if (priv->insert_handler <= 0) 
+		{
+			priv->insert_handler = 	g_signal_connect (G_OBJECT (dbe), 
+						"symbol-inserted", G_CALLBACK (on_symbol_inserted), dbvl);
+		}
+
+		if (priv->remove_handler <= 0)
+		{
+			priv->remove_handler = g_signal_connect (G_OBJECT (dbe), "symbol-removed",
+						  G_CALLBACK (on_symbol_removed), dbvl);
+		}	
+
+		if (priv->scan_end_handler <= 0)
+		{
+			priv->scan_end_handler = g_signal_connect (G_OBJECT (dbe), "scan-end",
+						  G_CALLBACK (on_scan_end), dbvl);
+		}
+	}
+	else		/* disconnect them, if they were ever connected before */
+	{
+		gtk_widget_set_sensitive (GTK_WIDGET (dbvl), FALSE);
+		
+		priv->recv_signals = FALSE;
+		if (priv->insert_handler >= 0) 
+		{
+			g_signal_handler_disconnect (G_OBJECT (dbe), priv->insert_handler);
+			priv->insert_handler = 0;
+		}
+
+		if (priv->remove_handler >= 0)
+		{
+			g_signal_handler_disconnect (G_OBJECT (dbe), priv->remove_handler);
+			priv->remove_handler = 0;
+		}	
+
+		if (priv->scan_end_handler >= 0)
+		{
+			g_signal_handler_disconnect (G_OBJECT (dbe), priv->scan_end_handler);
+			priv->scan_end_handler = 0;
+		}
+	}
+}
+
 void
 symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe,
 							  const gchar* filepath)
@@ -961,97 +1202,186 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 	SymbolDBViewLocalsPriv *priv;
 
 	SymbolDBEngineIterator *iterator;
-	GtkTreeStore *store;
-
+	GtkTreeStore *store;	
+	FileSymbolsStatus *fsstatus;
+	
 	g_return_if_fail (dbvl != NULL);
 	g_return_if_fail (filepath != NULL);
 	g_return_if_fail (dbe != NULL);
-		
-	/*DEBUG_PRINT ("symbol_db_view_locals_update_list  () for %s", filepath);*/
-	
+
 	priv = dbvl->priv;
+
+	/* we're not interested in giving user an updated gtktreestore if recv signals
+	 * is false. In that case we can have a project importing...
+	 */
+	if (priv->recv_signals == FALSE)
+	{		
+		gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), NULL);
+		return;
+	}
 	
+	/* ok, we can have a case where we're revisiting an old file with an already 
+	 * populated GtkTreeStore. We're gonna set that gtktreestore along with the
+	 * GTree(s) for nodes_displayed and waiting_for.
+	 */
+	fsstatus = NULL;
+	
+	if (priv->current_db_file != NULL)
+	{
+		FileSymbolsStatus *hash_node;
+		/* save current symbols status - e.g. gtktreestore, symbols_displayed etc.,
+		 * if it hasn't already been done.
+		 */
+		hash_node = g_hash_table_lookup (priv->files_view_status, 
+										 priv->current_db_file);
+		
+		/* did we find something? yes? well, then we should save nothing... */
+		if (hash_node == NULL)
+		{
+			/* found nothing? ok, save the status */
+			GtkTreeStore * store;
+		
+			/* remove the GSourceFunc, if it's running */
+			if (priv->insertion_idle_handler > 0)
+			{				
+				g_source_remove (priv->insertion_idle_handler);
+				priv->insertion_idle_handler = 0;
+			}
+			
+			store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));	
+			
+			if (store != NULL && priv->nodes_displayed != NULL 
+				&& priv->waiting_for != NULL)
+			{
+				FileSymbolsStatus *fss;
+				fss = g_new0 (FileSymbolsStatus, 1);
+				fss->store = store;
+				fss->nodes_displayed = priv->nodes_displayed;
+				fss->waiting_for = priv->waiting_for;
+				fss->symbols_inserted_ids = priv->symbols_inserted_ids;
+				
+				DEBUG_PRINT ("symbol_db_view_locals_update_list (): g_hash_table_insert ");
+				/* insert it */
+				g_hash_table_insert (priv->files_view_status, 
+									 g_strdup (priv->current_db_file), fss);
+			}			
+		}
+	}
+	
+
 	/* adjust some vars */
 	g_free (priv->current_db_file);
+	priv->current_db_file = NULL;
+	
 	g_free (priv->current_local_file_path);
+	priv->current_local_file_path = NULL;
 
 	priv->current_db_file = 
 		symbol_db_engine_get_file_db_path (dbe, filepath);
-	priv->current_local_file_path = g_strdup (filepath);		
+	if (priv->current_db_file == NULL) 
+	{
+		DEBUG_PRINT ("symbol_db_view_locals_update_list (): "
+					 "Warning: priv->current_db_file is NULL");
+		return;
+	}
+	priv->current_local_file_path = g_strdup (filepath);
 	
-	priv->nodes_displayed = g_tree_new_full ((GCompareDataFunc)&gtree_compare_func, 
+	/* try to see if we had something saved before in the hash table */
+	fsstatus = g_hash_table_lookup (priv->files_view_status,
+									priv->current_db_file);	
+
+	if (fsstatus != NULL)
+	{
+		/* restore the previous saved status ... */
+		
+		/* set the nodes_displayed */
+		priv->nodes_displayed = fsstatus->nodes_displayed;
+		
+		/* ... the waiting_for ... */
+		priv->waiting_for = fsstatus->waiting_for;
+	
+		/* ... the pending symbols_id to be inserted ... */
+		priv->symbols_inserted_ids = fsstatus->symbols_inserted_ids;
+
+		/* and last but not the least the store */
+		store = fsstatus->store;		
+
+		/* with this set there's no need to re-retrieve the symbols from db,
+		 * speeding up the things.
+		 */
+		DEBUG_PRINT ("symbol_db_view_locals_update_list (): "
+					 "setting gtk_tree_view_set_model to the saved one");
+		gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), GTK_TREE_MODEL (store));
+		
+		
+		/* check if we left some ids on the queue. In case restart the idle_function */
+		if (g_queue_get_length (priv->symbols_inserted_ids) > 0)
+		{
+			TraverseData *tdata;
+			
+			tdata = g_new0 (TraverseData, 1);
+			tdata->dbvl = dbvl;
+			tdata->dbe = dbe;
+			
+			priv->insertion_idle_handler = g_idle_add_full (G_PRIORITY_LOW, 
+						 (GSourceFunc) consume_symbols_inserted_queue_idle,
+						 (gpointer) tdata,
+						 (GDestroyNotify) consume_symbols_inserted_queue_idle_destroy);			
+		}		
+	}
+	else 
+	{	
+		priv->nodes_displayed = g_tree_new_full ((GCompareDataFunc)&gtree_compare_func, 
 										 NULL,
 										 NULL,
-										 (GDestroyNotify)&gtk_tree_row_reference_free);
+										 (GDestroyNotify)&gtk_tree_row_reference_free);		
 
-	priv->waiting_for = g_tree_new_full ((GCompareDataFunc)&gtree_compare_func, 
-									 NULL,
-									 NULL,
-									 NULL);
+		priv->waiting_for = g_tree_new_full ((GCompareDataFunc)&gtree_compare_func, 
+										 NULL,
+										 NULL,
+										 NULL);
+		
+		priv->symbols_inserted_ids = g_queue_new ();
 
-
-	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (dbvl)));	
-	
-	/* Removes all rows from tree_store */
-	gtk_tree_store_clear (store);
-
-	iterator = symbol_db_engine_get_file_symbols (dbe, filepath, SYMINFO_SIMPLE |
+		DEBUG_PRINT ("symbol_db_view_locals_update_list (): creating new store"); 
+		store = sdb_view_locals_create_new_store ();
+		gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), GTK_TREE_MODEL (store));
+		
+		/* Removes all rows from tree_store */
+		gtk_tree_store_clear (store);
+		iterator = symbol_db_engine_get_file_symbols (dbe, filepath, SYMINFO_SIMPLE |
 												  	SYMINFO_ACCESS |
 													SYMINFO_KIND);		
-	
-	if (iterator != NULL)
-	{
-		do {
-			gint curr_symbol_id;
-			SymbolDBEngineIteratorNode *iter_node;
+	 	
+		if (iterator != NULL)
+		{
+			do {
+				gint curr_symbol_id;
+				SymbolDBEngineIteratorNode *iter_node;
 
-			iter_node = SYMBOL_DB_ENGINE_ITERATOR_NODE (iterator);
+				iter_node = SYMBOL_DB_ENGINE_ITERATOR_NODE (iterator);
 			
-			curr_symbol_id = symbol_db_engine_iterator_node_get_symbol_id (iter_node);
+				curr_symbol_id = symbol_db_engine_iterator_node_get_symbol_id (iter_node);
 
-			/* we can just call the symbol inserted function. It'll think about
-			 * building the tree and populating it
-			 */
-			on_symbol_inserted (dbe, curr_symbol_id, (gpointer)dbvl);
-		} while (symbol_db_engine_iterator_move_next (iterator) == TRUE);
+				/* we can just call the symbol inserted function. It'll take care of
+			 	* building the tree and populating it
+			 	*/
+				on_symbol_inserted (dbe, curr_symbol_id, (gpointer)dbvl);
+			} while (symbol_db_engine_iterator_move_next (iterator) == TRUE);
 		
-		g_object_unref (iterator);
+			g_object_unref (iterator);
+		}
+
+		/* ok, there may be some symbols left on the waiting_for_list...
+ 	 	* launch the callback function by hand, flushing the list it in case 
+	 	*/
+		on_scan_end (dbe, dbvl);		
 	}
-
-	DEBUG_PRINT ("symbol_db_view_locals_update_list (): waiting for displaying: %d", 
-				 g_tree_nnodes (priv->waiting_for));
-	DEBUG_PRINT ("symbol_db_view_locals_update_list (): already displayed: %d", 
-				 g_tree_nnodes (priv->nodes_displayed));
-
-	/* ok, there may be some symbols left on the waiting_for_list...
- 	 * launch the callback function by hand, flushing the list it in case 
-	 */
-	on_scan_end (dbe, dbvl);
-
 	
 	/* only gtk 2.12 ...
 	 * gtk_tree_view_set_show_expanders (GTK_TREE_VIEW (dbvl)); */
 	
-	gtk_tree_view_expand_all (GTK_TREE_VIEW (dbvl));
-	
-	/* connect some signals */
-	if (priv->insert_handler <= 0) 
-	{
-		priv->insert_handler = 	g_signal_connect (G_OBJECT (dbe), "symbol_inserted",
-					  G_CALLBACK (on_symbol_inserted), dbvl);
-	}
-
-	if (priv->remove_handler <= 0)
-	{
-		priv->remove_handler = g_signal_connect (G_OBJECT (dbe), "symbol_removed",
-					  G_CALLBACK (on_symbol_removed), dbvl);
-	}	
-
-	if (priv->scan_end_handler <= 0)
-	{
-		priv->remove_handler = g_signal_connect (G_OBJECT (dbe), "scan_end",
-					  G_CALLBACK (on_scan_end), dbvl);
-	}	
+	gtk_tree_view_expand_all (GTK_TREE_VIEW (dbvl));	
 }
 
  
