@@ -23,7 +23,6 @@
 #include <signal.h>
 
 #include <libanjuta/anjuta-shell.h>
-#include <libanjuta/anjuta-children.h>
 #include <libanjuta/anjuta-debug.h>
 
 #include <libanjuta/interfaces/ianjuta-terminal.h>
@@ -31,9 +30,9 @@
 
 #include <signal.h>
 
-#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-terminal.ui"
+#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-terminal-plugin.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-terminal-plugin.glade"
-#define ICON_FILE "preferences-terminal.png"
+#define ICON_FILE "anjuta-terminal-plugin-48.png"
 
 /* Some desktop/gnome-terminal gconf keys. */
 #define GCONF_MONOSPACE_FONT      "/desktop/gnome/interface/monospace_font_name"
@@ -60,12 +59,11 @@
 
 #include <gconf/gconf-client.h>
 #include <vte/vte.h>
+#include <vte/reaper.h>
 #include <pwd.h>
 #include <gtk/gtk.h>
 #include <libanjuta/anjuta-plugin.h>
 
-extern char **environ;
-
 extern GType terminal_plugin_get_type (GTypeModule *module);
 #define ANJUTA_PLUGIN_TERMINAL_TYPE         (terminal_plugin_get_type (NULL))
 #define ANJUTA_PLUGIN_TERMINAL(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), ANJUTA_PLUGIN_TERMINAL_TYPE, TerminalPlugin))
@@ -80,7 +78,9 @@ typedef struct _TerminalPluginClass TerminalPluginClass;
 struct _TerminalPlugin{
 	AnjutaPlugin parent;
 	
-	AnjutaUI *ui;
+	gint uiid;
+	GtkActionGroup *action_group;	
+	
 	AnjutaPreferences *prefs;
 	pid_t child_pid;
 	GtkWidget *term;
@@ -89,8 +89,8 @@ struct _TerminalPlugin{
 	GtkWidget *scrollbar;
 	GtkWidget *pref_profile_combo;
 	GtkWidget *pref_default_button;
+	gboolean   widget_added_to_shell;
 	GList *gconf_notify_ids;
-	gboolean child_initizlized;
 #if OLD_VTE == 1
 	gboolean first_time_realization;
 #endif
@@ -134,11 +134,12 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	char *text;
 	int value;
 	gboolean setting;
-	GdkColor color;
+	GdkColor color[2];
+	GdkColor* foreground;
+	GdkColor* background;
 	GtkWidget *vte;
 	gchar *profile;
 	AnjutaPreferences *pref;
-	GSList *profiles;
 	
 	pref = term->prefs;
 	vte = term->term;
@@ -147,43 +148,20 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	g_return_if_fail (client != NULL);
 	
 	/* Update the currently available list of terminal profiles */
-	profiles = gconf_client_get_list (client, GCONF_PROFILE_LIST,
-											  GCONF_VALUE_STRING, NULL);
-	if (profiles)
-	{
-		if (term->pref_profile_combo)
-		{
-			GList *list = NULL;
-			GSList *node = profiles;
-			while (node)
-			{
-				if (node->data)
-					list = g_list_append (list, node->data);
-				node = g_slist_next (node);
-			}
-			gtk_combo_set_popdown_strings (GTK_COMBO (term->pref_profile_combo),
-										   list);
-			g_list_free (list);
-		}
-		g_slist_foreach (profiles, (GFunc)g_free, NULL);
-		g_slist_free (profiles);
-	}
 	setting = anjuta_preferences_get_int (pref,
 										  PREFS_TERMINAL_PROFILE_USE_DEFAULT);
 	if (setting)
 	{
 		/* Use the currently selected profile in gnome-terminal */
 		text = gconf_client_get_string (client, GCONF_DEFAULT_PROFILE, NULL);
-		if (!text)
-			text = g_strdup ("Default");
 	}
 	else
 	{
 		/* Otherwise use the user selected profile */
 		text = anjuta_preferences_get (pref, PREFS_TERMINAL_PROFILE);
-		if (!text)
-			text = g_strdup ("Default");
 	}
+	if (!text || (*text == '\0')) 
+			text = g_strdup ("Default");
 	profile = text;
 	
 	vte_terminal_set_mouse_autohide (VTE_TERMINAL (vte), TRUE);
@@ -197,8 +175,7 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	} else {
 		text = GET_PROFILE_STRING (GCONF_VTE_TERMINAL_FONT);
 	}
-	if (text && GTK_WIDGET (vte)->window)
-		vte_terminal_set_font_from_string (VTE_TERMINAL (vte), text);
+	vte_terminal_set_font_from_string (VTE_TERMINAL (vte), text);
 	g_free (text);
 	
 	setting = GET_PROFILE_BOOL (GCONF_CURSOR_BLINK);
@@ -254,18 +231,21 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	text = GET_PROFILE_STRING (GCONF_BACKGROUND_COLOR);
 	if (text)
 	{
-		gdk_color_parse (text, &color);
-		vte_terminal_set_color_background (VTE_TERMINAL (vte), &color);
+		gdk_color_parse (text, &color[0]);
 		g_free (text);
 	}
+	background = text ? &color[0] : NULL;
 	text = GET_PROFILE_STRING (GCONF_FOREGROUND_COLOR);
 	if (text)
 	{
-		gdk_color_parse (text, &color);
-		vte_terminal_set_color_foreground (VTE_TERMINAL (vte), &color);
-		vte_terminal_set_color_bold (VTE_TERMINAL (vte), &color);
+		gdk_color_parse (text, &color[1]);
 		g_free (text);
 	}
+	foreground = text ? &color[1] : NULL;
+	/* vte_terminal_set_colors works even if the terminal widget is not realized
+	 * which is not the case with vte_terminal_set_color_foreground and
+	 * vte_terminal_set_color_background */
+	vte_terminal_set_colors (VTE_TERMINAL (vte), foreground, background, NULL, 0);
 	g_free (profile);
 	g_object_unref (client);
 }
@@ -316,47 +296,17 @@ use_default_profile_cb (GtkToggleButton *button,
 		gtk_widget_set_sensitive (term->pref_profile_combo, TRUE);
 }
 
-static char **
-get_child_environment (void)
+static void
+terminal_child_exited_cb (VteReaper *vtereaper, gint pid, gint status, TerminalPlugin *term_plugin)
 {
-	/* code from gnome-terminal, sort of. */
-	char **p;
-	int i;
-	char **retval;
-#define EXTRA_ENV_VARS 6
-
-	/* count env vars that are set */
-	for (p = environ; *p; p++);
-
-	i = p - environ;
-	retval = g_new (char *, i + 1 + EXTRA_ENV_VARS);
-
-	for (i = 0, p = environ; *p; p++) {
-		/* Strip all these out, we'll replace some of them */
-		if ((strncmp (*p, "COLUMNS=", 8) == 0) ||
-		    (strncmp (*p, "LINES=", 6) == 0)   ||
-		    (strncmp (*p, "TERM=", 5) == 0)    ||
-		    (strncmp (*p, "GNOME_DESKTOP_ICON=", 19) == 0)) {
-			/* nothing: do not copy */
-		} else {
-			retval[i] = g_strdup (*p);
-			++i;
-		}
-	}
-
-	retval[i] = g_strdup ("TERM=xterm"); /* FIXME configurable later? */
-	++i;
-
-	retval[i] = NULL;
-
-	return retval;
+	g_signal_emit_by_name(term_plugin, "child-exited", pid, status);	
 }
 
 static pid_t
 terminal_execute (TerminalPlugin *term_plugin, const gchar *directory,
-				  const gchar *command)
+				  const gchar *command, gchar **environment)
 {
-	char **env, **args, **args_ptr;
+	char **args, **args_ptr;
 	GList *args_list, *args_list_ptr;
 	gchar *dir;
 	VteTerminal *term;
@@ -385,23 +335,18 @@ terminal_execute (TerminalPlugin *term_plugin, const gchar *directory,
 	
 	vte_terminal_reset (term, TRUE, TRUE);
 
-	env = get_child_environment ();
-	
 	term_plugin->child_pid = vte_terminal_fork_command (term, args[0], args,
-														env, dir, 0, 0, 0);
+														environment, dir, 0, 0, 0);
+	vte_reaper_add_child (term_plugin->child_pid);
+	
 	g_free (dir);
-	g_strfreev (env);
 	g_free (args);
 	g_list_foreach (args_list, (GFunc)g_free, NULL);
 	g_list_free (args_list);
 	
-	/* The fork command above overwirtes our SIGCHLD signal handler.
-	 * Restore it */
-	anjuta_children_recover ();
-	preferences_changed (term_plugin->prefs, term_plugin);
-	
-	anjuta_shell_present_widget (ANJUTA_PLUGIN (term_plugin)->shell,
-								 term_plugin->frame, NULL);
+	if (term_plugin->widget_added_to_shell)
+		anjuta_shell_present_widget (ANJUTA_PLUGIN (term_plugin)->shell,
+									 term_plugin->frame, NULL);
 	
 	return term_plugin->child_pid;
 }
@@ -421,20 +366,16 @@ terminal_init_cb (GtkWidget *widget, TerminalPlugin *term_plugin)
 		shell = "/bin/sh";
 		dir = "/";
 	}
-	terminal_execute (term_plugin, dir, shell);
+	terminal_execute (term_plugin, dir, shell, NULL);
 }
 
 static gboolean
 terminal_focus_cb (GtkWidget *widget, GdkEvent  *event,
 				   TerminalPlugin *term) 
 {
-	if (term->child_pid > 0)
-		term->child_initizlized = TRUE;
-	
-	if (term->child_initizlized == FALSE)
+	if (term->child_pid == 0)
 	{
 		terminal_init_cb (widget, term);
-		term->child_initizlized = TRUE;
 	}
 	gtk_widget_grab_focus (widget);
 	return FALSE;
@@ -471,6 +412,28 @@ terminal_keypress_cb (GtkWidget *widget, GdkEventKey  *event,
 	return FALSE;
 }
 
+static gboolean
+terminal_click_cb (GtkWidget *widget, GdkEventButton *event,
+					  TerminalPlugin *term) 
+{
+	if (event->button == 3)
+	{
+		AnjutaUI *ui;
+		GtkMenu *popup;
+		GtkAction *action;
+
+		ui = anjuta_shell_get_ui (ANJUTA_PLUGIN(term)->shell, NULL);
+		popup =  GTK_MENU (gtk_ui_manager_get_widget (GTK_UI_MANAGER (ui), "/PopupTerminal"));
+		action = gtk_action_group_get_action (term->action_group, "ActionCopyFromTerminal");
+		gtk_action_set_sensitive (action,vte_terminal_get_has_selection(VTE_TERMINAL(term->term)));
+		
+		gtk_menu_popup (popup, NULL, NULL, NULL, NULL,
+						event->button, event->time);
+	}
+	
+	return FALSE;
+}
+
 #if OLD_VTE == 1
 /* VTE has a terrible bug where it could crash when container is changed.
  * The problem has been traced in vte where its style-set handler does not
@@ -523,6 +486,38 @@ terminal_destroy_cb (GtkWidget *widget, TerminalPlugin *term)
 							G_CALLBACK (terminal_init_cb), term);
 }
 
+static void
+on_terminal_copy_cb (GtkAction * action, TerminalPlugin *term)
+{
+	if (vte_terminal_get_has_selection(VTE_TERMINAL(term->term)))
+		vte_terminal_copy_clipboard(VTE_TERMINAL(term->term));
+}
+
+static void
+on_terminal_paste_cb (GtkAction * action, TerminalPlugin *term)
+{
+	vte_terminal_paste_clipboard(VTE_TERMINAL(term->term));
+}
+
+static GtkActionEntry actions_terminal[] = {
+	{
+		"ActionCopyFromTerminal", 	              /* Action name */
+		GTK_STOCK_COPY,                           /* Stock icon, if any */
+		N_("_Copy"),		                      /* Display label */
+		NULL,                                     /* short-cut */
+		NULL,                                     /* Tooltip */
+		G_CALLBACK (on_terminal_copy_cb)          /* action callback */
+	},
+	{
+		"ActionPasteInTerminal",
+		GTK_STOCK_PASTE,
+		N_("_Paste"),
+		NULL,
+		NULL,
+		G_CALLBACK (on_terminal_paste_cb)
+	}
+};
+
 static void
 terminal_create (TerminalPlugin *term_plugin)
 {
@@ -543,14 +538,19 @@ terminal_create (TerminalPlugin *term_plugin)
 					  G_CALLBACK (terminal_init_cb), term_plugin);
 	g_signal_connect (G_OBJECT (term_plugin->term), "destroy",
 					  G_CALLBACK (terminal_destroy_cb), term_plugin);
-	g_signal_connect (G_OBJECT (term_plugin->term), "event",
+	g_signal_connect (G_OBJECT (term_plugin->term), "key-press-event",
 					  G_CALLBACK (terminal_keypress_cb), term_plugin);
+	g_signal_connect (G_OBJECT (term_plugin->term), "button-press-event",
+					  G_CALLBACK (terminal_click_cb), term_plugin);
 #if OLD_VTE == 1
 	g_signal_connect (G_OBJECT (term_plugin->term), "realize",
 					  G_CALLBACK (terminal_realize_cb), term_plugin);
 	g_signal_connect (G_OBJECT (term_plugin->term), "unrealize",
 					  G_CALLBACK (terminal_unrealize_cb), term_plugin);
 #endif
+	g_signal_connect (vte_reaper_get(), "child-exited",
+					  G_CALLBACK (terminal_child_exited_cb), term_plugin);
+	
 	sb = gtk_vscrollbar_new (GTK_ADJUSTMENT (VTE_TERMINAL (term_plugin->term)->adjustment));
 	GTK_WIDGET_UNSET_FLAGS (sb, GTK_CAN_FOCUS);
 	
@@ -567,6 +567,8 @@ terminal_create (TerminalPlugin *term_plugin)
 	term_plugin->scrollbar = sb;
 	term_plugin->frame = frame;
 	term_plugin->hbox = hbox;
+
+	terminal_init_cb (GTK_WIDGET (term_plugin->term), term_plugin);
 }
 
 static void
@@ -588,12 +590,21 @@ activate_plugin (AnjutaPlugin *plugin)
 {
 	TerminalPlugin *term_plugin;
 	static gboolean initialized = FALSE;
+	AnjutaUI *ui;
 	
 	DEBUG_PRINT ("TerminalPlugin: Activating Terminal plugin ...");
 	
 	term_plugin = ANJUTA_PLUGIN_TERMINAL (plugin);
-	term_plugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	term_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
+	term_plugin->widget_added_to_shell = FALSE;
+	ui = anjuta_shell_get_ui (plugin->shell, NULL);
+	term_plugin->action_group = anjuta_ui_add_action_group_entries (ui,
+										"ActionGroupTerminal",
+										_("terminal operations"),
+										actions_terminal,
+										G_N_ELEMENTS (actions_terminal),
+										GETTEXT_PACKAGE, TRUE, term_plugin);
+	term_plugin->uiid = anjuta_ui_merge (ui, UI_FILE);
 	
 	terminal_create (term_plugin);
 	
@@ -611,8 +622,14 @@ activate_plugin (AnjutaPlugin *plugin)
 							 "terminal-plugin-icon",
 							 ANJUTA_SHELL_PLACEMENT_BOTTOM, NULL);
 	/* terminal_focus_cb (term_plugin->term, NULL, term_plugin); */
-	
+	term_plugin->widget_added_to_shell = TRUE;
 	initialized = TRUE;
+
+	/* Set all terminal preferences, at that time the terminal widget is
+	 * not realized, a few vte functions are not working. Another
+	 * possibility could be to call this when the widget is realized */
+	preferences_changed (term_plugin->prefs, term_plugin);
+	
 	return TRUE;
 }
 
@@ -620,7 +637,17 @@ static gboolean
 deactivate_plugin (AnjutaPlugin *plugin)
 {
 	TerminalPlugin *term_plugin;
+	AnjutaUI *ui;	
+	
 	term_plugin = ANJUTA_PLUGIN_TERMINAL (plugin);
+
+	ui = anjuta_shell_get_ui (plugin->shell, NULL);
+	anjuta_ui_unmerge (ui, term_plugin->uiid);	
+	if (term_plugin->action_group)
+	{
+		anjuta_ui_remove_action_group (ui, term_plugin->action_group);
+		term_plugin->action_group = NULL;
+	}
 	
 	prefs_finalize (term_plugin);
 
@@ -641,7 +668,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	term_plugin->term = NULL;
 	term_plugin->scrollbar = NULL;
 	term_plugin->hbox = NULL;
-	term_plugin->child_initizlized = FALSE;
+	term_plugin->child_pid = 0;
 #if OLD_VTE == 1
 	term_plugin->first_time_realization = TRUE;
 #endif
@@ -653,13 +680,13 @@ deactivate_plugin (AnjutaPlugin *plugin)
 static void
 terminal_plugin_dispose (GObject *obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
 terminal_plugin_finalize (GObject *obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -667,8 +694,10 @@ terminal_plugin_instance_init (GObject *obj)
 {
 	TerminalPlugin *plugin = ANJUTA_PLUGIN_TERMINAL (obj);
 	plugin->gconf_notify_ids = NULL;
-	plugin->child_initizlized = FALSE;
+	plugin->child_pid = 0;
 	plugin->pref_profile_combo = NULL;
+	plugin->uiid = 0;
+	plugin->action_group = NULL;
 #if OLD_VTE == 1
 	plugin->first_time_realization = TRUE;
 #endif
@@ -690,7 +719,8 @@ terminal_plugin_class_init (GObjectClass *klass)
 static pid_t
 iterminal_execute_command (IAnjutaTerminal *terminal,
 						   const gchar *directory,
-						   const gchar *command, GError **err)
+						   const gchar *command,
+						   gchar **environment, GError **err)
 {
 	TerminalPlugin *plugin;
 	const gchar *dir;
@@ -702,7 +732,7 @@ iterminal_execute_command (IAnjutaTerminal *terminal,
 	else
 		dir = directory;
 	
-	return terminal_execute (plugin, directory, command);
+	return terminal_execute (plugin, directory, command, environment);
 }
 
 static void
@@ -711,20 +741,75 @@ iterminal_iface_init(IAnjutaTerminalIface *iface)
 	iface->execute_command = iterminal_execute_command;
 }
 
+static void
+on_add_string_in_store (gpointer data, gpointer user_data)
+{
+	GtkListStore* model = (GtkListStore *)user_data;
+	GtkTreeIter iter;
+	
+	gtk_list_store_append (model, &iter);
+	gtk_list_store_set (model, &iter, 0, (const gchar *)data, -1);
+}
+
+static void
+on_concat_string (gpointer data, gpointer user_data)
+{
+	GString* str = (GString *)user_data;	
+	
+	if (str->len != 0)
+		g_string_append_c (str, ',');
+	g_string_append (str, data);
+}
+
 static void
 ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
+	GSList *profiles;
+	GConfClient *client;	
+	GString *default_value;
+	
 	/* Create the terminal preferences page */
 	TerminalPlugin* term_plugin = ANJUTA_PLUGIN_TERMINAL (ipref);
 	GladeXML *gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog_terminal", NULL);
 	anjuta_preferences_add_page (term_plugin->prefs, gxml,
 									"Terminal", _("Terminal"), ICON_FILE);
+	
 	term_plugin->pref_profile_combo = glade_xml_get_widget (gxml, "profile_list_combo");
+
+	/* Update the currently available list of terminal profiles */
+	client = gconf_client_get_default ();
+	profiles = gconf_client_get_list (client, GCONF_PROFILE_LIST,
+									  GCONF_VALUE_STRING, NULL);
+	default_value = g_string_new (NULL);
+	if (profiles)
+	{
+		GtkListStore *store;
+			
+		store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (term_plugin->pref_profile_combo)));
+		
+		gtk_list_store_clear (store);
+		g_slist_foreach (profiles, on_add_string_in_store, store);
+		g_slist_foreach (profiles, on_concat_string, default_value);
+		g_slist_foreach (profiles, (GFunc)g_free, NULL);
+		g_slist_free (profiles);
+		
+	}
+	anjuta_preferences_register_property_raw (term_plugin->prefs,
+											  term_plugin->pref_profile_combo,
+											  PREFS_TERMINAL_PROFILE,
+											  default_value->str,
+											  1,
+											  ANJUTA_PROPERTY_OBJECT_TYPE_COMBO,
+											  ANJUTA_PROPERTY_DATA_TYPE_TEXT);
+	g_string_free (default_value, TRUE);
+		
 	term_plugin->pref_default_button =
 		glade_xml_get_widget (gxml,
 						"preferences_toggle:bool:1:0:terminal.default.profile");
+	use_default_profile_cb (GTK_TOGGLE_BUTTON (term_plugin->pref_default_button), term_plugin);
 	g_signal_connect (G_OBJECT(term_plugin->pref_default_button), "toggled",
 						  G_CALLBACK (use_default_profile_cb), term_plugin);
+	
 	g_object_unref (gxml);
 }
 
